#!/usr/bin/perl -I perl

# $Id$

=pod
=head1 Glintfreq.pl

Glintfreq.p - This program runs Primer3 and parses the output File

=head1  SYNOPSIS

glintfreqFile

=head1 DESCRIPTION

This program create a BoulderIO file from a multifasta file,
then runs Primer 3 using a system commandline
and finally it parses the outputfile and return all the informations 
on the bunch of primers

=cut

=head1 SUBROUTINES

IsUnique
IsWellPositionned

PushAmplicon
ExtractProductPcrSequence

=cut

package Probe;

use strict;
use warnings;

use Data::Dumper;
use base qw( LipmObject);

use Region;
use SuitableRegion;

############################################################
###                   CONSTRUCTEURS                      ###
############################################################

=head2 function _Init

 Title    : _Init
 Usage    : The constructor
 Access   : Private
 Function : Set all arguments
            Push the new probe in the probe array of the suitable subregion
 Args     : $id          => the probe id
            $leftPrimer  => the left primer sequence
            $rightPrimer => the right primer sequence
            $offset      => the probe offset
            $size        => the probe size
            $suitableId  => the suitable sub region iD fitted by the probe
            $ro_region   => the reference of the object region
                                                              
            $pair_penalty        => the score of both primer
            $primer_left_length  => length of the left primer
            $primer_right_start  => the start position of the right primer
            $primer_right_length => length of the right primer
            $primer_left_tm      => the melting temperature of the left primer
            $primer_left_gc      => the GC percent of the left primer
            $primer_right_gc     => the GC percent of the right primer
 Returns  : The object
 Globals  : none

=cut
sub _Init
{
	my ( $self, $id, $leftPrimer, $rightPrimer,  $offset, $size, $suitableId, $from, 
	$pair_penalty, $primer_left_length, $primer_right_start, $primer_right_length, 
	$primer_left_tm, $primer_right_tm, $primer_left_gc, $primer_right_gc) = @_ ;
	
	my $chr = undef;
	$self ->_Set ( '__chr', $chr );
	
	$self ->_Set ( '__id', $id );
	$self ->_Set ( '__pair_penalty', $pair_penalty );
	$self ->_Set ( '__leftPrimer', $leftPrimer );
	$self ->_Set ( '__rightPrimer', $rightPrimer );
	$self ->_Set ( '__offset', $offset );
	$self ->_Set ( '__primer_left_length', $primer_left_length );
	$self ->_Set ( '__primer_right_start', $primer_right_start );
	$self ->_Set ( '__primer_right_length', $primer_right_length );
	$self ->_Set ( '__primer_left_tm', $primer_left_tm );
	$self ->_Set ( '__primer_right_tm', $primer_right_tm );
	$self ->_Set ( '__primer_left_gc', $primer_left_gc );
	$self ->_Set ( '__primer_right_gc', $primer_right_gc );
	$self ->_Set ( '__size', $size );
	$self ->_Set ( '__suitableId', $suitableId );
	$self ->_Set ( '__from', $from );
	
	my $productPcrSeq = undef;
	$self ->_Set ( '__productPcrSeq', $productPcrSeq );
	
	if ( defined $id )
	{
		my $end = $from + $size ;
		$self->_Set ( '__end', $end );
	}
	my $isValid = 0;
	my $validProbeId = undef;
	$self ->_Set ( '__isValid', $isValid );
	$self ->_Set ( '__validProbeId', $validProbeId );
	
	my $glintVS = undef;
	$self->_Set( '__glintVS', $glintVS );
	
	my $rePcrVS = undef;
	$self->_Set( '__rePcrVs', $rePcrVS );
	
	my $overlap = 0;
	$self ->_Set ( '__overlap', $overlap );
	
	return $self;
}

############################################################
###                   GETTERS/SETTERS                    ###
############################################################

# GETTERS
=head2 function Chr

 Title        : Chr
 Usage        : $chr = $probes->Chr();
 Function     : Accessor, returns the sequence chromosome
 Access       : Public
 Returns      : $chr
 Args         : none

=cut
sub Chr
{
	my $self = shift;
	return $self->_Get('__chr');
}

=head2 function Id

 Title        : Id
 Usage        : $id = $probes->Id();
 Function     : Accessor, returns the sequence id
 Access       : Public
 Returns      : $id
 Args         : none

=cut
sub Id
{
	my $self = shift;
	return $self->_Get('__id');
}

=head2 function ValidProbeId

 Title        : ValidProbeId
 Usage        : $validProbeId = $probes->ValidProbeId();
 Function     : Accessor, returns the valid probe id
 Access       : Public
 Returns      : $validProbeId
 Args         : none

=cut
sub ValidProbeId
{
	my $self = shift;
	return $self->_Get('__validProbeId');
}

=head2 function LeftPrimer

 Title        : LeftPrimer
 Usage        : $leftPrimer = $probes->LeftPrimer();
 Function     : Accessor, returns the left primer sequence
 Access       : Public
 Returns      : $leftPrimer
 Args         : none

=cut
sub LeftPrimer
{
	my $self = shift;
	return $self->_Get('__leftPrimer');
}

=head2 function RightPrimer

 Title        : RightPrimer
 Usage        : $rightPrimer = $probes->rightPrimer();
 Function     : Accessor, returns the right primer sequence
 Access       : Public
 Returns      : $rightPrimer
 Args         : none

=cut
sub RightPrimer
{
	my $self = shift;
	return $self->_Get('__rightPrimer');
}

=head2 function Offset

 Title        : Offset
 Usage        : $offset = $probes->offset();
 Function     : Accessor, returns the probe offset
 Access       : Public
 Returns      : $offset
 Args         : none

=cut
sub Offset
{
	my $self = shift;
	return $self->_Get('__offset');
}

=head2 function Size

 Title        : Size
 Usage        : $size = $probes->size();
 Function     : Accessor, returns the probe size
 Access       : Public
 Returns      : $size
 Args         : none

=cut
sub Size
{
	my $self = shift;
	return $self->_Get('__size');
}

=head2 function From

 Title        : From
 Usage        : $from = $probes->From();
 Function     : Accessor, returns the probe start position in the region
 Access       : Public
 Returns      : $from
 Args         : none

=cut
sub From
{
	my $self = shift;
	return $self->_Get('__from');
}

=head2 function End

 Title        : End
 Usage        : $end = $probes->End();
 Function     : Accessor, returns the probe end position in the region
 Access       : Public
 Returns      : $end
 Args         : none

=cut
sub End
{
	my $self = shift;
	return $self->_Get('__end');
}

=head2 function Pair_Penalty

 Title        : Pair_Penalty
 Usage        : $pairPenalty = $probes->Pair_Penalty();
 Function     : Accessor, returns the probe pair penalty
 Access       : Public
 Returns      : $pairPenalty
 Args         : none

=cut
sub Pair_Penalty
{
	my $self = shift;
	return $self->_Get('__pair_penalty');
}

=head2 function SuitableId

 Title        : SuitableId
 Usage        : $suitableId = $probes->SuitableId();
 Function     : Accessor, returns the suitable subregion iD
 Access       : Public
 Returns      : $suitableId
 Args         : none

=cut
sub SuitableId
{
	my $self = shift;
	return $self->_Get('__suitableId');
}

=head2 function PrimerLeftTm

 Title        : PrimerLeftTm
 Usage        : $primerLeftTm = $probes->PrimerLeftTm();
 Function     : Accessor, returns the left primer Tm
 Access       : Public
 Returns      : $primerLeftTm
 Args         : none

=cut
sub PrimerLeftTm
{
	my $self = shift;
	return $self->_Get('__primer_left_tm');
}

=head2 function PrimerRightTm

 Title        : PrimerRightTm
 Usage        : $primerRightTm = $probes->PrimerRightTm();
 Function     : Accessor, returns the right primer Tm
 Access       : Public
 Returns      : $primerRightTm
 Args         : none

=cut
sub PrimerRightTm
{
	my $self = shift;
	return $self->_Get('__primer_right_tm');
}

=head2 function PrimerRightGc

 Title        : PrimerRightGc
 Usage        : $primerRightGc = $probes->PrimerRightGc();
 Function     : Accessor, returns the right primer Gc
 Access       : Public
 Returns      : $primerRightGc
 Args         : none

=cut
sub PrimerRightGc
{
	my $self = shift;
	return $self->_Get('__primer_right_gc');
}

=head2 function PrimerLeftGc

 Title        : PrimerLeftGc
 Usage        : $primerLeftgc = $probes->PrimerLeftGc();
 Function     : Accessor, returns the left primer Gc
 Access       : Public
 Returns      : $primerLeftGc
 Args         : none

=cut
sub PrimerLeftGc
{
	my $self = shift;
	return $self->_Get('__primer_left_gc');
}

=head2 function ProductPcrSeq

 Title        : ProductPcrSeq
 Usage        : $productPcrSeq = $probes->ProductPcrSeq();
 Function     : Accessor, returns the productPcrSeq
 Access       : Public
 Returns      : $productPcrSeq
 Args         : none

=cut
sub ProductPcrSeq
{
	my $self = shift;
	return $self->_Get('__productPcrSeq');
}

=head2 function SetId

 Title        : SetId
 Usage        : $probes->SetId( $id );
 Access       : Public
 Returns      : none
 Args         : $id

=cut
sub SetId
{
	my ( $self, $id ) = @_;
	$self->_Set('__id', $id);
}

=head2 function SetChr

 Title        : SetChr
 Usage        : $probes->SetChr( $chr );
 Access       : Public
 Returns      : none
 Args         : $chr

=cut
sub SetChr
{
	my ( $self, $chr ) = @_;
	$self->_Set('__chr', $chr);
}

=head2 function SetFrom

 Title        : SetFrom
 Usage        : $probes->SetFrom( $from );
 Access       : Public
 Returns      : none
 Args         : $from

=cut
sub SetFrom
{
	my ( $self, $from ) = @_;
	$self->_Set('__from', $from);
}

=head2 function SetEnd

 Title        : SetEnd
 Usage        : $probes->SetEnd( $end );
 Access       : Public
 Returns      : none
 Args         : $end

=cut
sub SetEnd
{
	my ( $self, $end ) = @_;
	$self->_Set('__end', $end);
}

=head2 function SetSize

 Title        : SetSize
 Usage        : $probes->SetSize( $size );
 Access       : Public
 Returns      : none
 Args         : $size

=cut
sub SetSize
{
	my ( $self, $size ) = @_;
	$self->_Set('__size', $size);
}

=head2 function SetProductPcrSeq

 Title        : SetProductPcrSeq
 Usage        : $probes->SetProductPcrSeq( $productPcrSeq );
 Access       : Public
 Returns      : none
 Args         : $productPcrSeq

=cut
sub SetProductPcrSeq
{
	my ( $self, $productPcrSeq ) = @_;
	return $self->_Set('__productPcrSeq', $productPcrSeq);
}

=head2 function SetValidProbeId

 Title        : SetValidPobeId
 Usage        : $probes->SetValidProbeId( $validProbeId );
 Access       : Public
 Returns      : none
 Args         : $validProbeId

=cut
sub SetValidProbeId
{
	my ( $self, $validProbeId ) = @_;
	return $self->_Set('__validProbeId', $validProbeId);
}

=head2 function IsValid

 Title        : IsValid
 Usage        : $validity = $probes->IsValid();
 Function     : Accessor, returns the probe validity
 Access       : Public
 Returns      : $validity
 Args         : none

=cut
sub IsValid
{
	my $self = shift;
	return $self->_Get('__isValid');
}

=head2 function Overlap

 Title        : Overlap
 Usage        : $overlap = $probes->Overlap();
 Function     : Accessor, returns 1 if the probe overlap another one
 Access       : Public
 Returns      : $overlap
 Args         : none

=cut
sub Overlap
{
	my $self = shift;
	return $self->_Get('__overlap');
}

=head2 function RePcrVS

 Title        : RePcrVS
 Usage        : $rePcrVS = $probes->RePcrVS();
 Function     : Accessor, returns 1 if the probe has more than one rePcr amplicon
 Access       : Public
 Returns      : $rePcrVS
 Args         : none

=cut
sub RePcrVS
{
	my $self = shift;
	return $self->_Get('__rePcrVs');
}

=head2 function GlintVS

 Title        : GlintVS
 Usage        : $glintVS = $probes->GlintVS();
 Function     : Accessor, returns 1 if the probe has more than one glint amplicon 
 Access       : Public
 Returns      : $qlintVS
 Args         : none

=cut
sub GlintVS
{
	my $self = shift;
	return $self->_Get('__glintVS');
}

=head2 function IsWellPositionned

 Title        : IsTomTom
 Usage        : $isTomTom = $probes->IsTomTom();
 Function     : Accessor, returns 1 if the unique Amplicon is at the expected position
 Returns      : $isTomTom
 Args         : none

=cut
sub IsWellPositionned
{
	my ( $self, $amplicon_from ) = @_;
	my $probePos = $self->From();
	if ( $amplicon_from == $probePos) 
		{ return 1; }
}



# SETTINGS
=head2 Procedure SetPair_Penalty

 Title        : SetPair_Penalty
 Usage        : $probe->SetPair_Penalty( $pair_penalty );
 Prerequisite : none
 Procedure    : none
 Args         : $pair_penalty

=cut
sub SetPair_Penalty
{
	my ( $self, $pair_penalty ) = @_ ;
	$self ->_Set ( '__pair_penalty', $pair_penalty );
}

=head2 Procedure SetOffset

 Title        : SetOffset
 Usage        : $probe->SetOffset( $offset );
 Prerequisite : none
 Procedure    : none
 Args         : $offset

=cut
sub SetOffset
{
	my ( $self, $offset ) = @_ ;
	$self ->_Set ( '__offset', $offset );
}

=head2 Procedure SetPrimer_Left_Length

 Title        : SetPrimer_Left_Length
 Usage        : $probe->SetPrimer_Left_Length( $primer_left_length );
 Prerequisite : none
 Procedure    : none
 Args         : $primer_left_length

=cut
sub SetPrimer_Left_Length
{
	my ( $self, $primer_left_length ) = @_ ;
	$self ->_Set ( '__primer_left_length', $primer_left_length );
}

=head2 Procedure SetPrimer_Right_Start

 Title        : SetPrimer_Right_Start
 Usage        : $probe->SetPrimer_Right_Start( $primer_right_start );
 Prerequisite : none
 Procedure    : none
 Args         : $primer_right_start

=cut
sub SetPrimer_Right_Start
{
	my ( $self, $primer_right_start ) = @_ ;
	$self ->_Set ( '__primer_right_start', $primer_right_start );
}

=head2 Procedure SetPrimer_Right_Length

 Title        : SetPrimer_Right_Length
 Usage        : $probe->SetPrimer_Right_Length( $primer_right_length );
 Prerequisite : none
 Procedure    : none
 Args         : $primer_right_length

=cut
sub SetPrimer_Right_Length
{
	my ($self, $primer_right_length ) = @_ ;
	$self ->_Set ( '__primer_right_length', $primer_right_length );
}

=head2 Procedure _SetPrimer_Right_Tm

 Title        : _SetPrimer_Right_Tm
 Usage        : $probe->_SetPrimer_Right_Tm( $primer_right_tm );
 Prerequisite : none
 Procedure    : none
 Args         : $primer_right_tm

=cut
sub _SetPrimer_Right_Tm
{
	my ( $self, $primer_right_tm ) = @_ ;
	$self ->_Set ( '__primer_right_tm', $primer_right_tm );
}

=head2 Procedure _SetPrimer_Left_Tm

 Title        : _SetPrimer_Left_Tm
 Usage        : $probe->_SetPrimer_Left_Tm( $primer_left_tm );
 Prerequisite : none
 Procedure    : none
 Args         : $primer_left_tm

=cut
sub _SetPrimer_Left_Tm
{
	my ( $self, $primer_left_tm ) = @_ ;
	$self ->_Set ( '__primer_left_tm', $primer_left_tm );
}

=head2 Procedure _SetPrimer_Right_Gc

 Title        : _SetPrimer_Right_Gc
 Usage        : $probe->_SetPrimer_Right_Gc( $primer_right_gc );
 Prerequisite : none
 Procedure    : none
 Args         : $primer_right_gc

=cut
sub _SetPrimer_Right_Gc
{
	my ( $self, $primer_right_gc ) = @_ ;
	$self ->_Set ( '__primer_right_gc', $primer_right_gc );
}

=head2 Procedure _SetPrimer_Left_Gc

 Title        : _SetPrimer_Left_Gc
 Usage        : $probe->_SetPrimer_Left_Gc( $primer_left_gc );
 Prerequisite : none
 Procedure    : none
 Args         : $primer_left_gc

=cut
sub _SetPrimer_Left_Gc
{
	my ( $self, $primer_left_gc ) = @_ ;
	$self ->_Set ( '__primer_left_gc', $primer_left_gc );
}

=head2 Procedure _SetProduct_Size

 Title        : _SetProduct_Size
 Usage        : $probe->_SetProduct_Size( $size );
 Prerequisite : none
 Procedure    : none
 Args         : $size

=cut
sub _SetProduct_Size
{
	my ( $self, $size ) = @_ ;
	$self ->_Set ( '__size', $size );
}

=head2 Procedure SetOverlap

 Title        : SetOverlap
 Usage        : $probe->SetOverlap($overlap);
 Function     : set a value overlap at true
 Prerequisite : none
 Procedure    : none
 Args         : 1

=cut
sub SetOverlap
{
	my ( $self, $overlap ) = @_;
	$self ->_Set ( '__overlap', $overlap );
}

=head2 Procedure SetValid

 Title        : SetValid
 Usage        : $probe->_SetValid($valid);
 Function     : set a value IsValid at true
 Prerequisite : none
 Procedure    : none
 Args         : none

=cut
sub SetValid
{
	my ( $self, $isValid ) = @_;
	$self ->_Set ( '__isValid', $isValid );
}

=head2 Procedure SetGlintVS

 Title        : SetGlintVS
 Usage        : $probe->SetGlintVS();
 Function     : set a value GlintVS at true
 Prerequisite : none
 Procedure    : none
 Args         : none

=cut
sub SetGlintVS
{
	my ( $self ) = @_;
	$self ->_Set ( '__glintVS', 1 );
}

=head2 Procedure SetRePcrVS

 Title        : SetRePcrVS
 Usage        : $probe->SetRePcrVS();
 Function     : set a value RePcrVS at true
 Prerequisite : none
 Procedure    : none
 Args         : none

=cut
sub SetRePcrVS
{
	my ( $self ) = @_;
	$self ->_Set ( '__rePcrVs', 1 );
}

############################################################
###                     FUNCTION                         ###
############################################################

=head2 function PushAmplicon

 Title        : PushAmplicon
 Usage        : $probe->PushAmplicon($Amplicon);
 Function     : Push a Amplicon in the Amplicon array of the probe object
 Access       : Public
 Returns      : none
 Args         : the Amplicon pushed

=cut
sub PushAmplicon
{
	my ( $self, $o_Amplicon ) = @_;
	my $rao_Amplicon = $self->_Get('__a_Amplicon');
	push(@$rao_Amplicon, $o_Amplicon);
}

=head2 function DuplicateProbe

 Title        : DuplicateProbe
 Usage        : $probe->DuplicateProbe;
 Function     : duplicate an object probe and return the copy
 Access       : Public
 Returns      : none
 Args         : the probe copy

=cut
sub DuplicateProbe
{
	my ( $self ) = @_;
	
	my $id = undef;
	my $suitableId = undef;
	my $primer_left_length = undef;
	my $primer_right_start = undef;
	my $primer_right_length = undef;
	
	my $chr             = $self->Chr();
	my $leftPrimer      = $self->LeftPrimer();
	my $rightPrimer     = $self->RightPrimer();
	my $offset          = $self->Offset();
	my $size            = $self->Size();
	my $from            = $self->From();
	my $pair_penalty    = $self->Pair_Penalty();
	my $primer_left_tm  = $self->PrimerLeftTm();
	my $primer_right_tm = $self->PrimerRightTm();
	my $primer_left_gc  = $self->PrimerLeftGc();
	my $primer_right_gc = $self->PrimerRightGc();
	
	my $o_probe = New Probe($id, $leftPrimer, $rightPrimer,  $offset, $size, $suitableId, $from, 
	$pair_penalty, $primer_left_length, $primer_right_start, $primer_right_length, 
	$primer_left_tm, $primer_right_tm, $primer_left_gc, $primer_right_gc);
	
	$o_probe->SetChr( $chr );
	
	return( \$o_probe );
}

=head2 function ExtractPcrProductSequence

 Title        : ExtractProductPcrSequence
 Usage        : $probe->ExtractProductPcrSequence( $file );
 Function     : Using Bioperl library, extract a Pcr product sequence corresponding to the probe and return it as a string
 Access       : Public
 Returns      : pcr product sequence string
 Args         : the source file, the offset and the size of the pcr product sequence

=cut
sub ExtractProductPcrSequence
{
	my ( $self, $file ) = @_;
	
	my $id     = $self->Id();
	my $from   = $self->From();
	my $offset = $self->Offset();
	my $size   = $self->Size();
	my $end    = $self->End();
	
	my $seq = Bio::SeqIO->new( -file => $file , -format => 'fasta' );
	my $seqObj = $seq->next_seq();
	my $subseq = $seqObj->subseq($offset+1,( $offset + $size ) );
	
	return ( $subseq."\n" );
}

1;
