#!/usr/bin/perl -I perl

# $Id$

=pod
=head1 Glintfreq.pl

Glintfreq.p - This program runs Primer3 and parses the output File

=head1  SYNOPSIS

glintfreqFile

=head1 DESCRIPTION

This program create a BoulderIO file from a multifasta file,
then runs Primer 3 using a system commandline
and finally it parses the outputfile and return all the informations 
on the bunch of primers

=cut
=head1 SUBROUTINES

GetAsuitable
GetAProbe
SuitableSize

PushSuitable
PushProbeRef
PushProbes
PushAmplicon

DetectSuitables
ParseFreq
DetectAllPobes
ParsePrimer3
Run
DisplaySTS
checkProbes
GetValid
=cut

package Region;

use strict;
use warnings;

use SuitableRegion;
use Probe;

use Cwd;
use Data::Dumper;
use File::Temp qw/ tempfile tempdir /;
use Term::ANSIColor;
use base qw( LipmObject);

use FindBin qw( $Bin );

use Bio::SeqIO;

use constant TRUE  => 1;
use constant FALSE => 0;

use constant start  => 'start';
use constant end    => 'end';
use constant middle => 'middle';


############################################################
#####                CONSTRUCTEUR                      #####
############################################################

=head2 function _Init

 Title    : _Init
 Usage    : The constructor
 Access   : Private
 Function : Set all arguments and create the probe array
 Args     : $chr     => the chromosome number
            $start   => the start position
            $end     => the end position
            $species => the species
            $ra_suitables => the suitable subregion reference array
 Returns  : The object
 Globals  : none

=cut
sub _Init
{
	my $self = shift;
	
	my %h_args = (	chr        => undef,
					start      => undef,
					end        => undef,
					species    => undef,
					chrom_file => undef,,
					index_file => undef,
					target     => undef,
					param      => undef,
					@_
				 );
	my ( $chr, $start, $end, $species, $chrom_file, $ra_index_file, $target_dir, $rh_param ) = @h_args{ qw( chr start end species chrom_file index_file target param ) };
	my %h_param = %$rh_param ;
	
	my %h_region = ( chr   => $chr,
	                 start => $start,
	                 end   => $end );
	
	$self->_Set( '__chr'  , $chr   );
	$self->_Set( '__start', $start );
	$self->_Set( '__end'  , $end   );
	
	my $size = $end-$start ;
	$self->_Set( '__size'   , $size );
	
	#Target directory issues
	unless ( defined $target_dir )
		{ $target_dir = tempdir( 'e-fish-XXXX', CLEANUP => 0 ); }
	$self->_Set( '__targetDir', $target_dir );
	print "Results available in ";
	print colored ( "$target_dir \n", 'bold cyan' );
	
	#Fasta files
	my @a_index_file = @$ra_index_file;
	my $index_file   = $a_index_file[0];
	$self->_Set( '__genome'      , $index_file );
	$self->_Set( '__chr_filename', $chrom_file );
	
	#Species information
	$self->_Set( '__species', $species );
	
	#Query file rewriting option
	my $query_Filename = $self->SetQueryFileName();
	$self->_Set( '__query_filename', $query_Filename      );
	$self->_Set( '__force'         , $h_param{ FORCE } );
	
	$self->ExtractRegionSequence( $species, \%h_region, $target_dir );
	
	#GlintFreq main arrays
	my @ao_suitables  = ();
	my @a_validProbes = ();
	$self->_Set( '__suitables'  , \@ao_suitables  );
	$self->_Set( '__validProbes', \@a_validProbes );
	
	#GlintFreq parameters
	$self->_Set( '__w', $h_param{ W });
	$self->_Set( '__d', $h_param{ D });
	$self->_Set( '__count_cutoff' , $h_param{ COUNT_CUTOFF }  );
	$self->_Set( '__conseq_freq'  , $h_param{ CONSEQ_FREQ }   );
	$self->_Set( '__conseq_unfreq', $h_param{ CONSEQ_UNFREQ } );
	$self->_Set( '__condemned_min_size', $h_param{ CONDEMNED_MIN_SIZE });
	$self->_Set( '__suitable_min_size' , $h_param{ SUITABLE_MIN_SIZE } );
	
	#Primer3 parameters
	my %h_primer3_param = 
						( PRIMER_OPT_SIZE => $h_param{ PRIMER_OPT_SIZE },
						  PRIMER_MAX_SIZE => $h_param{ PRIMER_MAX_SIZE },
						  PRIMER_PRODUCT_SIZE_RANGE => $h_param{ PRIMER_PRODUCT_SIZE_RANGE },
						);
	$self->_Set( '__primer3_param', \%h_primer3_param );
	
	#executables location
	$self->_Set( '__glintFreq_exe', $h_param{ GLINTFREQ_EXE } );
	$self->_Set( '__glint_exe'    , $h_param{ GLINT_EXE }     );
	$self->_Set( '__primer3_exe'  , $h_param{ PRIMER3_EXE }   );
	$self->_Set( '__rePcr_exe'    , $h_param{ rePCR_EXE }     );
	$self->_Set( '__fahash_exe'   , $h_param{ FAHASH_EXE }    );
	$self->_Set( '__famap_exe'    , $h_param{ FAMAP_EXE }     );
	
	#Covering
	my $regionCoverage  = undef;
	my $probeCoverage   = undef;
	my $representation  = undef;
	my $coverageRepStep = undef;
	$self->_Set( '__regionCoverage' , $regionCoverage  );
	$self->_Set( '__probeCoverage'  , $probeCoverage   );
	$self->_Set( '__representation' , $representation  );
	$self->_Set( '__coverageRepStep', $coverageRepStep );
	$self->_Set( '__coverageRepSize'     , $h_param{ COVERAGE_REP_SIZE }      );
	$self->_Set( '__coverageRepThreshold', $h_param{ COVERAGE_REP_THRESHOLD } );
	
	#Selection
	$self->_Set( '__select'          , $h_param{ SELECT } );
	$self->_Set( '__totalProbeNumber', $h_param{ TOTAL_PROBE_NUMBER } );
	$self->_Set( '__scoreThreshold'  , $h_param{ SCORING_THRESHOLD }  );
	
	#VSPrimer3Only
	my @ao_versusProbes = ();
	$self->_Set( '__versusProbes', \@ao_versusProbes     );
	$self->_Set( '__versus'      , $h_param{ VERSUS } );
	
	#verbose option
	$self->_Set( '__verbose', $h_param{ VERBOSE } );
	
	#debug option
	$self->_Set( '__debug', $h_param{ DEBUG } );
	if ( defined $self->_Get( '__select' ) )
	{
		$self->_Set( '__debug', 1 );
	}
	
	# re-PCR files famap and fahash
	my $seqName = &GetSeqName( $index_file );
	$self->_Set( '__seqName', $seqName );
	
	unless ( -e "../../data/pork_genome/$seqName".'map' )
	{
		die "You must run famap program before runing 'glintfreq.pl'\n
		-> $h_param{ FAMAP_EXE } -tN -b'genome.famap' $index_file";
	}
	if ( !( -e "./$target_dir/$seqName".'map' ) )
	{
		$self->__LaunchShellcommand("ln -s /home/asagne/Travail/Projets/ProberLikeSoft/data/pork_genome/$seqName"."map ./$target_dir/$seqName".'map');
	}

	unless ( -e "../../data/pork_genome/$seqName".'hash' )
	{
		die "You must run fahash program before runing 'glintfreq.pl' \n
		-> $h_param{ FAMAP_EXE } -b'genome.hash' -w18 -f1 'genome.famap'";
	}
	if ( !( -e "./$target_dir/$seqName".'hash' ) )
	{
		$self->__LaunchShellcommand("ln -s /home/asagne/Travail/Projets/ProberLikeSoft/data/pork_genome/$seqName"."hash ./$target_dir/$seqName".'hash');
	}
	
	return $self;
}

=head2 function ExtractRegionSequence

 Title    : ExtractRegionSequence
 Usage    : Extract a region sequence and create the query file in the target directory
 Access   : Public
 Function : Extract a region sequence and create the query file in the target directory
 Args     : $chromosome_file => the chromosome file '.fa'
            $rh_region       => the hash ( chr, start, end ) used to create the file
            $target_dir      => the target directory used 
 Returns  : the output file name
 Globals  : none

=cut
sub ExtractRegionSequence
{
	my ( $self, $species, $rh_region, $target_dir ) = @_;

	my ( $chrom, $start, $end ) = @{ $rh_region }{ qw( chr start end ) };
	
	my $chromosome_file = $self->_Get( '__chr_filename' );
	
	my $force = $self->_Get( '__force' );
	my $versus = $self->_Get( '__versus' );
	
	my $query_Filename = $self->_Get( '__query_filename' );
	
	my $output_file_path = $target_dir.'/'.$query_Filename;

	if ( !( -e $output_file_path ) || ( defined $force ) )
	{
		my $seq_in        = Bio::SeqIO->new( -file => "<$chromosome_file", -format => 'fasta' );
		my $seq_obj       = $seq_in->next_seq();
		my $seq_region    = $seq_obj->subseq( $start+1,$end+1 );
		my $seq_out       = Bio::SeqIO->new( -file => ">$output_file_path", -format => 'fasta' );
		my $seqregion_obj =  Bio::Seq->new( -seq => $seq_region, -display_id => "region_$chrom\_$start-$end" );
		$seq_out->write_seq( $seqregion_obj );
	}
}

sub GetSeqName
{
	my ( $genome ) = @_ ;
	if ( $genome =~ /^\/.*\/data\/(.*)\/(.*)\.(seq|fa)/ )
	{
		my $seqName = $2.'.'.$3;
		return $seqName;
	}
}

sub SetQueryFileName
{
	my ( $self ) = @_;
	
	my $sp    = $self->Species();
	my $chr   = $self->Chr();
	my $start = $self->Start();
	my $end   = $self->End();
	
	my $query_Filename = "QueryFileName_$sp.$chr:$start-$end";
	
	return $query_Filename;
}

############################################################
#####                 GETTERS/SETTERS                  #####
############################################################

=head2 function Chr

 Title        : Chr
 Usage        : $chr = $region->Chr();
 Function     : Accessor, returns the chromosome number
 Access       : Public
 Returns      : $chr
 Args         : none
=cut
sub Chr
{
	my $self = shift;
	return $self->_Get('__chr');
}

=head2 function Start

 Title        : Start
 Usage        : $start = $region->Start();
 Function     : Accessor, returns the start position
 Access       : Public
 Returns      : $start
 Args         : none
 
=cut
sub Start
{
	my $self = shift;
	return $self->_Get('__start');
}

=head2 function End

 Title        : End
 Usage        : $end = $region->end();
 Function     : Accessor, returns the end position
 Access       : Public
 Returns      : $end
 Args         : none
=cut
sub End
{
	my $self = shift;
	return $self->_Get('__end');
}

=head2 function Species

 Title        : Species
 Usage        : $species = $region->Species();
 Function     : Accessor, returns the species
 Access       : Public
 Returns      : $species
 Args         : none
=cut
sub Species
{
	my $self = shift;
	return $self->_Get('__species');
}

=head2 function Size

 Title        : Size
 Usage        : $size = $region->Size();
 Function     : Accessor, returns the region size
 Access       : Public
 Returns      : $size
 Args         : none
=cut
sub Size
{
	my $self = shift;
	return $self->_Get('__size');
}

=head2 function RegionCoverage

 Title        : RegionCoverage
 Usage        : $regionCoverage = $region->RegionCoverage();
 Function     : Accessor, returns the region coverage
 Access       : Public
 Returns      : $regionCoverage
 Args         : none
=cut
sub RegionCoverage
{
	my $self = shift;
	return $self->_Get('__regionCoverage');
}

=head2 function ProbeCoverage

 Title        : ProbeCoverage
 Usage        : $probeCoverage = $region->ProbeCoverage();
 Function     : Accessor, returns the region probe coverage
 Access       : Public
 Returns      : $probeCoverage
 Args         : none
=cut
sub ProbeCoverage
{
	my $self = shift;
	return $self->_Get('__probeCoverage');
}

=head2 function Suitables

 Title        : Suitables
 Usage        : $ra_suitables = $region->Suitables();
 Function     : Accessor, returns the suitable subregion array reference 
 Access       : Public
 Returns      : $ra_suitables
 Args         : none
=cut
sub Suitables
{
	my $self = shift;
	return $self->_Get('__suitables');
}

=head2 function VersusProbes

 Title        : VersusProbes
 Usage        : $ra_suitables = $region->Suitables();
 Function     : Accessor, returns the suitable subregion array reference 
 Access       : Public
 Returns      : $ra_suitables
 Args         : none
=cut
sub VersusProbes
{
	my $self = shift;
	return $self->_Get('__versusProbes');
}

=head2 function ValidProbes

 Title        : ValidProbes
 Usage        : $ra_validProbes = $region->ValidProbes();
 Function     : Accessor, returns the valid probes array reference 
 Access       : Public
 Returns      : $ra_validProbes
 Args         : none
 
=cut
sub ValidProbes
{
	my $self = shift;
	return $self->_Get('__validProbes');
}

=head2 function GetASuitable

 Title        : GetASuitable
 Usage        : $suitable = $region->GetASuitable($id);
 Function     : Accessor, returns the suitable subregion corresponding to the $id 
 Access       : Public
 Returns      : $suitable
 Args         : none
 
=cut
sub GetASuitable
{
	my ( $self, $id ) = @_;
	my $ra_suitables = $self->Suitables();
	return $$ra_suitables[$id-1];
}

=head2 function GetAProbe

 Title        : GetAProbe
 Usage        : $probe = $region->GetAProbe($id);
 Function     : Accessor, returns the probe corresponding to the $id 
 Access       : Public
 Returns      : $probe
 Args         : none
 
=cut
sub GetAProbe
{
	my ( $self, $id ) = @_;
	my $suitableId = int( $id/10 );
	my $o_suitable = $self->GetASuitable( $suitableId );
	my $o_probes = $o_suitable->GetAProbe( $id );
	
	return $o_probes;
}

=head2 function FindAProbe

 Title        : FindAProbe
 Usage        : $probe = $region->FindAProbe( $from, $end );
 Function     : Accessor, returns the the probe corresponding to the start and end position 
 Access       : Public
 Returns      : $Probe_id
 Args         : none
 
=cut
sub FindAProbe
{
	my ( $self, $from, $end ) = @_;
	
	my $rao_suitables = $self->Suitables();
	
	foreach my $o_suitable ( @$rao_suitables )
	{
		my $rao_probes = $o_suitable->Probes();
		
		foreach my $o_probe ( @$rao_probes )
		{
			my $probe_from = $o_probe->From();
			my $probe_end  = $o_probe->End();
			my $probe_id   = $o_probe->Id();
			
			if ( ( $probe_from == $from ) && ( $probe_end == $end ) )
			{
				return $probe_id;
			}
		}
	}
}

=head2 function FindAValidProbe

 Title        : FindAValidProbe
 Usage        : $probe = $region->FindAValidProbe( $from, $end );
 Function     : Accessor, returns the the valid probe corresponding to the start and end position 
 Access       : Public
 Returns      : $validProbe_id
 Args         : none
 
=cut
sub FindAValidProbe
{
	my ( $self, $from, $end ) = @_;
	
	my $rao_validProbe = $self->ValidProbes();
	
	foreach my $validProbe_id ( @$rao_validProbe )
	{
		my $o_validProbe = $self->GetAProbe( $validProbe_id );
		
		my $validProbe_from = $o_validProbe->From();
		my $validProbe_end  = $o_validProbe->End();
		
		if ( ( $validProbe_from == $from ) && ( $validProbe_end == $end ) )
		{
			return $validProbe_id;
		}
	}
}

=head2 function GetAVersusProbe

 Title        : GetAVersusProbe
 Usage        : $versusProbe = $region->GetAVersusProbe($id);
 Function     : Accessor, returns the versus probe corresponding to the $id 
 Access       : Public
 Returns      : $versusProbe
 Args         : none
 
=cut
sub GetAVersusProbe
{
	my ( $self, $id ) = @_;
	
	my $rao_versusProbes = $self->_Get( '__versusProbes' );
	
	return $$rao_versusProbes[ $id ];
}

=head2 function SetOverlapProbes

 Title        : SetOverlapProbes
 Usage        : $region->SetOverlapProbes();
 Function     : Set the overlap argument for all probes
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub SetOverlapProbes
{
	my ( $self ) = @_;
	
	my $rao_suitables = $self->Suitables();
	my @ao_suitables  = @$rao_suitables;
	
	foreach my $o_suitable ( @ao_suitables )
	{
		my $suitableOffset = $o_suitable->Offset();
		my $rao_probe = $o_suitable->Probes();
		my @ao_probe  = @$rao_probe;
		
		my ($start, $end ) = 0;
		foreach my $o_probe( @ao_probe )
		{
			my $start_probe = $o_probe->Offset();
			my $end_probe   = $start_probe + $o_probe->Size();
			
			if ( $start != 0 && ( ( $start_probe <= $start ) || ( ( $start_probe <= $end ) && ( $end <= $end_probe ) ) ) )
			{
				$o_probe->SetOverlap(1);
			}
			
			else
			{
				$start = $start_probe;
				$end   = $end_probe;
			}
		}
	}
}


=head2 function GetValidProbeInfos

 Title        : GetValidProbeInfos
 Usage        : $validProbeInfo = $region->GetValidProbeInfos($suitableId);
 Function     : Accessor, returns the valid probes infos 
                for a suitable subregion corresponding to the $suitableId
 Access       : Public
 Returns      : $validProbeInfos
 Args         : none
 
=cut
sub GetValidProbeInfos
{
	my ( $self, $suitableId ) = @_ ;
	
	my $ra_validProbes = $self->ValidProbes() ;
	my @a_validProbes  = @$ra_validProbes ;
	
	my $result = '';
	foreach my $validProbe ( @a_validProbes )
	{
		if ($validProbe =~ /^$suitableId\t\d+\t\d+\t\d+/)
		{
			$result .= "$validProbe \n";
		}
	}
	
	return $result;
}

=head2 function SuitableSize

 Title        : SuitableSize
 Usage        : $nbSuitable = $region->SuitableSize();
 Function     : Accessor, returns the suitable subregion array lenght 
 Access       : Public
 Returns      : $nbSuitable
 Args         : none
 
=cut
sub SuitableSize
{
	my $self = shift;
	
	my $ra_tab = $self->_Get('__suitables');;
	my @a_tab = @$ra_tab;
	
	return $#a_tab+1;
}

sub GetProbePosition
{
	my ( $self, $probe_id ) = @_;
	
	my $o_probe = $self->GetAProbe( $probe_id );
	
	my $probeFrom = $o_probe->From();
	my $probeEnd  = $o_probe->End() ;
	
	my $regionSize  = $self->Size();
	my $regionStart = $regionSize*0.35;
	my $regionEnd   = $regionSize*0.65 ;

	my $regionMiddle_start = $regionSize*0.4;
	my $regionMiddle_end   = $regionSize*0.6;
	
	if ( $probeEnd < $regionStart )
	{
		return start;
	}
	elsif ( $probeFrom > $regionEnd )
	{
		return end;
	}
	elsif ( ( $probeFrom > $regionMiddle_start ) && ( $probeEnd < $regionMiddle_end ) )
		{
			return middle; }
	else { return 'inbetween'; }
}

############################################################
#####                   FUNCTIONS                      #####
############################################################

=head2 function CntToId

 Title        : CntToId
 Usage        : &CntToId($cnt);
 Function     : Transform a counter of probes into the correct probe id
 Access       : Public
 Returns      : none
 Args         : $cnt => prober counter

=cut
sub CntToId
{
	my ( $cnt ) = @_ ;
	my $id = undef;
	
	if ( ( $cnt%5 )== 0 ) 
	{
		$id = ( ( $cnt/5 )*10 )+4 ;
	}
	else
	{
		if ( ( $cnt < 10 && $cnt > 5 ) || ( $cnt - ( int($cnt / 10 ) * 10 ) > 5 ) )
		{
			$id = ( ( ( int( $cnt/5 )+1 )*10 ) + ( ($cnt - ( int( $cnt / 10 ) * 10 )-6) ) );
		}
		else
		{
			$id = ( ( ( int( $cnt/5 )+1 )*10 ) + ( ($cnt - ( int( $cnt / 10 ) * 10 )-1) ) );
		}
	}
	return  $id ;
}

###### PUSHING ######

=head2 function PushSuitable

 Title        : PushSuitable
 Usage        : $region->PushProbes($suitable);
 Function     : Push a suitable subregion in the suitable subregion array of the region object
 Access       : Public
 Returns      : none
 Args         : $suitable => the suitable subregion pushed

=cut
sub PushSuitable
{
	my ( $self, $o_suitable ) = @_;
	
	my $rao_suitables = $self->_Get( '__suitables' );
	push( @$rao_suitables, $o_suitable );
}

=head2 function PushValidProbe

 Title        : PushValidProbe
 Usage        : $region->PushValidProbe($validProbes);
 Function     : Push a valid probe information line in the valid probe array 
 Access       : Public
 Returns      : none
 Args         : $validProbes => tabulated string (suitableId\tProbeId\tOffset\tSize)

=cut
sub PushValidProbe
{
	my ( $self, $validProbes ) = @_;
	
	my $ra_validProbes = $self->_Get( '__validProbes' );
	push( @$ra_validProbes, $validProbes );
}

=head2 function PushVersusProbe

 Title        : PushVersusProbe
 Usage        : $region->PushVersusProbe($o_versusProbes);
 Function     : Push a versus probe object in the versus probe object array 
 Access       : Public
 Returns      : none
 Args         : $o_versusProbes 

=cut
sub PushVersusProbe
{
	my ( $self, $o_versusProbes ) = @_;
	
	my $rao_versusProbes = $self->_Get( '__versusProbes' );
	push( @$rao_versusProbes, $o_versusProbes );
}

#~ =head2 function PushProbeRef
#~ 
 #~ Title        : PushProbeRef
 #~ Usage        : $region->PushProbeRef($ro_suitable);
 #~ Function     : Push in a specific subregion a probe
 #~ Access       : Public
 #~ Returns      : none
 #~ Args         : $ro_suitable => the suitable subregion reference
                #~ $probe       => the probe pushed
 #~ Todo         : delete this fonction
#~ =cut
#~ sub PushProbeRef
#~ {
	#~ my ( $self, $ro_suitable, $o_probe ) = @_;
	#~ 
	#~ $$ro_suitable->PushProbes($o_probe);
#~ }
#~ 
#~ =head2 function PushProbes
#~ 
 #~ Title        : PushProbes
 #~ Usage        : $region->PushProbes( $id, $probe );
 #~ Function     : Push a probe in the suitable subregion (corresponding to the $id)
                #~ array of the region object
 #~ Access       : Public
 #~ Returns      : none
 #~ Args         : $probe => the probe pushed
#~ 
#~ =cut
#~ sub PushProbes
#~ {
	#~ my ( $self, $id, $o_probe ) = @_;
	#~ 
	#~ my $o_suitable = $self->GetASuitable( $id ) ;
	#~ $o_suitable->PushProbe( $o_probe );
#~ }

###### TEMPFILE ######

=head2 function GetTempFileName

 Title        : GetTempFileName
 Usage        : $tempFileName = &GetTempFileName($process);
 Function     : Accessor, returns the Amplicon start position
 Access       : Public
 Returns      : $from
 Args         : none

=cut
sub GetTempFileName
{
	my ( $processus, $type, $ext ) = @_;
	
	unless ( defined $ext ) { $ext = '' ; }
	die "\n Error : Wrong argument in GetTempFileName $!\n" if ( $type !~ /(in|out|log|param|err)/ );
	
	my $tempFileName = $processus.'.'.$type.'.GF-tmp'.$ext;
	
	return $tempFileName;
}

=head2 function CleanTempFileName

 Title        : CleanTempFileName
 Usage        : &CleanTempFileName();
 Function     : Erase all files created
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub CleanTempFileName
{
	my $command = "rm -I *.GF-tmp*";
	system ( $command );
}

###### GLINTFREQ ######

=head2 function DetectSuitableSubregion

 Title        : DetectSuitableSubregion
 Usage        : $region->DetectSuitableSubregion( $bank, $queryFile );
 Function     : Run glintfreq program 
                Create the inputFile for Primer3
                and push all the suitable subregion in the region object
 Access       : Public
 Returns      : none
 Args         : $bank  => the gene fasta file path 
                $query => the query fasta file path

=cut
sub DetectSuitableSubregion
{
	my $self = shift;
	
	my $select = $self->_Get( '__select' );
	
	unless ( defined $select ) { $self->LaunchGlintFreq(); }
	
	$self->LoadSuitable();
	
	my $rao_suitables = $self->Suitables();
	if ( scalar @$rao_suitables < 2 ) 
	{ 
		print colored ( "\nSorry, Insufficient number of suitable subregion ( less than 2 )\n", 'red' ) ;
		print 'Maybe you should modify the glintFreq parameters in the cfg file, ';
		print "try to increase count_cutoff and conseqUnFreq.\n\n";
		
		exit(1);
	}
}

=head2 function LaunchGlintFreq

 Title        : LaunchGlintFreq
 Usage        : &LaunchGlintFreq();
 Function     : Run GlintFreq software
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub LaunchGlintFreq
{
	my $self = shift ;
	
	my $glintFreq_exe = $self->_Get( '__glintFreq_exe' ) ;
	
	my $param = $self->GetGlintFreqParameters();
	
	my $bank      = $self->_Get( '__genome' ) ;
	my $queryFile = $self->_Get( '__query_filename' ) ;
	
	my $output_file_name = &GetTempFileName( 'glintfreq', 'out' ) ;
	my $log_file_name    = &GetTempFileName( 'glintfreq', 'log' ) ;
	my $err_file_name    = &GetTempFileName( 'glintfreq', 'err' ) ;
	
	my $command = $glintFreq_exe." $bank $queryFile -C 0 $param >$log_file_name -o $output_file_name 2>$err_file_name";
	$self->__LaunchShellcommand( $command );
}

=head2 function GetGlintFreqParameters

 Title        : GetGlintFreqParameters
 Usage        : $glintParam = $self->GetGlintFreqParameters;
 Function     : Return a string wth all the glint param from ParamParser
 Access       : Public
 Returns      : $glint param
 Args         : none

=cut
sub GetGlintFreqParameters
{
	my $self = shift;
	
	my $glintParam = undef;
	
	my $w             = '-w ';
	my $count_cutoff  = '--count_cutoff ';
	my $conseq_freq   = '--conseqFreq '  ;
	my $conseq_unfreq = '--conseqUnFreq ';
	my $condemned_min_size = '--condemned_min_size ';
	my $suitable_min_size  = '--suitable_min_size ' ;
	
	$w             .= $self->_Get( '__w' );
	$count_cutoff  .= $self->_Get( '__count_cutoff' );
	$conseq_freq   .= $self->_Get( '__conseq_freq'  );
	$conseq_unfreq .= $self->_Get( '__conseq_unfreq');
	$condemned_min_size .= $self->_Get( '__condemned_min_size' );
	$suitable_min_size  .= $self->_Get( '__suitable_min_size'  );
	
	$glintParam = "$w $count_cutoff $conseq_freq $conseq_unfreq $condemned_min_size $suitable_min_size ";
	
	return $glintParam;
}

=head2 function LoadSuitable

 Title        : LoadSuitable
 Usage        : $region->LoadSuitable( $glintfreqFile );
 Function     : Parses the glintfreq output
                Create the inputFile for Primer3
                and Push all the suitable subregion in the region object
 Access       : Public
 Returns      : none
 Args         : $glintfreqFile => the glintfreq output file path 
                
=cut
sub LoadSuitable
{
	my $self             = shift;
	my $region_start     = $self->Start();
	my $output_file_name = &GetTempFileName( 'glintfreq', 'out' ) ;
	
	#Get all the lines of the file
	my $all_lines = '';
	my $line      = '';
	
	open ( FILE, $output_file_name ) || die "\nGlint Freq :\nErreur E/S : $output_file_name $!\n";
		while ( $line = <FILE> ) { $all_lines.= $line; }
	close ( FILE );
	
	#Split the multifasta file with "\n"
	my @a_sequences = split ( '\n', $all_lines );
	
	# Fill the hash in the same time
	my @field_names = qw( id header offset end );
	
	foreach my $sequence ( @a_sequences )
	{
		if ( $sequence =~ /^>(\d+)\|(\S+)\tstart : (\d+)\tend : (\d+)/ )
		{
			my %h_line;
			@h_line{ @field_names } = ( $1, $2, $3, $4 );
			
			my $id     = $h_line { id } ;
			my $header = $h_line { header } ;
			my $offset = $h_line { offset } ;
			my $end    = $h_line { end } ;
			my $from   = ( $offset + $region_start );
			my $size   = ( $end - $offset ) ;
			
			my $o_suitable = New SuitableRegion( $id, $offset, $from, $size ) ;
			$self->PushSuitable( $o_suitable ) ;
		}
	}
}

###### PRIMER 3 ######

=head2 function GetPrimer3Parameters

 Title        : GetPrimer3Parameters
 Usage        : $primer3Param = $self->GetPrimer3Parameters( $rh_primer3_param );
 Function     : Return a string wth all the glint param from ParamParser
 Access       : Public
 Returns      : $glint param
 Args         : none

=cut
sub GetPrimer3Parameters
{
	my ( $self, $rh_primer3_param, $versus ) = @_;
	my %h_primer3_param  = %$rh_primer3_param;
	
	my $primerParamFile = &GetTempFileName( 'Primer3', 'param' );
	
	open( FILE, ">$primerParamFile" ) || die "Primer3 :\nErreur E/S: $primerParamFile $!\n";
		print FILE "Primer3 File - http://primer3.sourceforge.net\n";
		print FILE "P3_FILE_TYPE=settings\n";
		
		while ( my ( $key, $value ) = each %h_primer3_param )
		{
		  print FILE "$key=$h_primer3_param{$key}\n";
		}
		if ( defined $versus ) 
		{
			my $ra_validProbes = $self->ValidProbes();
			my $nb_validProbes = scalar @$ra_validProbes;
			my $primer_num_returned = $nb_validProbes;
			print FILE "PRIMER_NUM_RETURN=$primer_num_returned\n";
		}
		print FILE '=';
	close ( FILE );
}

=head2 function DetectAllProbes

 Title        : DetectAllProbes
 Usage        : $region->DetectAllProbes( $primerInputFile );
 Function     : Run Primer3 program 
                Parses the output file of Primer3
                and push all the probes in the region object
 Access       : Public
 Returns      : none
 Args         : $primerInputFile => the primer3 input file path 

=cut
sub DetectAllProbes
{
	my $self = shift;
	
	my $primer3_exe    = $self->_Get( '__primer3_exe'    );
	my $query_Filename = $self->_Get( '__query_filename' );
	
	my $ra_subregion = $self->Suitables();
	
	my $rh_primer3_param = $self->_Get( '__primer3_param' );
	$self->GetPrimer3Parameters( $rh_primer3_param );
	
	if ( defined $self->_Get( '__verbose' ) ) { print colored ( "\n->$primer3_exe -p3_settings_file primerParamFile <primerInputFile -output primerOutputFile\n", 'blue'), ; }
	
	foreach my $subregion ( @$ra_subregion )
	{
		$subregion->GeneratePrimer3InputFile( $query_Filename );
		$subregion->LaunchPrimer3( $primer3_exe );
		$subregion->LoadProbe();
	}
	$self->SetOverlapProbes();
}

###### CHEKING ######

=head2 function LoadValidProbes

 Title        : LoadValidProbes
 Usage        : $region->LoadValidProbes();
 Function     : Push valid probe in the valideProbe array
 Access       : Public
 Returns      : none

=cut
sub LoadValidProbes
{
	my $self = shift;
	
	my $ra_validProbes = $self->ValidProbes();
	
	my $rao_suitables = $self->Suitables();
	my @ao_suitables  = @$rao_suitables;
	
	my $cnt = 0;
	foreach my $o_suitable ( @ao_suitables )
	{
		my $rao_probe = $o_suitable->Probes() ;
		my @ao_probe  = @$rao_probe;
		
		foreach my $o_probe ( @ao_probe )
		{
			if ( $o_probe->IsValid == 1 )
			{
				$cnt++;
				$o_probe->SetValidProbeId( $cnt );
				push( @$ra_validProbes, $o_probe->Id() );
			}
		}
	}
	if ( $cnt == 0 ) 
	{
		print colored ( "\nSorry, No valid probe found!\n\n", 'red' ) ;
		exit(1);
	}
}

      ###### RePcr

=head2 GenerateRePcrInputFile

 Usage    : Create the UniSTS File for rePCR soft
 Function : GenerateRePcrInputFile
 Return   : void

 Title        : GenerateRePcrInputFile
 Usage        : &GenerateRePcrInputFile()
 Prerequisite : none
 Function     : Create the UniSTS file for rePCR soft
 Returns      : void
 Args         : none
 Globals      : none

=cut
sub GenerateRePcrInputFile
{
	my ( $self, $versus ) = @_;
	
	my $file_name = &GetTempFileName( 'RePcr', 'in' );
	if (defined $versus )
	{
		$file_name = &GetTempFileName( 'RePcrVersus', 'in' );
	}
	
	my $cpt = 0;
	my $leftPrimer  = undef;
	my $rightPrimer = undef;
	my $size        = undef;
	
	#write the inputFile with tags and values
	open( FILE, ">$file_name" ) || die "Erreur E/S:$!\n";

	if ( ! defined $versus )
	{
		my $rao_suitables = $self->Suitables();
		my @ao_suitables  = @$rao_suitables;
		
		foreach my $o_suitable ( @ao_suitables )
		{
			my $rao_probe = $o_suitable->Probes();
			my @ao_probe  = @$rao_probe;
			foreach my $o_probe( @ao_probe )
			{
				$cpt ++;
				$leftPrimer   = $o_probe->LeftPrimer();
				$rightPrimer  = $o_probe->RightPrimer();
				$size         = $o_probe->Size();
				my $id        = $o_probe->Id();
				my $pair_penalty = $o_probe->Pair_Penalty();
				
				print FILE "$cpt\t$leftPrimer\t$rightPrimer\t$size\t$id\t$pair_penalty\n";
				
			}
		}
	}
	else
	{
		my $rao_versusProbes = $self->VersusProbes();
		my @ao_versusProbes  = @$rao_versusProbes;
		
		foreach my $o_versusProbe ( @ao_versusProbes )
		{
			
			$leftPrimer   = $o_versusProbe->LeftPrimer();
			$rightPrimer  = $o_versusProbe->RightPrimer();
			$size         = $o_versusProbe->Size();
			my $id        = $o_versusProbe->Id();
			my $pair_penalty = $o_versusProbe->Pair_Penalty();
			
			print FILE "$cpt\t$leftPrimer\t$rightPrimer\t$size\t$id\t$pair_penalty\n";
			$cpt ++;
		}
	}
	
	close ( FILE );
}

=head2 function LaunchRePcr

 Title        : LaunchRePcr
 Usage        : &LaunchRePcr();
 Function     : Run RePcr software
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub LaunchRePcr
{
	my ( $self, $versus ) = @_;
	
	my $rePcrOutputFile = undef;
	my $rePcrInputFile  = undef;
	
	my $rePCR_exe = $self->_Get( '__rePcr_exe' );
	my $seqName   = $self->_Get( '__seqName'   );
	
	my $rePcr_log_File  = &GetTempFileName( 'RePcr', 'log'  );
	
	if ( defined $versus )
	{
		$rePcrOutputFile = &GetTempFileName( 'RePcrVersus', 'out' );
		$rePcrInputFile  = &GetTempFileName( 'RePcrVersus', 'in'  );
	}
	else
	{
		$rePcrOutputFile = &GetTempFileName( 'RePcr', 'out' );
		$rePcrInputFile  = &GetTempFileName( 'RePcr', 'in'  );
	}
	
	my $command = $rePCR_exe." -S $seqName".'hash'." -n0 -g0 -o'$rePcrOutputFile' -O+ $rePcrInputFile -C 500 >$rePcr_log_File 2>$rePcr_log_File";
	$self->__LaunchShellcommand( $command );
}

=head2 CheckProbes

 Usage    : Check probes genome localisation using the rePCR soft output
            Check : uniqueness, positionning, length and non-overlapping
 Function : CheckProbes
 Arg      : none
 Return   : void

 Title        : CheckProbes
 Usage        : $region->CheckProbes( )
 Prerequisite : none
 Function     : Check probes genome localisation and specifity using the rePCR soft output
 Returns      : void
 Args         : none
 Globals      : none

=cut
sub CheckProbes
{
	my $self = shift;
	
	$self->GenerateRePcrInputFile();
	
	my $select = $self->_Get( '__select' );
	unless ( defined $select )
	{
		$self->LaunchRePcr();
	}
	
	my $rePCROutputFile = &GetTempFileName('RePcr', 'out' ) ;
	
	#Get all the lines of the file
	open ( FILE, $rePCROutputFile ) || die "\nCheck Probes : \nErreur E/S : $rePCROutputFile $!\n";
		my @a_line;
		my $line = '';
		while( $line = <FILE> ) 
		{
			next if ( $line =~ /^#/);
			push ( @a_line, $line ) ; 
		}
	close ( FILE );
	
	#Launch Amplicons
	my @field_names = qw( probe_id id strand from to mism gaps actLen expLen );
	my %h_line;
	
	# test the uniqueness 
	my %h_rePCRresult ;
	foreach $line ( @a_line )
	{
		if ($line =~ /^(\d+)\t(.*)\t([+,-])\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+).(\d+)-(\d+)/ )
		{
			my $probe_id = &CntToId($1);
			if ( !defined $h_rePCRresult{ $probe_id } )
				{ $h_rePCRresult{ $probe_id } = 1; }
			else 
				{ $h_rePCRresult{ $probe_id } += 1; }
		}
	}
	foreach my $key ( keys %h_rePCRresult )
	{
		if ( $h_rePCRresult{$key} != 1 ) 
			{ 
				delete $h_rePCRresult{$key}; 
			}
	}
	foreach $line ( @a_line )
	{
		if ($line =~ /^(\d+)\t(.*)\t([+,-])\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+).(\d+)-(\d+)/ )
		{
			my $probe_id = &CntToId($1);
			@h_line{ @field_names } = ( $probe_id, $2, $3, $4, $5, $6, $7, $8, $9 );
			my $o_probe  = $self->GetAProbe( $h_line{ probe_id } );
			if ( defined $h_rePCRresult{ $probe_id } )
			{
				if ( ( $o_probe->IsWellPositionned( $h_line { from } ) == 1 ) && ( $o_probe->Overlap == 0 ) )	# test on positionning and overlapping
				{
					if ( $h_line { actLen } == $h_line { expLen } )										# test on length
					{
						$o_probe->SetValid(1);
					}
				}
			}
		}
	}
}

      ###### Glint

=head2 GenerateGlintInputFile

 Usage    : Create the File for the glint soft
 Function : GenerateGlintInputFile
 Return   : void

 Title        : GenerateGlintInputFile
 Usage        : &GenerateGlintInputFile()
 Prerequisite : none
 Function     : Create the file for glint software
 Returns      : void
 Args         : none
 Globals      : none

=cut
sub GenerateGlintInputFile
{
	my ( $self , $tag ) = @_;
	
	my $file_name = &GetTempFileName( 'glint', 'in' );
	
	if ( defined $tag  )
	{
		$file_name = &GetTempFileName( 'glintVersus', 'in' );
	}
	my $query     = $self->_Get( '__query_filename' );
	
	#write the inputFile with tags and values
	open( FILE, ">$file_name" ) || die "Glint :\nErreur E/S: $file_name $!\n";
	
	my $seq      = undef;
	my $line     = undef;
	my $probe_id = undef;
	
	if ( !defined $tag )
	{
		my $rao_suitables = $self->Suitables();
		my @ao_suitables  = @$rao_suitables;
		foreach my $o_suitable ( @ao_suitables )
		{
			my $rao_probe = $o_suitable->Probes();
			my @ao_probe  = @$rao_probe;
			
			foreach my $o_probe( @ao_probe )
			{
				if ( $o_probe->IsValid == 1 )
				{
					$probe_id = $o_probe->Id();
					$line = ">$probe_id\n";
					$seq  = $o_probe->ExtractProductPcrSequence( $query );
					$o_probe->SetProductPcrSeq( $seq );
					$line .= $seq;
					print FILE $line ;
				}
			}
		}
	}
	else 
	{
		my $rao_versusProbes = $self->_Get( '__versusProbes' );
		my  @ao_versusProbes = @$rao_versusProbes;
		
		foreach my $o_versusProbe ( @ao_versusProbes )
		{
			$probe_id = $o_versusProbe->Id();
			
			my $start = $o_versusProbe->Offset();
			my $size  = $o_versusProbe->Size();
			my $end   = $start+$size;
			
			$seq = Bio::SeqIO->new( -file => $query , -format => 'fasta' );
			my $seqObj = $seq->next_seq();
			my $subseq = $seqObj->subseq( $start,$end );
			
			print FILE ">$probe_id\n" ;
			print FILE "$subseq\n" ;
		}
	}
	close ( FILE ) ;
}

=head2 function LaunchGlint

 Title        : LaunchGlint
 Usage        : &LaunchGlint();
 Function     : Run Glint software
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub LaunchGlint
{
	my ( $self, $tag ) = @_ ;
	
	my $processus = 'glint';
	if ( defined $tag ) { $processus = 'glintVersus'; }
	
	my $genome    = $self->_Get( '__genome' );
	my $glint_exe = $self->_Get( '__glint_exe' );
	
	my $glintInputFile  = &GetTempFileName( $processus, 'in'  );
	my $glintOutputFile = &GetTempFileName( $processus, 'out' );
	my $glint_log_File  = &GetTempFileName( $processus, 'log' );
	
	my $command = $glint_exe." $genome $glintInputFile -m 2 -o $glintOutputFile >$glint_log_File 2>$glint_log_File";
	$self->__LaunchShellcommand( $command );
}

=head2 CheckPcrProduct

 Usage    : Check all the pcr product using the glint soft
 Function : CheckPcrProduct
 Arg      : none
 Return   : void

 Title        : CheckPcrProduct
 Usage        : $region->CheckPcrProduct( )
 Prerequisite : none
 Function     : Check pcr product uniqueness and position
 Returns      : void
 Args         : none
 Globals      : none

=cut
sub CheckPcrProduct
{
	my $self = shift;
	
	$self->GenerateGlintInputFile();
	
	my $select = $self->_Get( '__select' );
	unless ( defined $select )
	{
		$self->LaunchGlint();
	}
	
	# Get all the lines of the file
	my $glintOutputFile = &GetTempFileName( 'glint', 'out' );
	
	open ( FILE, $glintOutputFile ) || die "\nCheck PCR Product :\nErreur E/S : $glintOutputFile $!\n";
		my $lines = '';
		my $line  = '';
		while ( $line = <FILE> )
		{
			next if ( $line =~ /^#/ );
			$lines .= $line ;
		}
	close ( FILE );
	
	# Parses the glint output file
	my @a_probe_id_to_delete =();
	
	my @a_lines = split ( /\n/, $lines );
	
	my @field_names = qw( q_name q_start q_end s_name s_start s_end strand score size );
	foreach my $line ( @a_lines )
	{
		if ( $line =~ /(\d+)\t(\d+)\t(\d+)\t(.+)\t(\d+)\t(\d+)\t([+,-])\t(\d+)\t(\d+)/ )
		{
			my %h_line ;
			@h_line{ @field_names } = ( $1, $2, $3, $4, $5, $6, $7, $8, $9 );
			if ( $h_line{ q_end } != $h_line{ score } )
			{
				push ( @a_probe_id_to_delete, $h_line{ q_name } );
			}
		}
	}
	foreach my $probe_id ( @a_probe_id_to_delete )
	{
		my $o_probe  = $self->GetAProbe( $probe_id );
		$o_probe->SetValid(0);
	}
}

###### COVERING ISSUES ###### 

=head2 function _SetCoverage

 Title        : _SetCoverage
 Usage        : $region->_SetCoverage();
 Function     : Calculate and Set the region coverage argument
 Access       : Private
 Returns      : none
 Args         : none
 
=cut
sub _SetCoverage
{
	my $self = shift;
	
	my $rao_suitables = $self->Suitables();
	my @ao_suitables  = @$rao_suitables;
	
	my $min = undef ;
	my $max = undef ;
	
	my $probe_sum    = 0;
	my $region_size  = $self->Size() ;
	
	my $validProbes_cnt = 0;
	
	foreach my $o_suitable ( @ao_suitables )
	{
		my $rao_probe = $o_suitable->Probes() ;
		my @ao_probe  = @$rao_probe;
		
		foreach my $o_probe ( @ao_probe )
		{
			if ( $o_probe->IsValid == 1 )
			{
				$validProbes_cnt++;
				$probe_sum += $o_probe->Size();
				if ( !defined $min || $min > $o_probe->From() ) { $min = $o_probe->From() ; }
				if ( !defined $max || $max < $o_probe->End() )  { $max = $o_probe->End() ; }
			}
		}
	}
	
	print colored ( "\n\t\t$validProbes_cnt Valid Probes\n"      , 'green' );
	print colored ( "\t\t$#ao_suitables Suitables subregions\n", 'green' );
	
	if ( $probe_sum != 0 )
	{
		my $regionCoverage = 100*( $max - $min ) / $region_size  ;
		my $probeCoverage  =  100*$probe_sum / ( $max - $min ) ;
		
		$self->_Set( '__regionCoverage', $regionCoverage );
		print "\t\tRegion coverage : ", $self->RegionCoverage(), " %\n";
		
		$self->_Set( '__probeCoverage',  $probeCoverage );
		print "\t\tProbe  coverage : ", $self->ProbeCoverage(), " %\n";
	}
	
	 $self->CoverageRepresentation();
}

=head2 function CoverageRepresentation

 Title        : CoverageRepresentation
 Usage        : $region->CoverageRepresentation( $coverageSubregionArray_size, $element_threshold );
 Function     : Print in the STDOUT a schematic representation of the region
                a '_', when there is no valid probe in the step
                a '-', when the coverage of the step is under the element threshold
                a '#', when the coverage of the step is over the element threshold
 Access       : Private
 Returns      : none
 Args         : coverageSubregionArray_size, length of the representation
                element_threshold, threashold used for representation
 
=cut
sub CoverageRepresentation
{
	my $self = shift;
	
	my $element_threshold           = $self->_Get( '__coverageRepThreshold' );
	my $coverageSubregionArray_size = $self->_Get( '__coverageRepSize' );
	
	my @a_coverageSusbregion = ();
	my $region_size  = $self->Size() ;
	my $region_start = $self->Start();
	
	my $rao_suitables = $self->Suitables();
	my @ao_suitables  = @$rao_suitables;
	
	foreach my $o_suitable ( @ao_suitables )
	{
		my $rao_probe = $o_suitable->Probes() ;
		my @ao_probe  = @$rao_probe;
		
		foreach my $o_probe ( @ao_probe )
		{
			if ( $o_probe->IsValid == 1 )
			{
				my $end   = $o_probe->End();
				my $size  = $o_probe->Size();
				
				my $pos = int((( $end-$region_start ) *$coverageSubregionArray_size ) / $region_size );
				$a_coverageSusbregion[ $pos-1 ] += $size;
			}
		}
	}
	my $threshold = $region_size*$element_threshold/$coverageSubregionArray_size;
	my $step = $region_size/$coverageSubregionArray_size/1000;
	$self->_Set( '__coverageRepStep', $step );
	if ( defined $self->_Get( '__verbose' ) ) { print "\nRegion Coverage Schematic Representation (threshold : $element_threshold% ; step : $step Mb ; size : $coverageSubregionArray_size )\n"; }
	
	my $representation = '';
	for( my $i = 0; $i < $coverageSubregionArray_size-1; $i++ )
	{
		my $pos = $a_coverageSusbregion[ $i ];
		
		if    ( !defined $pos     ) { $representation .= '_' ; }
		elsif ( $threshold < $pos ) { $representation .= '#' ; }
		else  { $representation .= '-' ; }
	}
	$representation .= "\n";
	
	if ( defined $self->_Get( '__verbose' ) ) { print $representation ; }
	$self->_Set( '__representation', $representation );
}

###### PROBE SELECTION ######

sub SelectProbeByScoring
{
	my ( $self, $score ) = @_;
	
	my @a_selectedProbe = ();
	my $ra_validProbesId = $self->ValidProbes();
	
	foreach my $validProbeId ( @$ra_validProbesId )
	{
		my $o_probe = $self->GetAProbe( $validProbeId );
		my $probePenalty = $o_probe->Pair_Penalty();
		if ( $probePenalty < $score )
			{ push ( @a_selectedProbe, $validProbeId ); }
	}
	return @a_selectedProbe;
}

sub SelectProbeByPosition
{
	my ( $self, $position, @a_probes ) = @_;
	
	if ( $position =~ /(m|mi|mid|middle)/i ) { $position = middle; }
	if ( $position =~ /(^e|en|end)/i )       { $position = end;    }
	if ( $position =~ /(s|st|sta|start)/i )  { $position = start;  }
	
	my @a_selectedProbe = ();
	
	#Selection on a preselect array of probe
	if ( ! @a_probes )
	{
		my $ra_validProbesId = $self->ValidProbes() ;
		@a_probes = @$ra_validProbesId ;
	}
	
	foreach my $validProbeId ( @a_probes )
	{
		my $probe_position = $self->GetProbePosition( $validProbeId );
		if ( $probe_position eq $position )
			{ push( @a_selectedProbe, $validProbeId ); }
	}
	return @a_selectedProbe;
}

#select probes choosing there position and a scoring threshold
sub SelectProbes
{
	my ( $self, $position, $score, $cnt ) = @_;
	my ( @a_selectedProbeByScoring, @a_selectedProbeByPosition ) = ();

	#Scoring threshold
	@a_selectedProbeByScoring = $self->SelectProbeByScoring( $score );
	
	#Position selection
	my $cnt_of_start  = undef ;
	my $cnt_of_middle = undef ;
	my $cnt_of_end    = undef ;
	
	my @field_names = qw( start middle end );
	
	if ( $position =~ /(\d+)\/(\d+)\/(\d+)/ )
	{
		my %h_line ;
		@h_line{ @field_names } = ( $1, $2, $3 );
		$cnt_of_start  = $h_line{ start  };
		$cnt_of_middle = $h_line{ middle };
		$cnt_of_end    = $h_line{   end  };
		
		my @a_startProbes = $self->SelectProbeByPosition( start, @a_selectedProbeByScoring );
		push ( @a_selectedProbeByPosition, @a_startProbes[0..$cnt_of_start-1] );
		
		my @a_middleProbes = $self->SelectProbeByPosition( middle, @a_selectedProbeByScoring );
		push ( @a_selectedProbeByPosition, @a_middleProbes[0..$cnt_of_middle-1] );
		
		my @a_endProbes = $self->SelectProbeByPosition( end, @a_selectedProbeByScoring );
		push ( @a_selectedProbeByPosition, @a_endProbes[0..$cnt_of_end-1] );
	}
	elsif ( $position eq start  )
	{
		my @a_startProbes = $self->SelectProbeByPosition( start, @a_selectedProbeByScoring );
		push ( @a_selectedProbeByPosition, @a_startProbes );
	}
	elsif ( $position eq middle )
	{
		my @a_middleProbes = $self->SelectProbeByPosition( middle, @a_selectedProbeByScoring );
		push ( @a_selectedProbeByPosition, @a_middleProbes );
	}
	elsif ( $position eq end )
	{
		my @a_endProbes = $self->SelectProbeByPosition( end, @a_selectedProbeByScoring );
		push ( @a_selectedProbeByPosition, @a_endProbes );
	}
	else
	{
		my $nb = int($position/3);
		
		my @a_startProbes = $self->SelectProbeByPosition( start, @a_selectedProbeByScoring );
		push ( @a_selectedProbeByPosition, @a_startProbes[0..$nb-1] );
		
		my @a_middleProbes = $self->SelectProbeByPosition( middle, @a_selectedProbeByScoring );
		push ( @a_selectedProbeByPosition, @a_middleProbes[0..$nb-1] );
		
		my @a_endProbes = $self->SelectProbeByPosition( end, @a_selectedProbeByScoring );
		push ( @a_selectedProbeByPosition, @a_endProbes[0..$nb-1] );
	}
	
	return @a_selectedProbeByPosition;
}

sub SelectionExemple
{
	my $self = shift;
	my $verbose = $self->_Get( '__verbose' );
	
	my $ra_validProbes = $self->ValidProbes();
	my @a_validProbes  = @$ra_validProbes;
	
	my @a_selectedProbe  = ();
	my $select = $self->_Get( '__select' );

	if ( defined $select )
	{
		my $totalProbeNumber = $self->_Get( '__totalProbeNumber' );
		my $score_threshold  = $self->_Get( '__scoreThreshold'   );
		@a_selectedProbe = $self->SelectProbes( $totalProbeNumber, $score_threshold );
		if ( defined $verbose ) { print "\nTest on selection of probe wth a low penalty score ($score_threshold), result $totalProbeNumber valid probes\n\tlist of probes id :\n"; }
		if ( defined $verbose ) { if ( ! @a_selectedProbe ) { print "none"; } }
		if ( defined $verbose ) { print Dumper @a_selectedProbe; }
	}
	else
	{
		if ( defined $verbose ) { print "\nTest on selection of probe wth a low penalty score (0.01)\n\tlist of probes id :\n"; }
		@a_selectedProbe = $self->SelectProbeByScoring( 0.01 );
		if ( defined $verbose ) { if ( ! @a_selectedProbe ) { print "none\n"; } }
		if ( defined $verbose ) { print Dumper @a_selectedProbe; }
		
		@a_selectedProbe = ();
		if ( defined $verbose ) { print "\nTest on selection of probe wth a low penalty score (0.04) and 2/2/2 position repartition\n\tlist of probes id :\n"; }
		@a_selectedProbe = $self->SelectProbes( '2/2/2', 0.04 );
		if ( defined $verbose ) { if ( ! @a_selectedProbe ) { print "none"; } }

		if ( defined $verbose ) { print Dumper @a_selectedProbe;}
	}
	
	$self->CreateNarcisseInput();
	return @a_selectedProbe;
}

###### OUTPUTS ######

=head2 function GetTempFileName

 Title        : GetTempFileName
 Usage        : $tempFileName = &GetTempFileName($process);
 Function     : Accessor, returns the Amplicon start position
 Access       : Public
 Returns      : $from
 Args         : none

=cut
sub GetOutputFileName
{
	my ( $processus, $ext ) = @_;
	
	unless ( defined $ext ) { $ext = '' ; }
	
	my $tempFileName = 'probeSelection_Output'.$processus.'.GF'.$ext;
	
	return $tempFileName;
}

=head2 function Display1

 Title        : Display1
 Usage        : $self->Display1();
 Function     : Returns a file wth all the valid Probes sequences.
 Access       : Public
 Returns      : the display1 file name
 Args         : none
 
=cut
sub Display1
{
	my ( $self ) = @_ ;
	
	my $file_name = &GetOutputFileName( 1 );
	
	my $ra_validProbes    = undef;
	my $probe_id          = undef;
	my $validId           = undef;
	my $probe_leftPrimer  = undef;
	my $probe_rightPrimer = undef;
	
	#write the inputFile with tags and values
	open( FILE, ">$file_name" ) || die "Display1 :\nErreur E/S: $file_name $!\n";
	
	$ra_validProbes = $self->ValidProbes();
	
	foreach my $validProbeId ( @$ra_validProbes )
	{
		my $o_probe        = $self->GetAProbe( $validProbeId );
		
		$probe_id          = $o_probe->Id();
		$validId           = $o_probe->ValidProbeId();
		$probe_leftPrimer  = $o_probe->LeftPrimer();
		$probe_rightPrimer = $o_probe->RightPrimer();
		
		print FILE $validId          , "\t";
		print FILE $probe_leftPrimer , "\t";
		print FILE $probe_rightPrimer, "\t\n";
	}
	
	close FILE;
	
	return $file_name;
}

=head2 function Display2

 Title        : Display2
 Usage        : $self->Display2();
 Function     : Returns a file wth all the valid Probes sequences and all the informations possibly needed.
 Access       : Public
 Returns      : the display2 file name
 Args         : none
 
=cut
sub Display2
{
	my ( $self ) = @_ ;
	
	my $file_name = &GetOutputFileName( 2 );
	
	#write the inputFile with tags and values
	open( FILE, ">$file_name" ) || die "Display1 :\nErreur E/S: $file_name $!\n";
	
	#Region informations
	my $species               = $self->Species();
	my $chr                   = $self->Chr();
	my $start                 = $self->Start();
	my $end                   = $self->End();
	
	print FILE "#####\tGLINT FREQ\t#####\n\n";
	print FILE "ø Species : $species\n";
	print FILE "ø Chr : $chr\tStart : $start\tEnd : $end\n\n";
	
	#Global results
	my $ra_validProbes = $self->ValidProbes();
	my $nb_validProbe  = scalar @$ra_validProbes ;
	my $ra_suitables   = $self->Suitables();
	my $nb_suitable    = scalar @$ra_suitables ;
	my $regionCoverage = $self->RegionCoverage();
	my $probeCoverage  = $self->ProbeCoverage();
	
	my $coverageRepStep       = $self->_Get( '__coverageRepStep' );
	my $region_representation = $self->_Get( '__representation' );
	
	print FILE "ø $nb_suitable suitables subregion\n";
	print FILE "ø $nb_validProbe valide probes\n";
	print FILE "ø Region coverage : $regionCoverage %\n";
	print FILE "ø Probe coverage  : $probeCoverage %\n";
	print FILE "____________________________________________\n\n";
	
	#Parameters
	my $count_cutoff       = $self->_Get( '__count_cutoff' );
	my $suitable_min_size  = $self->_Get( '__suitable_min_size' );
	my $total_probe_number = $self->_Get( '__totalProbeNumber' );
	my $score_threshold    = $self->_Get( '__scoreThreshold'   );
	my $rh_primer3_param   = $self->_Get( '__primer3_param' );
	my %h_primer3_param    = %$rh_primer3_param;
	my $product_size_range = $h_primer3_param{ PRIMER_PRODUCT_SIZE_RANGE };
	
	print FILE "# PARAMETERS USED\n";
	print FILE "*Count cutoff : $count_cutoff\n";
	print FILE "*Suitable subregion min size : $suitable_min_size\n";
	print FILE "*Probe product size range : $product_size_range\n";
	print FILE "*$total_probe_number probe to select\n";
	print FILE "*Penalty threshold on probe selection : $score_threshold\n";
	print FILE "____________________________________________\n\n";
	
	#Valid probes informations
	my $probe_id          = undef;
	my $validProbe_id     = undef;
	my $probe_from        = undef;
	my $probe_end         = undef;
	my $probe_size        = undef;
	my $probe_leftPrimer  = undef;
	my $probe_rightPrimer = undef;
	my $probe_penalty     = undef;
	my $probe_leftGc      = undef;
	my $probe_rightGc     = undef;
	my $probe_leftTm      = undef;
	my $probe_rightTm     = undef;
	my $probe_pcrProduct  = undef;
	my $probe_pos         = undef;
	
	foreach my $validProbeId ( @$ra_validProbes )
	{
		my $o_probe = $self->GetAProbe( $validProbeId );
		
		$probe_id          = $o_probe->Id()  ;
		$validProbe_id     = $o_probe->ValidProbeId() ;
		$probe_from        = $o_probe->From();
		$probe_end         = $o_probe->End() ;
		$probe_size        = $o_probe->Size();
		$probe_leftPrimer  = $o_probe->LeftPrimer()   ;
		$probe_rightPrimer = $o_probe->RightPrimer()  ;
		$probe_penalty     = $o_probe->Pair_Penalty() ;
		$probe_leftGc      = $o_probe->PrimerLeftGc() ;
		$probe_rightGc     = $o_probe->PrimerRightGc();
		$probe_leftTm      = $o_probe->PrimerLeftTm() ;
		$probe_rightTm     = $o_probe->PrimerRightTm();
		$probe_pcrProduct  = $o_probe->ProductPcrSeq();
		
		print FILE "Id : $validProbe_id\n";
		print FILE "From : $probe_from\tTo : $probe_end\tSize : $probe_size\n";
		print FILE "$probe_leftPrimer\t";
		print FILE "$probe_rightPrimer\n";
		print FILE "Pair Penalty : $probe_penalty\n";
		print FILE "Left gc% : $probe_leftGc\tRight gc% : $probe_rightGc\n";
		print FILE "Left tm  : $probe_leftTm\tRight tm  : $probe_rightTm\n";
		print FILE "PCR Product : $probe_pcrProduct\n\n";
		print FILE "Localisation on the region : \n";
		print FILE "$region_representation";
		
		$probe_pos = int( $probe_end / ( $coverageRepStep*1000 ) )-2;
		my $line = '' ;
		if ( $probe_pos > 0 ) { while ( $probe_pos-- ) { $line .= ' '; } }
		$line .= '^' ;
		print FILE $line;
		
		print FILE "\n====================================================\n\n";
	}
	
	close FILE;
	
	return $file_name; 
}


###### NARCISSE ######

=head2 CreateNarcisseInput

 Usage    : Create a file input for narcisse karyotype display
 Function : CreateNarcisseInput
 Arg      : $ra_selectedProbes reaference array t o the selected valid probe
 Return   : void

 Title        : CreateNarcisseInput
 Usage        : $self->CreateNarcisseInput( \@a_selectedProbes )
 Function     : Create a file corresponding to the csv data file format used as an input for Narcisse 
 Returns      : void
 Args         : $ra_selectedProbes reference array to selected valid probes.
 Globals      : none

=cut
sub CreateNarcisseInput
{
	my ( $self ) = @_;
	my $versus = $self->_Get( '__versus' );
	my $narcisseFile_name = &GetTempFileName( 'narcisse', 'in' );
	
	my $chr        = undef;
	my $color      = undef;
	my $probe_id   = undef;
	my $probe_end  = undef;
	my $probe_from = undef;
	my $probe_chr  = undef;
	my $rePcrVS    = undef;
	
	my @a_color = qw( black blue brown olive indigo yellowgreen red darkcyan gold green pink orange skyblue stateblue );
	
	$chr = $self->Chr();
	my $ra_validProbes = $self->ValidProbes();
	my @a_validProbes  = @$ra_validProbes;
	
	open( FILE, ">$narcisseFile_name" ) || die "Erreur E/S:$!\n";
	
	if ( defined $versus )
	{
		my $rao_versusProbes = $self->VersusProbes();
		my @ao_versusProbes  = @$rao_versusProbes;
		
		foreach my $o_versusProbe ( @ao_versusProbes )
		{
			$rePcrVS = $o_versusProbe->RePcrVS();
			if ( defined $rePcrVS   ) { $color = 'orange'; }
			if ( ! defined $rePcrVS ) { $color = 'red';    }
			$probe_id   = $o_versusProbe->Id();
			$probe_from = $o_versusProbe->From();
			$probe_end  = $o_versusProbe->End();
			$probe_chr  = $o_versusProbe->Chr();
			print FILE "$probe_chr\tvs$probe_id\t$probe_from\t$probe_end\t$color\n";
		}
	}
	
	$color = 'green';
	foreach $probe_id ( @a_validProbes )
	{
		if ( defined $probe_id )
		{
			my $o_probe = $self->GetAProbe( $probe_id );
			$probe_from = $o_probe->From();
			$probe_end  = $o_probe->End();
			print FILE "$chr\t$probe_id\t$probe_from\t$probe_end\t$color\n";
		}
	}
	close ( FILE );
}

###### Primer3 VS Mine ######

=head2 function GeneratePrimer3InputFile

 Title        : GeneratePrimer3InputFile
 Usage        : $region->GeneratePrimer3InputFile();
 Function     : Create the Primer3 input file using the region object
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub GeneratePrimer3InputFile
{
	my $self = shift;
	
	my $query_Filename = $self->_Get( '__query_filename' );
	my $input_Filename = &GetTempFileName( 'Primer3VSMine', 'in' );
	
	my $start = $self->Start();
	my $end   = $self->End();
	my $seq   = undef;
	
	#Get all the lines of the file
	open ( FILE, $query_Filename ) || die "\nCompare Primer3 Vs Mine :\nErreur E/S : $query_Filename $!\n";
		my $line  = '';
		while ( $line = <FILE> )
		{
			next if ( $line =~ /^>/ );
			chomp $line;
			$seq .= $line ;
		}
	close ( FILE );
	
	open( FILE, ">$input_Filename" ) || die "Erreur E/S:$!\n";
		print FILE "SEQUENCE_ID=Region |\tstart : $start\tend : $end\n";
		print FILE "SEQUENCE_TEMPLATE=$seq\n=\n";
	close ( FILE );
}

=head2 function LaunchPrimer3

 Title        : LaunchPrimer3
 Usage        : &LaunchPrimer3();
 Function     : Run Primer3 software
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub LaunchPrimer3
{
	my $self = shift;
	
	my $primer3_exe = $self->_Get( '__primer3_exe' );
	
	my $primerInputFile  = &GetTempFileName( 'Primer3VSMine', 'in' );
	my $primerOutputFile = &GetTempFileName( 'Primer3VSMine', 'out');
	
	my $rh_primer3_param = $self->_Get( '__primer3_param' );
	$self->GetPrimer3Parameters( $rh_primer3_param, 'versus' );
	my $primerParamFile  = &GetTempFileName( 'Primer3', 'param');
	
	my $command = $primer3_exe." -p3_settings_file $primerParamFile <$primerInputFile -output $primerOutputFile";
	$self->__LaunchShellcommand( $command ) ;
}

=head2 function LoadProbe

 Title        : LoadProbe
 Usage        : $subregion->LoadProbe();
 Function     : Parses the Primer3 output
                and Load Probes in the region object
 Access       : Public
 Returns      : none
 Args         : none
 
=cut
sub LoadVersusProbe
{
	my $self = shift;
	
	my $output_file = &GetTempFileName( 'Primer3VSMine', 'out' );

	#Get all the lines of the file
	open ( FILE, $output_file ) || die "\nPrimer3VSMine :\nErreur E/S : $output_file $!\n";
		my $all_lines = '';
		my $line      = undef;
		while( $line = <FILE> ) { $all_lines .= $line ; }
	close ( FILE );
	
	#Split the different sequences with '='
	my @a_line  = split ( /\n/, $all_lines ) ;
	
	my $id  = 0 ;
	my $region_start = $self->Start();
	
	for ( my $i = 6 ; $i < $#a_line ; $i += 20 ) 
	{
		my $pair_id             = undef ;
		my $pair_penalty        = undef ;
		my $primer_right_seq    = undef ;
		my $primer_left_seq     = undef ;
		my $from                = undef ;
		my $offset              = undef ;
		my $primer_left_length  = undef ;
		my $primer_right_start  = undef ;
		my $primer_right_length = undef ;
		my $primer_right_tm     = undef ;
		my $primer_left_tm      = undef ;
		my $primer_right_gc     = undef ;
		my $primer_left_gc      = undef ;
		my $size                = undef ;
		
		for ( my $j = $i ; $j <= $i + 19 ; $j++ )
		{
			if ( $a_line[$j] =~ /PRIMER_PAIR_(\d+)_PENALTY=(.*)/ )
			{
				$pair_id = $1 ;
				$pair_penalty = $2 ;
			}
			if ( $a_line[$j] =~ /PRIMER_LEFT_\d+_SEQUENCE=(.*)/  ) { $primer_left_seq = $1 ;  }
			if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+_SEQUENCE=(.*)/ ) { $primer_right_seq = $1 ; }
			if ( $a_line[$j] =~ /PRIMER_LEFT_\d+=(\d+),(\d+)/ )
			{
				$offset = $1 ;
				$primer_left_length = $2 ;
			}
			if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+=(\d+),(\d+)/ )
			{
				$primer_right_start = $1 ;
				$primer_right_length = $2 ;
			}
			if ( $a_line[$j] =~ /PRIMER_LEFT_\d+_TM=(.*)/   ) { $primer_left_tm= $1 ;   }
			if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+_TM=(.*)/ ) { $primer_right_tm = $1 ; }

			if ( $a_line[$j] =~ /PRIMER_RIGHT_\d+_GC_PERCENT=(.*)/  ) { $primer_right_gc = $1 ; }
			if ( $a_line[$j] =~ /PRIMER_LEFT_\d+_GC_PERCENT=(.*)/  ) { $primer_left_gc = $1 ;  }
			if ( $a_line[$j] =~ /PRIMER_PAIR_\d+_PRODUCT_SIZE=(.*)/ ) { $size = $1 ;            }
		}
		my $versusProbe_id = $pair_id;
		$from = $region_start + $offset;
		
		my $o_probe = New Probe($versusProbe_id, $primer_left_seq, $primer_right_seq, $offset, $size, $id, $from , $pair_penalty, $primer_left_length, $primer_right_start, $primer_right_length, $primer_left_tm, $primer_right_tm, $primer_left_gc, $primer_right_gc);
		$self->PushVersusProbe( $o_probe );
	}
}

sub RePcrPrimer3VsMine
{
	my ( $self ) = @_;
	my $species  = $self->Species();;
	
	# Get all the lines of the file
	my $rePcrVersusOutputFile = &GetTempFileName( 'RePcrVersus', 'out' );
	
	open ( FILE, $rePcrVersusOutputFile ) || die "\nCompare Primer3 Vs Mine :\nErreur E/S : $rePcrVersusOutputFile $!\n";
		my $lines = '';
		my $line  = '';
		while ( $line = <FILE> )
		{
			next if ( $line =~ /^#/ );
			$lines .= $line ;
		}
	close ( FILE );
	
	# Parses the glint output file
	my @a_lines = split ( /\n/, $lines );
	my @field_names = qw( probe_id chr strand from to mism gaps actLen expLen );
	
	# test the uniqueness 
	my %h_lines ;
	foreach my $line ( @a_lines )
	{
		if ($line =~ /^(\d+)\t(.*)\t([+,-])\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+).(\d+)-(\d+)/ )
		{
			my %h_line;
			@h_line{ @field_names } = ( $1, $2, $3, $4, $5, $6, $7, $8, $9 );
			
			my $versusProbe_id = $h_line{ probe_id };
			if ( !defined $h_lines{ $versusProbe_id } ) { $h_lines{ $versusProbe_id } = 1; }
			else { $h_lines{ $versusProbe_id } += 1; }
		}
	}
	foreach my $versusProbe_id ( sort keys %h_lines )
	{
		if ( $h_lines{ $versusProbe_id } != 1 )
		{
			my $o_versusProbe = $self->GetAVersusProbe( $versusProbe_id );
			my $versusProbe_start = $o_versusProbe->From();
			foreach my $line ( @a_lines )
			{
				if ( $line =~ /^($versusProbe_id)\t(.*)\t([+,-])\t(\d+)\t(\d+)\t(\d+)\t(\d+)\t(\d+).(\d+)-(\d+)/ )
				{
					my %h_line;
					@h_line{ @field_names } = ( $1, $2, $3, $4, $5, $6, $7, $8, $9 );

					if ( $versusProbe_start != $h_line{ from } )
					{
						my $ro_duplicatedProbe = $o_versusProbe->DuplicateProbe();
						$$ro_duplicatedProbe->SetRePcrVS();
						if ( $h_line{ chr } =~ /$species(.+)/) { $$ro_duplicatedProbe->SetChr( $1 ); }
						$$ro_duplicatedProbe->SetId  ( $versusProbe_id );
						$$ro_duplicatedProbe->SetFrom( $h_line{ from   } );
						$$ro_duplicatedProbe->SetEnd ( $h_line{ to     } );
						$$ro_duplicatedProbe->SetSize( $h_line{ actLen } );
						
						$self->PushVersusProbe( $$ro_duplicatedProbe );
					}
				}
			}
		}
	}
	
	print colored ( "\n# id\tvalidity\trePcr occ\tchr\tstart\t\tend\t\tsize\n", 'black bold' );
	
	my $rao_versusProbes = $self->VersusProbes();
	my @ao_versusProbes  = @$rao_versusProbes;
	foreach my $o_versusProbe ( @ao_versusProbes )
	{
		my $repcrVs = $o_versusProbe->RePcrVS();
		if ( defined $repcrVs )
		{
			my $versusProbe_id    = $o_versusProbe->Id();
			my $versusProbe_chr   = $o_versusProbe->Chr();
			my $versusProbe_start = $o_versusProbe->From();
			my $versusProbe_end   = $o_versusProbe->End();
			my $versusProbe_size  = $o_versusProbe->Size();
		
			print colored ( "  $versusProbe_id\t   0\t\t   $h_lines{ $versusProbe_id }\t\t $versusProbe_chr\t$versusProbe_start   \t$versusProbe_end   \t$versusProbe_size\n", 'yellow' );
		}
	}
}


sub GlintPrimer3VsMine
{
	my ( $self ) = @_;
	my $species  = $self->Species();;
	
	# Get all the lines of the file
	my $glintOutputFile = &GetTempFileName( 'glintVersus', 'out' );
	
	open ( FILE, $glintOutputFile ) || die "\nCompare Primer3 Vs Mine :\nErreur E/S : $glintOutputFile $!\n";
		my $lines = '';
		my $line  = '';
		while ( $line = <FILE> )
		{
			next if ( $line =~ /^#/ );
			$lines .= $line ;
		}
	close ( FILE );
	
	# Parses the glint output file
	my @a_lines = split ( /\n/, $lines );
	
	my @field_names = qw( q_name q_start q_end s_name s_start s_end strand score size );
	my %h_lines ;
	foreach my $line ( @a_lines )
	{
		if ( $line =~ /(\d+)\t(\d+)\t(\d+)\t(.+)\t(\d+)\t(\d+)\t([+,-])\t(\d+)\t(\d+)/ )
		{
			my %h_line;
			@h_line{ @field_names } = ( $1, $2, $3, $4, $5, $6, $7, $8, $9 );
			
			my $versusProbe_id = $h_line{ q_name };
			if ( !defined $h_lines{ $versusProbe_id } ) { $h_lines{ $versusProbe_id } = 1; }
			else { $h_lines{ $versusProbe_id } += 1; }
		}
	}
	
	foreach my $versusProbe_id ( sort keys %h_lines )
	{
		my $o_versusProbe = $self->GetAVersusProbe( $versusProbe_id );
		
		if ( $h_lines{ $versusProbe_id } == 1 )
		{
			foreach my $line ( @a_lines )
			{
				if ( $line =~ /($versusProbe_id)\t(\d+)\t(\d+)\t(.+)\t(\d+)\t(\d+)\t([+,-])\t(\d+)\t(\d+)/ )
				{
					my %h_line;
					@h_line{ @field_names } = ( $1, $2, $3, $4, $5, $6, $7, $8, $9 );
					
					if ( $h_line{ s_name } =~ /$species(.+)/) { $o_versusProbe->SetChr( $1 ); }
					$o_versusProbe->SetFrom( $h_line{ s_start } );
					$o_versusProbe->SetEnd ( $h_line{ s_end   } );
					$o_versusProbe->SetSize( $h_line{ size    } );
				}
			}
		}
		else
		{
			my $versus_cnt = 0;
			foreach my $line ( @a_lines )
			{
				if ( $line =~ /($versusProbe_id)\t(\d+)\t(\d+)\t(.+)\t(\d+)\t(\d+)\t([+,-])\t(\d+)\t(\d+)/ )
				{
					my %h_line;
					@h_line{ @field_names } = ( $1, $2, $3, $4, $5, $6, $7, $8, $9 );
					
					$versus_cnt++;
					if ( $versus_cnt == 1 )
					{
						if ( $h_line{ s_name } =~ /$species(.+)/) { $o_versusProbe->SetChr( $1 ); }
						$o_versusProbe->SetFrom( $h_line{ s_start } );
						$o_versusProbe->SetEnd ( $h_line{ s_end   } );
						$o_versusProbe->SetSize( $h_line{ size    } );
					}
					else
					{
						my $ro_duplicatedProbe = $o_versusProbe->DuplicateProbe();
						
						if ( $h_line{ s_name } =~ /$species(.+)/) { $$ro_duplicatedProbe->SetChr( $1 ); }
						$$ro_duplicatedProbe->SetId  ( "$versusProbe_id$versus_cnt" );
						$$ro_duplicatedProbe->SetFrom( $h_line{ s_start } );
						$$ro_duplicatedProbe->SetEnd ( $h_line{ s_end   } );
						$$ro_duplicatedProbe->SetSize( $h_line{ size    } );

						$self->PushVersusProbe( $$ro_duplicatedProbe );
					}
				}
			}
		}
	}
	
	# Sorting versus probes and printing
	print colored ( "\n# id\tvalidity\tglint occ\n", 'bold black' );
	
	my $o_versusProbe     = undef;
	my $versusProbe_start = undef;
	my $versusProbe_end   = undef;
	my $versusProbe_id    = undef;
	my $validProbe_id     = undef;
	my $probe_id          = undef;
	my $versusProbe_score = undef;
	my $versusProbe_size  = undef;
	my $leftPrimer        = undef;
	my $rightPrimer       = undef;
	
	my $cnt_glint = 0 ;
	my $cnt_valid = 0 ;
	my $cnt_overlap = 0 ;
	my $cnt = 0;
	my $cnt_unknown = 0 ;
	
	my $string = undef;
	my $ping   = undef;
	
	# Pcr product hybridization
	foreach my $versusProbe_id ( sort keys %h_lines )
	{
		if ( $h_lines{ $versusProbe_id } != 1 )
			{
				print colored ( "  $versusProbe_id\t   0\t\t   $h_lines{ $versusProbe_id }\n", 'red' );
				$cnt_glint++;
				delete $h_lines{ $versusProbe_id };
			}
	}
	print colored ( "-> $cnt_glint __\n", 'red' );
	
	# Valid
	print colored ( "\n# id\tvalidity\tglint occ\tvalidProbe_id\n", 'bold black' );
	foreach my $versusProbe_id ( sort keys %h_lines )
	{
		$o_versusProbe     = $self->GetAVersusProbe( $versusProbe_id );
		$versusProbe_start = $o_versusProbe->From();
		$versusProbe_end   = $o_versusProbe->End();
		
		$validProbe_id = $self->FindAValidProbe( ($versusProbe_start), ($versusProbe_end) );
		if ( ( defined $validProbe_id ) && ( $validProbe_id ne '' ) )
		{
			print colored ( "  $versusProbe_id\t   1\t\t   $h_lines{ $versusProbe_id }\t\t   $validProbe_id\n", 'green bold' );
			$cnt_valid++;
			delete $h_lines{ $versusProbe_id };
		}
	}
	print colored ( "-> $cnt_valid __\n", 'green bold' );
	
	#~ # Overlap
	#~ print colored ( "\n# id\tvalidity\tglint occ\tvalidProbe_id\n", 'bold black' );
	#~ my ($start, $end ) = 0;
	#~ foreach my $versusProbe_id ( sort keys %h_lines )
	#~ {
		#~ $o_versusProbe     = $self->GetAVersusProbe( $versusProbe_id );
		#~ $versusProbe_start = $o_versusProbe->From();
		#~ $versusProbe_end   = $o_versusProbe->End();
		#~ $versusProbe_size  = $o_versusProbe->Size();
		#~ 
		#~ if ( $start != 0 && ( ( $versusProbe_start == $start ) ||  ( $end == $versusProbe_end ) ) )
		#~ {
			#~ $ping++;
			#~ if ( $ping == 1 )
			#~ {
				#~ $string = "  ";
				#~ $string .= $versusProbe_id-1 ."\t   1\t\t   ".$h_lines{ $versusProbe_id-1 }."\t\t   ".$start."   \t".$end."      \t$versusProbe_size\toverlap\n";
				#~ print colored ( $string, 'bold' );
				#~ $cnt_overlap++;
				#~ delete $h_lines{ $versusProbe_id-1 };
			#~ }
			#~ print colored ( "  $versusProbe_id\t   1\t\t   $h_lines{ $versusProbe_id }\t\t   $versusProbe_start   \t$versusProbe_end      \t$versusProbe_size\toverlap\n", 'bold' );
			#~ $cnt_overlap++;
			#~ delete $h_lines{ $versusProbe_id };
		#~ }
		#~ 
		#~ else
		#~ {
			#~ $ping  = 0;
			#~ $start = $versusProbe_start;
			#~ $end   = $versusProbe_end;
		#~ }
	#~ }
	#~ print colored ( "-> $cnt_overlap __\n", 'bold' );
#~ 

}

sub ComparePrimer3VSMine
{
	my ( $self ) = @_;
	
	# get the probes from primer 3 directly
	$self->GeneratePrimer3InputFile();
	$self->LaunchPrimer3();
	$self->LoadVersusProbe();
	
	#Check primer matching uniqueness
	$self->GenerateRePcrInputFile( 'versus' );
	$self->LaunchRePcr( 'versus' );
	$self->RePcrPrimer3VsMine();
	
	#Check pcr product uniqueness
	$self->GenerateGlintInputFile( 'versus' );
	$self->LaunchGlint( 'versus' );
	$self->GlintPrimer3VsMine();

}


###### RUN ######

=head2 function Run

 Title        : Run
 Usage        : $region->Run( $bank, $queryFile, $printable );
 Function     : Set all Suitable subregion and Probes
 Access       : Public
 Returns      : none
 Args         : none

=cut
sub Run
{
	my $self = shift;
	
	my $verbose = $self->_Get( '__verbose' );
	
	my $sav_dir = getcwd();
	chdir( $self->_Get( '__targetDir' ) );
	
	# Glintfreq.cpp - Detect Suitable subregions
	if ( defined $verbose ) { print "\n\t1. Glintfreq.cpp - Detect Suitable subregions\n"; }
	$self->DetectSuitableSubregion() ;
	
	# Primer3 - Detect all probes
	if ( defined $verbose ) { print "\n\t2. Primer3       - Detect all probes\n"; }
	$self->DetectAllProbes() ;

	# Probes validation
	if ( defined $verbose ) { print "\n\t3. Glint / RePcr - Probes validation\n"; }
	$self->CheckProbes();
	$self->CheckPcrProduct();
	$self->LoadValidProbes();
	
	# Displays
	if ( defined $verbose ) { print "\n\t4. Displays\n"; }
	$self->_SetCoverage();
	$self->Display1();
	$self->Display2();
	
	# Versus
	my $versus = $self->_Get( '__versus' );
	if ( defined $versus ) 
	{ 
		if ( defined $verbose ) { print "\n\t5. Versus\n"; }
		$self->ComparePrimer3VSMine(); 
	}
	
	# Selection
	if ( defined $verbose ) { print "\n\t6. Selection\n"; }
	$self->SelectionExemple();
	
	# Erasing the tmp file part
	my $debug = $self->_Get( '__debug' );
	unless ( defined $debug ) { &CleanTempFileName(); }
	
	chdir( $sav_dir );
}

=head2 function __LaunchShellcommand

 Title        : __LaunchShellcommand
 Usage        : __LaunchShellcommand( $command );
 Function     : Set all Suitable subregion and Probes
 Access       : Public
 Returns      : none
 Args         : shel command

=cut
sub __LaunchShellcommand
{
	my ( $self, $command ) = @_;
	my $verbose = $self->_Get( '__verbose' );
	if ( defined $verbose ) { print colored ( "\n->$command\n\n", 'blue' ) ; }
	system( $command );
}

1;
