# 
# 	Sebastien.Letort@toulouse.inra.fr
#	Created: April 05, 2007
#	Last Updated: April 16, 2007
#

package Colors::Color;

=pod

=head1 NAME

Colors::Color - a class to easily manipulate RGB color

=head1 SYNOPSIS

	$o_blue = New Colors::Color(__blue => 255);
	$o_yellow = $o_blue->GetComplement();
	($red, $green, $blue) = $o_yellow->GetRGB();	#return (255,255,0)
	$o_black = $o_blue->Add($o_yellow);
=head1 DESCRIPTION

	If colors are used somewhere, you can use object colors to help dealing with them

=head1 AUTHOR

Letort Sebastien : Sebastien.Letort@toulouse.inra.fr

=cut

use warnings;
use strict;

#uniquement pour tester les nombres
use Scalar::Util;

use General;
use LipmError::ParameterException;

# 0 <= color < 256
use constant MIN	=>	0;
use constant MAX	=>	256;


BEGIN
{
    our $VERSION = do {my @r = (q$Rev$ =~ /\d+/g); $r[0]};
}

=head2 function New

	Title      :	New
	Usage      :	$o_color = New Color(%h_val)
	Prerequiste:	none
	Function   :	constructor of the object
	Returns    :	an object
	Args       :	%h_val, a hash which keys are
                		__blue, __red, __green
                		and which values are integers lesser than MAX(256) and greater than MIN(0)
	Globals    :	none

=cut

# sub New
# {
# 	my $class = shift;
# 	my $self = {};
# 	%$self = (
# 	#hash tab _ ce qu'on veut
# 		__blue => 0,
# 		__green => 0,
# 		__red => 0,
# 		@_
# 	);
# 	bless($self, $class);
# 
# 	return undef	unless($self->_AssertsAttributes());
# 
# 	return($self);
# }


sub _Init
{
	my $self = shift;
	$self->SUPER::_Init();
	%$self = (
	#hash tab _ ce qu'on veut
		__blue => 0,
		__green => 0,
		__red => 0,
		@_
	);

	$self->_AssertsAttributes();

	return;
}


=head2 function _AssertsAttributes

	Title      :	_AssertsAttributes
	Usage      :	$ok = $o_color->_AssertsAttributes()
	Prerequiste:	none
	Function   :	checks that all keys of the object are valids integers
                	which means MIN <= X < MAX
	Returns    :	TRUE if all keys are valid integers, FALSE otherwise
	Args       :	none
	Globals    :	none

=cut

sub _AssertsAttributes
{
	my $self = shift;
	my $are_numbers = &TRUE;

	foreach my $key ( keys(%$self) )
	{	$self->__IsValidNumber($key);	}

	return ;
}


=head2 function __IsValidNumber

	Title      :	__IsValidNumber
	Usage      :	$self->__IsValidNumber($key)
	Prerequiste:	none
	Function   :	checks that attribute $key is a valid integer
	Returns    :	nothing, throw ParameterException if NaN between MIN et MAX
	Args       :	$key, 	the name of the attribute
	Globals    :	none

=cut

sub __IsValidNumber
{
	my $self = shift;
	my $key = shift;
	my $num = $$self{$key};

	# if NaN
	if( !Scalar::Util::looks_like_number($num) )
	{	throw LipmError::ParameterException($key, 'type', -should_be => "an integer");	}

	if( ($num < &MIN) or (&MAX > $num) )
	{
		throw LipmError::ParameterException($key . '(' . $num . ')', 'value',
										-should_be => &MIN . "<= X < " . &MAX);
	}
}

=head2 Procedure Complement

	Title      :	Complement
	Usage      :	$o_color->Complement()
	Prerequiste:	none
	Procedure  :	Complement the color ie change (MAX - val) for RGB
	Returns    :	nothing
	Args       :	none
	Globals    :	none

=cut

sub Complement
{
	my $self = shift;

	$$self{__red} = &MAX-1 - $$self{__red};
	$$self{__green} = &MAX-1 - $$self{__green};
	$$self{__blue} = &MAX-1 - $$self{__blue};

	return;
}

=head2 Function GetComplement

	Title      :	GetComplement
	Usage      :	$o_complement = $o_color->GetComplement()
	Prerequiste:	none
	Function   :	return the complement to the color ie return (MAX - val) for RGB
	Returns    :	a new color that is a complement to the one calling
	Args       :	none
	Globals    :	none

=cut

sub GetComplement
{
	my $self = shift;

	my $red = &MAX-1 - $$self{__red};
	my $green = &MAX-1 - $$self{__green};
	my $blue = &MAX-1 - $$self{__blue};

	return New Colors::Color(__red=>$red, __green=>$green, __blue=>$blue);
}

=head2 Function GetRGB

	Title      :	GetRGB
	Usage      :	($red, $green, $blue) = $o_color->GetRGB()
	Prerequiste:	none
	Function   :	accessor to primaries colors
	Returns    :	an array with RGB values
	Args       :	none
	Globals    :	none

=cut

sub GetRGB
{
	my $self = shift;

	return ($$self{__red}, $$self{__green}, $$self{__blue});
}

=head2 Procedure Add

	Title      :	Add
	Usage      :	$o_black = $o_blue->Add($o_yellow);
	Prerequiste:	none
	Function   :	Add a color to another
	Returns    :	a color object, can throw an error if
	Args       :	$o_yellow, the color to add
	Globals    :	none

=cut

sub Add
{
	my $self = shift;
	my ($o_color) = @_;

	if ( !$o_color->isa("Colors::Color") )
	{
		throw LipmError::ParameterException($o_color, 'type',
										-should_be => "a color object",
										-text => "Colors::Color::Add = your first argument is not a color object.\n");
	}
	else
	{
		my ($r, $g, $b) = $o_color->GetRGB();
		my $red = $$self{__red} + $r;
		my $green = $$self{__green} + $g;
		my $blue = $$self{__blue} + $b;
		return New Colors::Color(__red=>$red, __green=>$green, __blue=>$blue);
	}

	return;
}


1;