# $Id: Pili.pm 57 2006-01-26 13:40:36Z manu $
#

package Pili;

# Copyright Notice

# Copyright I.N.R.A. - C.N.R.S. 

# emmanuel.courcelle@toulouse.inra.fr
# jerome.gouzy@toulouse.inra.fr

# This software is a perl module whose purpose is to help you writing 
# your own scripts

# This software is governed by the CeCILL  license under French law and
# abiding by the rules of distribution of free software.  You can  use, 
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# http://www.cecill.info" 

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability. 

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or 
# data to be ensured and,  more generally, to use and operate it in the 
# same conditions as regards security. 

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=head1 NAME
 Pili - An object to help you writing PIpe-LInes

 
=head1 DESCRIPTION

 Here is a very simple pipe-line:

 my $o_pili = New Pili;
 $o_pili->MainLopp();
 exit(0);
 
 sub step_1 {
     ...your code goes here ...
 }
 sub check_1 {
     ...not mandatory, BUT important to check everything is OK...
     if (OK)
     {
         return 1;
     }
     else
     {
         return 0;
     };
 }
 sub help_1 {
     ...not mandatory, BUT very king for your users...
 };
 
=head1 PUBLIC METHODS

 New
    my $o_pili = New Pili;
    my $o_pili = New Pili(-switches => ['--switch1=s','--switch2=i'...],
                          -config   => 'f1.cfg,f2.cfg,f3.cfg',
                          -log_level=> '2');
    my $o_pili = New Pili(-step     => ['sun','sdeux','strois'],
                          -check    => ['cun'],
                          -help     => ['hun','','htrois']);

 $o_pili->MainLoop();

 GetParamParser
    my $o_params = $o_pili->GetParamParser();
    
 GetLogger
    my $o_params = $o_pili->GetLogger();
    

=head1 METHODS AND SUBROUTINES - DETAILED DESCRIPTION

=cut

use strict;
use warnings;
use Carp;
use File::Basename;

use ParamParser;
use Logger;
use Sufine;

BEGIN
{
    our $VERSION = do {my @r = (q$Rev: 57 $ =~ /\d+/g); $r[0]};
}

=pod

=head2    New 

 Usage:     my $o_pili = New Pili(-switches => '--switch1=s,--switch2=i,...',
                                  -config   => 'f1.cfg,f2.cfg,f3.cfg',
                                  -step     => ['sun','sdeux','strois'],
                                  -check    => ['cun'],
                                  -help     => ['hun','','htrois']);
);
 Function:  Initialize the object, creating a ParamParser and a Logger object
 Return:    the created object
 Args:      -switches = an array of switches (passed by LONGOPT to ParamParser) (OPTIONAL)
            -config   = a string, with one a more configuration files to pass to the ParamParser object
            -step, -check, -help = Rename the Step_n, Check_n, Help_n subroutines
 
=cut

sub New {
    my $class = shift;
    my $self = bless {}, ref($class) || $class;
    my %args = @_;

    #
    # Create a ParamParser
    #
    
    my @param_switches = ('start=i','stop=i','log_level=s','help');
    if (exists $args{'-switches'})
    {
        my $suppl_switches = $args{'-switches'};
        push (@param_switches,split(',',$args{'-switches'}));
    };
    
    my @param_files = ();
    if (exists $args{'-config'}) {
        @param_files = split(',',$args{'-config'});
    };
    
    my $o_params = New ParamParser();
    $o_params->SetBehaviour('exit_on_getopt_error');
    $o_params->SetUsage('USAGE_DELAYED');
    $o_params->Set('Pili',$self);   # Keep a reference to the Pili object in ParamParser

    unshift (@param_switches, 'GETOPTLONG','A');
    $o_params->Update(@param_switches);

    # 
    # Update from the files, if necessary 
    # First file is read in Append mode, then 2nd, etc.
    #
    
    foreach my $f (@param_files) 
    {
        $o_params->Update($f, 'A') if (-e $f);
    };

    # We are only interested in paramters whose name starts with 'PILI_'
    
    $o_params->SelectNameSpace('PILI_');
    
    #
    # Build an array of references or names for the Step_n, Check_n, Help_n functions
    #
    #     The rules are as follows:
    #         1/ Use the data in the constructor parameter, if any
    #         2/ OR use the data in $o_param, if any
    #         3/ OR use the default data
    #
    
    my @functions=();
    my ($step,$check,$help);
    
    $step = $args{'-step'} if (exists $args{'-step'});    
    $check= $args{'-check'} if (exists $args{'-check'});    
    $help = $args{'-help'} if (exists $args{'-help'});    

    my $n=0;
    while(1)
    {
        my %f=();
        my $m = $n + 1;
        
        # Default names
        
        my $sub_step_name = "Step_$m";
        my $sub_check_name= "Check_$m";
        my $sub_help_name = "Help_$m";
        
        # Names taken from $o_params
        
        $sub_step_name    = $o_params->Get($sub_step_name)  if ($o_params->IsDefined($sub_step_name));
        $sub_check_name   = $o_params->Get($sub_check_name) if ($o_params->IsDefined($sub_check_name));
        $sub_help_name    = $o_params->Get($sub_help_name)  if ($o_params->IsDefined($sub_help_name));
        
        # Names taken from the parameters
        
        $sub_step_name    = $step->[$n]  if (defined  $step and defined $step->[$n]  and $step->[$n]  ne '');
        $sub_check_name   = $check->[$n] if (defined $check and defined $check->[$n] and $check->[$n] ne '');
        $sub_help_name    = $help->[$n]  if (defined $help  and defined $help->[$n]  and $help->[$n]  ne '');

        # Store the names
        
        $f{'step_name'}   = $sub_step_name;
        $f{'check_name'}  = $sub_check_name;
        $f{'help_name'}   = $sub_help_name;

        # Complete the names before referencing the subs
        
        $sub_step_name    = "main::$sub_step_name";
        $sub_check_name   = "main::$sub_check_name";
        $sub_help_name    = "main::$sub_help_name";

        # Search the subs
        
        my $sub_step      = \&$sub_step_name;
        my $sub_check     = \&$sub_check_name;
        my $sub_help      = \&$sub_help_name;
        
        # Store the references if possible
        
        last unless (defined(&$sub_step));  # If step not defined, exit from loop
        
        $f{'step_ref'}   = \&$sub_step;
        $f{'check_ref'}  = \&$sub_check if (defined(&$sub_check));
        $f{'help_ref'}   = \&$sub_help  if (defined(&$sub_help));
        
        # Keep the result
        
        push(@functions,\%f);
        $n++;
    };
    
    if ($n==0)
    {
        confess "PILI ERROR - You should define at least main::Step_1\n";
    };

    $self->{'__functions'} = \@functions;
    $o_params->SetUsage(\&Pili::__Usage);
    
   
    #
    # Croak with an error if $start,$stop not correct
    #

    my $max_step = $self->__CalcMaxStep();
    my ($start, $stop);

    if ($o_params->IsDefined('start')) {
        $start = $o_params->Get('start');
        if ($start !~ /^[\+\-]*\d+$/)      # If not a number, it is a function name
        {
            my $tmp = $start;
            $start = $self->__GetFunctionIndex($start);
            croak("PILI ERROR - Could not find $tmp") unless (defined $start);
            $o_params->Set('start',$start);
        }
    } else {
        $start = 1;
        $o_params->Set('start',1);
    };

    if ($o_params->IsDefined('stop')) {
        $stop = $o_params->Get('stop');
        if ($stop !~ /^[\+\-]*\d+$/)      # If not a number, it is a function name
        {
            my $tmp = $stop;
            $stop = $self->__GetFunctionIndex($stop);
            croak("PILI ERROR - Could not find $tmp") unless (defined $start);
            $o_params->Set('stop',$stop);
        }
     } else {
        $stop = $max_step;
        $o_params->Set('stop',$max_step);
    };
     
    croak("PILI ERROR - start ($start) must be >= 1") if ($start < 1);
    croak("PILI ERROR - start ($start) must be <= $max_step") if ($start > $max_step);
    croak("PILI ERROR - stop  ($stop)  must be >= start ($start)") if ($stop < $start);
    croak("PILI ERROR - stop  ($stop)  must be <= $max_step") if ($stop > $max_step);
    
    #
    # Create a Logger object
    #
    # N.B. The log level may be configured by a parameter, so we check log_level
    #      If a parameter is found, it takes precedence over the constructor parameter $log_level
    #      Pili logs its messages with a '.' log_type, so they are ALWAYS logged
    #
    
    my $method = basename($0);
    my $log_file = &Sufine::GetFileName(-name=>'>pili',-ext=>'log');
    my $log_level= '.';
    $log_level = $o_params->Get('log_level') if ($o_params->IsDefined('log_level'));
    my $o_log = New Logger(">$log_file",$log_level,'NOCOMPRESSION');
    my $msg = "PILI - Version $Pili::VERSION - Called from $method Version $main::VERSION\n";
    $o_log->Trace($msg, "*",'.');
    $msg = "PILI - $max_step steps found - start = $start - stop = $stop\n";
    $o_log->Trace($msg, "*",'.');

    $self->{'__o_params'} = $o_params;
    $self->{'__o_log'}    = $o_log;
    $self->{'__method'}   = $method;

    return $self;
};

=head2     MainLoop

 Usage:     my $o_pili->MainLoop();
 Procedure: Create a Logger object and a ParamParser object
            For n = $start .. $stop:
               Call &main::Step_n
               Call &main::Check_n if possible, and exit if this sub does not return 0
            
 Return:    Nothing
 Args:      None
 
=cut

sub MainLoop {
    my $self = shift;
    my $functions= $self->{'__functions'};
    my $o_params = $self->{'__o_params'};
    my $o_log    = $self->{'__o_log'};
    my $start = $o_params->Get('start');
    my $stop  = $o_params->Get('stop');

    
    #
    #  For each step:
    #          -log something
    #          -execute the main::step_$i sub
    #          -log something
    #          -execute the main::check_$i sub, if it exists
    #          -log the result of this check
    #          -Croak if this sub does not return 0
    #
    
    for (my $i=$start; $i <= $stop; $i++) 
    {
        my $f = $functions->[$i-1];
        my $step_name = $f->{'step_name'};
        my $step_ref  = $f->{'step_ref'};
        $o_log->Trace("PILI - NOW STARTING $step_name \n",'*','.');
        $o_log->StartTimer('STEP');
        
        &$step_ref($self);

        $o_log->Trace("PILI - NOW FINISHED $step_name\n",'STEP','.');
        
        if (exists $f->{'check_ref'})          # There is a check function, we execute it
        {
            my $check_name = $f->{'check_name'};
            my $check_ref  = $f->{'check_ref'};
            my $chk = &$check_ref($self);
            if ($chk==0)
            {
                $o_log->Trace("PILI - NOW CHECKING $step_name (using $check_name) - OK\n",'.','.');
            }
            else
            {
                $o_log->Trace("PILI - NOW CHECKING $step_name (using $check_name) - VALUE RETURNED $chk - STOPPING\n",'*','.');
                &croak("NOW CHECKING $step_name (using $check_name) - VALUE RETURNED $chk - STOPPING\n");
            }
        }
        else
        {
            $o_log->Trace("PILI - NO CHECK FUNCTION DEFINED FOR $step_name\n",'*','.');
        };
    };
    $o_log->Trace("PILI - Exit from MainLoop\n","*",".");
};
        
=head2     __GetFunctionIndex

 Usage:    my $n = $o_pili->__GetFunctionIndex('some_function')
 Access:   Private
 Function: Given the function name, return its index in the array of functions
           (the indexes start at 1)
 Args:     A function name, 
 Return:   The index, or undef if not found

=cut

sub __GetFunctionIndex 
{
    my $self = shift;
    my $fname = shift;
    my $functions = $self->{'__functions'};
    my $i = 0;
    my $found = 0;
    foreach my $f (@$functions)
    {
        $i++;
        if ($f->{'step_name'} eq $fname)
        {
            $found=1;
            last;
        };
    };
    return ($found==1) ? $i : undef;
};
    
=head2     __CalcMaxStep

 Usage:     my $n_max = $o_pili_>__CalcMaxStep();
 Access:    Private
 Function:  Return the number of Step_n functions
 Args:      None
 
=cut

sub  __CalcMaxStep {
    my $self = shift;
    my $functions = $self->{'__functions'};
    return $#$functions+1;
};

=pod

=head2 GetParamParser

 Usage:    my $o_params = $o_pili->GetParamParser()
 Access:   Public
 Function: Return the ParamParser object
 Args:     none
 Return:   The ParamParser object
 
=cut

sub GetParamParser {
    my $self = shift;
    return $self->{__o_params};
};

=pod

=head2 GetLogger

 Usage:    my $o_logger = $o_pili->GetLogger()
 Access:   Public
 Function: Return the Logger object
 Args:     none
 Return:   The Logger object
 
=cut

sub GetLogger {
    my $self = shift;
    return $self->{__o_log};
};

=pod

=head2 __Usage

 Usage:     Called when the --help switch is detected
 Access:    Private
 Procedure: Print a Help message, calling the Help_n functions, if they exist
 Args:      The ParamParser object
 Return:    DO NOT RETURN, Exit from the program
    
=cut

sub __Usage {
    my $o_param = shift;
    my $o_pili  = $o_param->Get('Pili');
    my $functions = $o_pili->{'__functions'};
    my $max_step = $o_pili->__CalcMaxStep();
    my $method = basename($0);

    print "Usage: $method [--start=n][--stop=p][--log_level][--help][--may_be_other_switch]\n";
    print "       start = n  Start at step n (default 1)\n";
    print "       stop  = n  Stop  at step p (default $max_step)\n";
    print "   log_level = XXX Default to ., but you may log to other log_levels if you want\n";

    for (my $i=1; $i <= $max_step; $i++) 
    {
        my $f = $functions->[$i-1];
        my $step_name = $f->{'step_name'};
        
        print "\n$step_name ***********************************************\n";
        if (exists $f->{'help_ref'})     # OK - There is a help function: Execute it
        {
            my $help_ref  = $f->{'help_ref'};
            &$help_ref();
        }
        else
        {
            print "No help function defined for $step_name\n";
        };
    };
    exit;
};

1;

