#!/usr/bin/env perl

package iANT::DB::Fasta;

use strict;
use warnings;
use Carp;
use XML::Twig;

use iANT::Entry;
use iANT::Entry::Sequence;
use iANT::Entry::Definition;
use iANT::Entry::Link;

=head1 NAME

iANT::DB::Fasta - Object


=head1 SYNOPSIS

    use iANT::DB::Fasta;
    my $o_db_feature = New iANT::DB::Fasta('./db_feature.xml');



=head1 DESCRIPTION

    iANT::DB::Fasta is designed to make it reasonably easy to create, consume or modify a db of fasta using the XML syntax:
    




=cut



=head1 PUBLIC METHODS


=pod 


=head3 function New

 Title        : New
 Usage        : New iANT::DB::Fasta;
 Prerequisite : none
 Function     : Create an object iANT::DB::Fasta
 Returns      : An object iANT::DB::Fasta
 Args         : none
 Globals      : none

=cut

BEGIN
{
    our $VERSION = "0.1";

}

sub New
{
    my ($class, $xml_source) = (@_);

    my $self = {
                __fasta     => "",
                __directory => ''
                };

    if (defined $xml_source)
    {
        $self = &Update($self, $xml_source);
    }
    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_property->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing iANT::DB::Feature object
 Returns      : An object iANT::DB::Feature
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_twig_db_feature = XML::Twig->new();

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        $o_twig_db_feature->parsefile($source);
        $self = &__InitializeFromXml($self, $o_twig_db_feature);
    }
    else    #raw XML data
    {
        $o_twig_db_feature->parse($source);
        $self = &__InitializeFromXml($self, $o_twig_db_feature);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_xml_feature->GetXml();
 Prerequisite : none
 Function     : Retrieve a Feature in xml format
 Returns      : An xml string
 Args         : none
 Globals      : none

=cut

sub GetXml
{
    my ($self) = (@_);

    my $db_feature_xml_header = '<entry_features accession="' . $self->GetAccession() . '">';

    my $db_feature_xml_body = '';

    my $ra_o_feature = $self->GetFeatures();
    if (defined $ra_o_feature && ($ra_o_feature ne ''))
    {
        foreach my $o_feature (@{$ra_o_feature})
        {
            $db_feature_xml_body .= $o_feature->GetXml();
        }
    }

    my $o_twig_db_feature = XML::Twig->new();

    my $db_feature_xml_footer = '</entry_features>';

    return $db_feature_xml_header . $db_feature_xml_body . $db_feature_xml_footer;
}

=pod

=head3 procedure DeleteFeatures

 Title        : DeleteFeatures
 Usage        : $o_db_feature->DeleteFeatures($rh_select_params);
 Prerequisite : none
 Function     : Delete features from iANT::DB::Feature object according to
                a list of selection parameters
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub DeleteFeatures
{
    my ($self, $rh_selection_parameters) = (@_);

    my $ra_o_features           = $self->GetFeatures();
    my @a_o_notdeleted_features = ();
    if (defined $ra_o_features && ($ra_o_features ne '') && (ref($ra_o_features) =~ /array/i))
    {
        foreach my $o_feature (@{$ra_o_features})
        {
            my $match_selection_parameters = 0;

            foreach my $selection_parameter (keys %{$rh_selection_parameters})
            {
                if (ref($rh_selection_parameters->{$selection_parameter}) =~ /hash/i)
                {
                    my $o_parameter_element = $o_feature->Get('o_' . $selection_parameter);

                    foreach my $element_selection_parameter (keys %{$rh_selection_parameters->{$selection_parameter}})
                    {
                        if ($o_parameter_element->Get($element_selection_parameter) eq
                            $rh_selection_parameters->{$selection_parameter}->{$element_selection_parameter})
                        {
                            $match_selection_parameters = 1;
                        }
                        else
                        {
                            $match_selection_parameters = 0;
                            last;
                        }
                    }
                    if (!$match_selection_parameters)
                    {
                        last;
                    }
                }
                else
                {
                    if ($o_feature->Get($selection_parameter) eq $rh_selection_parameters->{$selection_parameter})
                    {
                        $match_selection_parameters = 1;
                    }
                    else
                    {
                        $match_selection_parameters = 0;
                        last;
                    }
                }
            }

            if (!$match_selection_parameters)
            {
                push(@a_o_notdeleted_features, $o_feature);
            }
        }

        $self->{__ra_features} = \@a_o_notdeleted_features;
    }

}

=pod

=head3 procedure SelectFeatures

 Title        : SelectFeatures
 Usage        : $o_db_feature->SelectFeatures($rh_select_params);
 Prerequisite : none
 Function     : Select features from iANT::DB::Feature object according to
                a list of selection parameters
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub SelectFeatures
{
    my ($self, $rh_selection_parameters) = (@_);

    my $ra_o_features         = $self->GetFeatures();
    my @a_o_selected_features = ();
    if (defined $ra_o_features && ($ra_o_features ne '') && (ref($ra_o_features) =~ /array/i))
    {
        foreach my $o_feature (@{$ra_o_features})
        {
            my $match_selection_parameters = 0;

            foreach my $selection_parameter (keys %{$rh_selection_parameters})
            {
                if (ref($rh_selection_parameters->{$selection_parameter}) =~ /hash/i)
                {
                    my $o_parameter_element = $o_feature->Get('o_' . $selection_parameter);

                    foreach my $element_selection_parameter (keys %{$rh_selection_parameters->{$selection_parameter}})
                    {
                        if ($o_parameter_element->Get($element_selection_parameter) eq
                            $rh_selection_parameters->{$selection_parameter}->{$element_selection_parameter})
                        {
                            $match_selection_parameters = 1;
                        }
                        else
                        {
                            $match_selection_parameters = 0;
                            last;
                        }
                    }
                    if (!$match_selection_parameters)
                    {
                        last;
                    }
                }
                else
                {
                    if ($o_feature->Get($selection_parameter) eq $rh_selection_parameters->{$selection_parameter})
                    {
                        $match_selection_parameters = 1;
                    }
                    else
                    {
                        $match_selection_parameters = 0;
                        last;
                    }
                }
            }

            if ($match_selection_parameters)
            {
                push(@a_o_selected_features, $o_feature);
            }
        }

        $self->{__ra_features} = \@a_o_selected_features;
    }

}

=pod

=head3 procedure AddFeatures

 Title        : AddFeatures
 Usage        : $o_db_feature->AddFeatures($ra_o_feature);
 Prerequisite : none
 Function     : Add features to iANT::DB::Feature object
 Returns      : An xml string
 Args         : a single iANT::Feature object or a List Ref of such objects
 Globals      : none

=cut

sub AddFeatures
{
    my ($self, $ra_o_feature) = (@_);

    print ref($ra_o_feature);

    if (defined $ra_o_feature && ($ra_o_feature ne '') && (ref($ra_o_feature) =~ /array/i))
    {
        foreach my $o_feature (@{$ra_o_feature})
        {
            $self->SetFeature($o_feature);
        }
    }
    elsif (defined $ra_o_feature && ($ra_o_feature ne '') && (ref($ra_o_feature) =~ /iANT::Feature/i))
    {
        my $o_feature = $ra_o_feature;
        $self->SetFeature($o_feature);
    }
    else
    {
        confess("Bad input type for AddFeatures methode: $ra_o_feature");
    }

}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $xml_db_feature->Dump();
 Prerequisite : none
 Function     : Dump a DB::Feature in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    my $print_file = 0;
    if ((defined $target) && ($target ne ''))
    {
        if ($target !~ /^>/)
        {
            croak("\n\nYou must define the opening file mode: \">\" for overwrite or \">>\" for append mode\n\n");
        }
        else
        {
            open(OUT, "$target") || confess("$!:$target");
            select OUT;
            $print_file = 1;
        }
    }

    my $o_twig = XML::Twig->new();
    $o_twig->set_pretty_print('indented');
    $o_twig->parse($self->GetXml());
    print $o_twig->sprint();

    if ($print_file)
    {
        select STDOUT;
        close OUT;
    }
    return;
}

=pod

=head3 procedure BuildEntries

 Title        : Dump
 Usage        : $xml_db_fasta->BuildEntries();
 Prerequisite : none
 Function     : Build Entries XML files and directories
 Returns      : none
 Args         : 
 Globals      : none

=cut

sub BuildEntries
{
    my ($self) = (@_);

    my $fasta = $self->Get('fasta');
    confess("BuildEntries can not be executed unless iANT::DB::Fasta 'fasta' attribute is undefined")
      if ((!defined $fasta) || ($fasta eq ''));

    __BuildEntryObjects($fasta);

}

sub __BuildEntryObjects
{
    my $fasta = shift();

    my @a_fasta_lines = ();
    if (($fasta !~ /\n/) && -e $fasta)
    {
        open(FASTA, $fasta) || confess($!);
        while (my $line = <FASTA>)
        {
            chomp $line;
            push(@a_fasta_lines, $line);
        }
        close FASTA;
    }
    else
    {
        @a_fasta_lines = split(/\n/, $fasta);
    }
    my $sequence         = '';
    my $rh_sequence_info = '';

    foreach my $fasta_line (@a_fasta_lines)
    {

        if ($fasta_line =~ /^>/)
        {
            chomp $fasta_line;
            if ($sequence ne '')
            {

                #build iANT::Entry
                my $o_entry = New iANT::Entry();

                if (defined $rh_sequence_info && (ref($rh_sequence_info) =~ /hash/i))
                {
                    if (ref($rh_sequence_info->{ra_definitions}) =~ /array/i)
                    {
                        foreach my $rh_definition (@{$rh_sequence_info->{ra_definitions}})
                        {
                            my $o_entry_definition = New iANT::Entry::Definition($rh_definition);
                            $o_entry->SetDefinition($o_entry_definition);
                        }
                        delete $rh_sequence_info->{ra_definitions};
                    }

                    if (ref($rh_sequence_info->{ra_links}) =~ /array/i)
                    {
                        foreach my $rh_link (@{$rh_sequence_info->{ra_links}})
                        {
                            my $o_entry_link = New iANT::Entry::Link($rh_link);
                            $o_entry->SetLink($o_entry_link);
                        }
                        delete $rh_sequence_info->{ra_links};
                    }
                    $rh_sequence_info->{species} = '' if (!defined $rh_sequence_info->{species});
                    my $o_organism_species = New iANT::Organism::Species({code => $rh_sequence_info->{species}});
                    delete $rh_sequence_info->{species};
                    my $o_organism_taxonomy = New iANT::Organism::Taxonomy();
                    $rh_sequence_info->{organelle} = '' if (!defined $rh_sequence_info->{organelle});
                    delete $rh_sequence_info->{organelle};
                    my $o_organism_organelle =
                      New iANT::Organism::Organelle({content => $rh_sequence_info->{organelle}});

                    my $o_organism =
                      New iANT::Organism(
                                         {
                                          o_species   => $o_organism_species,
                                          o_taxonomy  => $o_organism_taxonomy,
                                          o_organelle => $o_organism_organelle
                                         }
                                         );
                    $o_entry->SetOrganism($o_organism);
                    my $o_entry_sequence = New iANT::Entry::Sequence({content => $sequence});
                    $o_entry->SetSequence($o_entry_sequence);
                    $o_entry->Update($rh_sequence_info);
                    $o_entry->Dump();
                }
            }
            $rh_sequence_info = __ParseFastaHeader($fasta_line);
        }
        else
        {
            $sequence .= $fasta_line;
        }
    }
    my $o_entry = New iANT::Entry();

    if (defined $rh_sequence_info && (ref($rh_sequence_info) =~ /hash/i))
    {
        if (ref($rh_sequence_info->{ra_definitions}) =~ /array/i)
        {
            foreach my $rh_definition (@{$rh_sequence_info->{ra_definitions}})
            {
                my $o_entry_definition = New iANT::Entry::Definition($rh_definition);
                $o_entry->SetDefinition($o_entry_definition);
            }
            delete $rh_sequence_info->{ra_definitions};
        }

        if (ref($rh_sequence_info->{ra_links}) =~ /array/i)
        {
            foreach my $rh_link (@{$rh_sequence_info->{ra_links}})
            {
                my $o_entry_link = New iANT::Entry::Link($rh_link);
                $o_entry->SetLink($o_entry_link);
            }
            delete $rh_sequence_info->{ra_links};
        }
        $rh_sequence_info->{species} = '' if (!defined $rh_sequence_info->{species});
        my $o_organism_species = New iANT::Organism::Species({code => $rh_sequence_info->{species}});
        delete $rh_sequence_info->{species};
        my $o_organism_taxonomy = New iANT::Organism::Taxonomy();
        $rh_sequence_info->{organelle} = '' if (!defined $rh_sequence_info->{organelle});
        delete $rh_sequence_info->{organelle};
        my $o_organism_organelle = New iANT::Organism::Organelle({content => $rh_sequence_info->{organelle}});

        my $o_organism =
          New iANT::Organism(
                             {
                              o_species   => $o_organism_species,
                              o_taxonomy  => $o_organism_taxonomy,
                              o_organelle => $o_organism_organelle
                             }
                             );
        $o_entry->SetOrganism($o_organism);
        my $o_entry_sequence = New iANT::Entry::Sequence({content => $sequence});
        $o_entry->SetSequence($o_entry_sequence);
        $o_entry->Update($rh_sequence_info);
        $o_entry->Dump();
    }
}

sub __ParseFastaHeader
{
    my $fasta_header    = shift;
    my %h_sequence_info = ();
    my @a_definitions   = ();
    my @a_links         = ();
    my ($id) = ($fasta_header =~ /^>(\S+)/);
    $fasta_header =~ s/^>(\S+)//;
    $h_sequence_info{id}        = $id;
    $h_sequence_info{accession} = $id;

    if ($fasta_header =~ / def=(.+)/)
    {
        push(@a_definitions, {ontology => 'NONE', id => 'product', content => "$1"});
        $fasta_header =~ s/ def=(.+)//;
    }
    while ($fasta_header =~ /(\S+)=(\S+)/g)
    {

        #>id acc=sp:P15711 len=87 chr=1 sp=code_prj k=[EBVA] pm=1689460;1689461 type=gene|cds|protein def=description
        $h_sequence_info{accession} = $2 if ($1 eq 'acc');
        $h_sequence_info{length}    = $2 if ($1 eq 'len');
        $h_sequence_info{type}      = $2 if ($1 eq 'type');
        $h_sequence_info{species}   = $2 if ($1 eq 'sp');
        if ($1 eq 'pm')
        {
            my @a_pmid = split(/;/, $2);
            foreach my $pmid (@a_pmid)
            {
                push(@a_definitions, {ontology => 'pubmed', id => $pmid});
            }
        }
        push(@a_links, {type => 'partOf', accession => "$1$2"}) if ($1 eq 'chr');
        $h_sequence_info{organelle} = "$1$2" if ($1 eq 'chr');

    }
    $h_sequence_info{ra_definitions} = \@a_definitions;
    $h_sequence_info{ra_links}       = \@a_links;
    return \%h_sequence_info;
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);

}

=item GetFasta

    Returns      : DB::Fasta fasta

=cut

sub GetFasta
{
    my ($self) = (@_);
    $self->Get('fasta');
}

=item SetFasta

    Set DB::Fasta fasta

=cut

sub SetFasta
{
    my ($self, $fasta) = (@_);
    $self->Set('fasta', $fasta);
}

=pod
 


=head1 PRIVATE METHODS

=cut

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_feature,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing feature object from a hash (attr => value, ...)
 Returns      : An object iANT::Feature
 Args         : $o_feature
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    $self->{__fasta}     = $rh_attributes->{fasta}     if (defined $rh_attributes->{fasta});
    $self->{__directory} = $rh_attributes->{directory} if (defined $rh_attributes->{directory});

    return $self;
}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_db_feature,$o_twig_db_feature);
 Prerequisite : none
 Function     : Fill attributes of an existing db feature object from a XML::Twig object
 Returns      : An object iANT::DB::Feature
 Args         : $o_db_feature
                $o_twig_db_feature
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_twig_db_feature) = (@_);
    my $entry_features_elt = $o_twig_db_feature->root;

    $self->{__accession} = $entry_features_elt->{'att'}->{'accession'}
      if defined $entry_features_elt->{'att'}->{'accession'};

    my @a_feature_elts = $entry_features_elt->children('feature');
    my @a_o_features   = ();
    foreach my $feature_elt (@a_feature_elts)
    {
        my $o_feature = New iANT::Feature($feature_elt->sprint);
        push(@a_o_features, $o_feature);
    }

    $self->{__ra_features} = \@a_o_features;

    return $self;
}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Feature

=cut

1;
