#!/usr/bin/env perl

package iANT::DB::Feature;

use strict;
use warnings;
use Carp;
use XML::Twig;

use iANT::Common;
use iANT::Feature;
use iANT::Pattern;

=head1 NAME

iANT::DB::Feature - Object to deal with Entry's features pooled into one XML file


=head1 SYNOPSIS

    use iANT::DB::Feature;
    my $o_db_feature = New iANT::DB::Feature('./db_feature.xml');

    print "Acc   ",$o_db_feature->GetAccession(),"\n";
    foreach my $o_feature (@{$o_db_feature->GetFeatures()})
    {
        print $o_feature->GetDate, "\t";
        print $o_feature->GetSoftware->GetName, "\t";
        print $o_feature->GetDatabase->GetName, "\n";
    }

    $o_db_feature->DeleteFeatures({software => { name => 'BLASTP'}, database => { name => 'sptr'}});

    print "After deletion\n";
    foreach my $o_feature (@{$o_db_feature->GetFeatures()})
    {
        print $o_feature->GetDate, "\t";
        print $o_feature->GetSoftware->GetName, "\t";
        print $o_feature->GetDatabase->GetName, "\n";
    }

    $o_db_feature = New iANT::DB::Feature('./db_feature.xml');
    $o_db_feature->SelectFeatures({software => { name => 'BLASTP'}, database => { name => 'sptr'}});

    print "After selection\n";
    foreach my $o_feature (@{$o_db_feature->GetFeatures()})
    {
        print $o_feature->GetDate, "\t";
        print $o_feature->GetSoftware->GetName, "\t";
        print $o_feature->GetDatabase->GetName, "\n";
    }

    $o_db_feature = New iANT::DB::Feature('./db_feature.xml');
    $o_db_feature->AddFeatures($o_db_feature->GetFeatures());
    print "After add\n";

    foreach my $o_feature (@{$o_db_feature->GetFeatures()})
    {
        print $o_feature->GetDate, "\t";
        print $o_feature->GetSoftware->GetName, "\t";
        print $o_feature->GetDatabase->GetName, "\n";
    }
    
    $o_db_feature = New iANT::DB::Feature('./db_feature.xml');
    $o_db_feature->AddPatterns($o_db_feature->GetPatterns());
    $o_db_feature->DeletePatterns({type => 'transcriptomic_\w+'});
    print "XML:\n";
    $o_db_feature->Dump();



=head1 DESCRIPTION

    iANT::DB::Feature is designed to make it reasonably easy to create, consume or modify a db of features using the XML syntax:
    <entry_features>
		<feature type="computed" path=".../AL646057/FT/RS02279_AA/RS02279_AA.raw.prodom" date="20060314">
			<database name="prodom" version="2005.1" comment=""/>
			<sequence id="RSc0092" accession="RS02279_AA" length="113" definition="CONSERVED HYPOTHETICAL PROTEIN" begin="22" end="95"/>
			<hit id="PD000001" accession="PD000001" definition="domaine super" length="113" begin="22" end="95">
				<property id="score" value="274"/>
				<property id="e-value" value="2e-25"/>
			</hit>
    	       </feature>
		<pattern type="transcriptomic_array" definition="mutant1 vs wt">-0.35</pattern>
		<pattern type="transcriptomic_array" definition="wt vs mutant2">+12</pattern>
		<pattern type="transcriptomic_est" definition="root">120</pattern>
		<pattern type="transcriptomic_est" definition="leaf">23</pattern>
		<pattern type="proteomic"></pattern>
	</entry_features>



=cut



=head1 PUBLIC METHODS


=pod 


=head3 function New

 Title        : New
 Usage        : New iANT::DB::Feature;
 Prerequisite : none
 Function     : Create an object iANT::DB::Feature
 Returns      : An object iANT::DB::Feature
 Args         : none
 Globals      : none

=cut

BEGIN
{
    our $VERSION = "0.1";

}

sub New
{
    my ($class, $xml_source) = (@_);

    my $self = {
                __accession   => "",
                __ra_features => "",
                __ra_patterns => ""
                };

    if (defined $xml_source)
    {
        $self = &Update($self, $xml_source);
    }
    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_property->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing iANT::DB::Feature object
 Returns      : An object iANT::DB::Feature
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_twig_db_feature = XML::Twig->new();

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        $o_twig_db_feature->parsefile($source);
        $self = &__InitializeFromXml($self, $o_twig_db_feature);
    }
    else    #raw XML data
    {
        $o_twig_db_feature->parse($source);
        $self = &__InitializeFromXml($self, $o_twig_db_feature);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_xml_feature->GetXml();
 Prerequisite : none
 Function     : Retrieve a Feature in xml format
 Returns      : An xml string
 Args         : none
 Globals      : none

=cut

sub GetXml
{
    my ($self) = (@_);

    my $db_feature_xml_header = '<entry_features accession="' . $self->GetAccession() . '">';

    my $db_feature_xml_body = '';

    my $ra_o_feature = $self->GetFeatures();
    if (defined $ra_o_feature && (ref($ra_o_feature) =~ /array/i) && ($ra_o_feature ne ''))
    {
        foreach my $o_feature (@{$ra_o_feature})
        {
            $db_feature_xml_body .= $o_feature->GetXml();
        }
    }
    my $ra_o_patterns = $self->GetPatterns();
    if (defined $ra_o_patterns && (ref($ra_o_patterns) =~ /array/i) && ($ra_o_patterns ne ''))
    {
        foreach my $o_pattern (@{$ra_o_patterns})
        {
            $db_feature_xml_body .= $o_pattern->GetXml();
        }
    }

    my $o_twig_db_feature = XML::Twig->new();

    my $db_feature_xml_footer = '</entry_features>';

    return $db_feature_xml_header . $db_feature_xml_body . $db_feature_xml_footer;
}

=pod

=head3 procedure DeleteFeatures

 Title        : DeleteFeatures
 Usage        : $o_db_feature->DeleteFeatures($rh_select_params);
 Prerequisite : none
 Function     : Delete features from iANT::DB::Feature object according to
                a list of selection parameters
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub DeleteFeatures
{
    my ($self, $rh_selection_parameters) = (@_);
    &iANT::Common::Backup($self);
    my $ra_o_features           = $self->GetFeatures();
    my @a_o_notdeleted_features = ();
    if (defined $ra_o_features && ($ra_o_features ne '') && (ref($ra_o_features) =~ /array/i))
    {
        foreach my $o_feature (@{$ra_o_features})
        {
            my $match_selection_parameters =
              iANT::Common::IsMatchingSelectionParameter($o_feature, $rh_selection_parameters);
            if (!$match_selection_parameters)
            {
                push(@a_o_notdeleted_features, $o_feature);
            }
        }

        $self->{__ra_features} = \@a_o_notdeleted_features;
    }

}

=pod

=head3 procedure SelectFeatures

 Title        : SelectFeatures
 Usage        : $o_db_feature->SelectFeatures($rh_select_params);
 Prerequisite : none
 Function     : Select features from iANT::DB::Feature object according to
                a list of selection parameters
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub SelectFeatures
{
    my ($self, $rh_selection_parameters) = (@_);
    &iANT::Common::Backup($self);
    my $ra_o_features         = $self->GetFeatures();
    my @a_o_selected_features = ();
    if (defined $ra_o_features && ($ra_o_features ne '') && (ref($ra_o_features) =~ /array/i))
    {
        foreach my $o_feature (@{$ra_o_features})
        {
            my $match_selection_parameters =
              iANT::Common::IsMatchingSelectionParameter($o_feature, $rh_selection_parameters);
            if ($match_selection_parameters)
            {
                push(@a_o_selected_features, $o_feature);
            }
        }

        $self->{__ra_features} = \@a_o_selected_features;
    }

}

=pod

=head3 procedure AddFeatures

 Title        : AddFeatures
 Usage        : $o_db_feature->AddFeatures($ra_o_feature);
 Prerequisite : none
 Function     : Add features to iANT::DB::Feature object
 Returns      : An xml string
 Args         : a single iANT::Feature object or a List Ref of such objects
 Globals      : none

=cut

sub AddFeatures
{
    my ($self, $ra_o_feature) = (@_);
    &iANT::Common::Backup($self);
    if (defined $ra_o_feature && ($ra_o_feature ne '') && (ref($ra_o_feature) =~ /array/i))
    {
        foreach my $o_feature (@{$ra_o_feature})
        {
            $self->SetFeature($o_feature);
        }
    }
    elsif (defined $ra_o_feature && ($ra_o_feature ne '') && (ref($ra_o_feature) =~ /iANT::Feature/i))
    {
        my $o_feature = $ra_o_feature;
        $self->SetFeature($o_feature);
    }
    else
    {
        confess("Bad input type for AddFeatures methode: $ra_o_feature");
    }

}

=pod

=head3 procedure DeletePatterns

 Title        : DeletePatterns
 Usage        : $o_db_feature->DeletePatterns($rh_select_params);
 Prerequisite : none
 Function     : Delete patterns from iANT::DB::Feature object according to
                a list of selection parameters
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub DeletePatterns
{
    my ($self, $rh_selection_parameters) = (@_);

    if (!defined $self->{__o_backup})
    {
        &iANT::Common::Backup($self);
    }

    my $ra_o_patterns           = $self->GetPatterns();
    my @a_o_notdeleted_patterns = ();
    if (defined $ra_o_patterns && ($ra_o_patterns ne '') && (ref($ra_o_patterns) =~ /array/i))
    {
        foreach my $o_pattern (@{$ra_o_patterns})
        {
            my $match_selection_parameters = 0;

            foreach my $selection_parameter (keys %{$rh_selection_parameters})
            {
                if (ref($rh_selection_parameters->{$selection_parameter}) =~ /hash/i)
                {
                    my $o_parameter_element = $o_pattern->Get('o_' . $selection_parameter);

                    foreach my $element_selection_parameter (keys %{$rh_selection_parameters->{$selection_parameter}})
                    {
                        if ($o_parameter_element->Get($element_selection_parameter) =~
                            /^$rh_selection_parameters->{$selection_parameter}->{$element_selection_parameter}$/)
                        {
                            $match_selection_parameters = 1;
                        }
                        else
                        {
                            $match_selection_parameters = 0;
                            last;
                        }
                    }
                    if (!$match_selection_parameters)
                    {
                        last;
                    }
                }
                else
                {
                    if ($o_pattern->Get($selection_parameter) =~ /^$rh_selection_parameters->{$selection_parameter}$/)
                    {
                        $match_selection_parameters = 1;
                    }
                    else
                    {
                        $match_selection_parameters = 0;
                        last;
                    }
                }
            }

            if (!$match_selection_parameters)
            {
                push(@a_o_notdeleted_patterns, $o_pattern);
            }
        }

        $self->{__ra_patterns} = \@a_o_notdeleted_patterns;
    }

}

=pod

=head3 procedure SelectPatterns

 Title        : SelectPatterns
 Usage        : $o_db_feature->SelectPatterns($rh_select_params);
 Prerequisite : none
 Function     : Select patterns from iANT::DB::Feature object according to
                a list of selection parameters
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub SelectPatterns
{
    my ($self, $rh_selection_parameters) = (@_);

    if (!defined $self->{__o_backup})
    {
        &iANT::Common::Backup($self);
    }

    my $ra_o_patterns         = $self->GetPatterns();
    my @a_o_selected_patterns = ();
    if (defined $ra_o_patterns && ($ra_o_patterns ne '') && (ref($ra_o_patterns) =~ /array/i))
    {
        foreach my $o_pattern (@{$ra_o_patterns})
        {
            my $match_selection_parameters = 0;

            foreach my $selection_parameter (keys %{$rh_selection_parameters})
            {
                if (ref($rh_selection_parameters->{$selection_parameter}) =~ /hash/i)
                {
                    my $o_parameter_element = $o_pattern->Get('o_' . $selection_parameter);

                    foreach my $element_selection_parameter (keys %{$rh_selection_parameters->{$selection_parameter}})
                    {
                        if ($o_parameter_element->Get($element_selection_parameter) =~
                            /^$rh_selection_parameters->{$selection_parameter}->{$element_selection_parameter}$/)
                        {
                            $match_selection_parameters = 1;
                        }
                        else
                        {
                            $match_selection_parameters = 0;
                            last;
                        }
                    }
                    if (!$match_selection_parameters)
                    {
                        last;
                    }
                }
                else
                {
                    if ($o_pattern->Get($selection_parameter) =~ /^$rh_selection_parameters->{$selection_parameter}$/)
                    {
                        $match_selection_parameters = 1;
                    }
                    else
                    {
                        $match_selection_parameters = 0;
                        last;
                    }
                }
            }

            if ($match_selection_parameters)
            {
                push(@a_o_selected_patterns, $o_pattern);
            }
        }

        $self->{__ra_patterns} = \@a_o_selected_patterns;
    }

}

=pod

=head3 procedure AddPatterns

 Title        : AddPatterns
 Usage        : $o_db_feature->AddPatterns($ra_o_patterns);
 Prerequisite : none
 Function     : Add patterns to iANT::DB::Feature object
 Returns      : An xml string
 Args         : a single iANT::Pattern object or a List Ref of such objects
 Globals      : none

=cut

sub AddPatterns
{
    my ($self, $ra_o_patterns) = (@_);

    if (!defined $self->{__o_backup})
    {
        &iANT::Common::Backup($self);
    }

    if (defined $ra_o_patterns && ($ra_o_patterns ne '') && (ref($ra_o_patterns) =~ /array/i))
    {
        foreach my $o_pattern (@{$ra_o_patterns})
        {
            $self->SetPattern($o_pattern);
        }
    }
    elsif (defined $ra_o_patterns && ($ra_o_patterns ne '') && (ref($ra_o_patterns) =~ /iANT::Pattern/i))
    {
        my $o_pattern = $ra_o_patterns;
        $self->SetPattern($o_pattern);
    }
    else
    {
        confess("Bad input type for AddFeatures methode: $ra_o_patterns");
    }

}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $xml_db_feature->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump a DB::Feature in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    &iANT::Common::Dump(@_);
}

=pod

=head3 function Restore

 Title        : Restore
 Usage        : $o_db_feature = $o_db_feature->Restore();
 Prerequisite : uses iANT::Common::Restore method
 Function     : Restores original object (before add/select/delete members)
 Returns      : none
 Args         : none
 Globals      : none

=cut

sub Restore
{
    my $self = shift();
    &iANT::Common::Restore($self);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);

}

=item GetAccession

    Returns      : DB Feature accession

=cut

sub GetAccession
{
    my ($self) = (@_);
    $self->Get('accession');
}

=item SetAccession

    Set DB Feature accession

=cut

sub SetAccession
{
    my ($self, $accession) = (@_);
    $self->Set('accession', $accession);
}

=item GetFeatures

    Returns      : Features objects list ref

=cut

sub GetFeatures
{
    my ($self) = (@_);
    $self->Get('ra_features');
}

=item SetFeature

    Add a Feature in DB Feature

=cut

sub SetFeature
{
    my ($self, $o_feature) = @_;
    my @a_o_features = ();
    if (defined $self->{__ra_features} && ($self->{__ra_features} ne ''))
    {
        @a_o_features = @{$self->{__ra_features}};
    }
    push(@a_o_features, $o_feature);
    $self->{__ra_features} = \@a_o_features;
    return;
}

=item GetPatterns

    Returns      : Pattern objects list ref

=cut

sub GetPatterns
{
    my ($self) = (@_);
    $self->Get('ra_patterns');
}

=item SetPattern

    Add a Pattern in DB Feature

=cut

sub SetPattern
{
    my ($self, $o_pattern) = @_;
    my @a_o_patterns = ();
    if (defined $self->{__ra_patterns} && (ref($self->{__ra_patterns}) =~ /array/i) && ($self->{__ra_patterns} ne ''))
    {
        @a_o_patterns = @{$self->{__ra_patterns}};
    }
    push(@a_o_patterns, $o_pattern);
    $self->{__ra_patterns} = \@a_o_patterns;
    return;
}

=cut



=head1 PRIVATE METHODS

=cut

#=pod
#
#=head3 function __InitializeFromHash
#
# Title        : __InitializeFromHash
# Usage        : __InitializeFromHash($o_feature,$rh_attributes);
# Prerequisite : none
# Function     : Fill attributes of an existing feature object from a hash (attr => value, ...)
# Returns      : An object iANT::Feature
# Args         : $o_feature
#                $rh_attributes
# Globals      : none
#
#=cut
#
#sub __InitializeFromHash
#{
#    my ($self, $rh_attributes) = (@_);
#    $self->{__type} = $rh_attributes->{type}    if (defined $rh_attributes->{type});
#    $self->{__path} = $rh_attributes->{path}    if (defined $rh_attributes->{path});
#    $self->{__date} = $rh_attributes->{date}    if (defined $rh_attributes->{date});
#    $self->{__o_software} = $rh_attributes->{o_software}    if (defined $rh_attributes->{o_software});
#    $self->{__o_database} = $rh_attributes->{o_database}    if (defined $rh_attributes->{o_database});
#    $self->{__o_sequence} = $rh_attributes->{o_sequence}    if (defined $rh_attributes->{o_sequence});
#    $self->{__o_hit} = $rh_attributes->{o_hit}    if (defined $rh_attributes->{o_hit});
#
#    return $self;
#}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_db_feature,$o_twig_db_feature);
 Prerequisite : none
 Function     : Fill attributes of an existing db feature object from a XML::Twig object
 Returns      : An object iANT::DB::Feature
 Args         : $o_db_feature
                $o_twig_db_feature
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_twig_db_feature) = (@_);
    my $entry_features_elt = $o_twig_db_feature->root;

    $self->{__accession} = $entry_features_elt->{'att'}->{'accession'}
      if defined $entry_features_elt->{'att'}->{'accession'};

    my @a_feature_elts = $entry_features_elt->children('feature');
    my @a_o_features   = ();
    foreach my $feature_elt (@a_feature_elts)
    {
        my $o_feature = New iANT::Feature($feature_elt->sprint);
        push(@a_o_features, $o_feature);
    }

    $self->{__ra_features} = \@a_o_features;

    my @a_pattern_elts = $entry_features_elt->children('pattern');
    my @a_o_patterns   = ();
    foreach my $pattern_elt (@a_pattern_elts)
    {
        my $o_pattern = New iANT::Pattern($pattern_elt->sprint);
        push(@a_o_patterns, $o_pattern);
    }

    $self->{__ra_patterns} = \@a_o_patterns;

    return $self;
}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Feature

=cut

1;
