#!/usr/bin/env perl

package iANT::Feature::Hit::Property;

use strict;
use warnings;
use XML::Twig;
use iANT::Common;
use iANT::feature::hit::property::property;

=head1 NAME

iANT::Feature::Hit::Property - Feature hit property object


=head1 SYNOPSIS

    my %h_feature = (type => 'manual', path => '/path/to/analysis/file');
    my $o_feature = New iANT::Feature(\%h_feature);
    $o_feature->Set('date','14032006');

    my %h_software    = (name => 'blastp', version  => '2.2.6');
    my $o_feature_software = New iANT::Feature::Software(\%h_software);
    $o_feature->SetSoftware($o_feature_software);

    my %h_database    = (name => 'sptr', version  => 'mars2006', comment => "un commentaire");
    my $o_feature_database = New iANT::Feature::Database(\%h_database);
    $o_feature->SetDatabase($o_feature_database);

    my %h_sequence = (id => 'RSc0092', accession => 'RS02279_AA', length => 113, begin => 22, end => 95, definition => 'proteine transsmembranaire');
    my $o_feature_sequence = New iANT::Feature::Sequence(\%h_sequence);
    $o_feature->SetSequence($o_feature_sequence);

    my %h_hit = (id => 'Q8Y388_RALSO', accession => 'Q8Y388', length => 113, begin => 22, end => 95, definition => 'PROBABLE TRANSMEMBRANE PROTEIN');
    my $o_feature_hit = New iANT::Feature::Hit(\%h_hit);

    my $o_hit_raw = New iANT::Feature::Hit::Raw({content => "> pas de contenu <"});
    $o_feature_hit->SetRaw($o_hit_raw);

    my $o_hit_property = New iANT::Feature::Hit::Property({id => 'score', value => '100'});
    $o_feature_hit->SetProperty($o_hit_property);
    $o_hit_property = New iANT::Feature::Hit::Property({id => 'gap', value => '0'});
    $o_feature_hit->SetProperty($o_hit_property);

    $o_feature->SetHit($o_feature_hit);

    $o_feature->Dump();

    print "\n\n\nRaw: ", $o_feature->GetHit->GetRaw->GetContent(),"\n";

    my $o_hit = $o_feature->GetHit();
    print "Hit   ","\n";
    print "      Id:",$o_hit->GetId(),"\n";
    print "      Ac:",$o_hit->Get('accession'),"\n";


=head1 DESCRIPTION

    iANT::Feature::Hit::Property is designed to make it reasonably easy to create, consume or modify a feature hit property the XML syntax:
    
    <feature type="" path="" date="">
        <software name="" version="" comment=""/>
        <database name="" version="" comment=""/>  
        <sequence id="" accession="" length="" definition="" begin="" end=""/>
        <hit id="" accession="" definition="" length="" begin="" end="">
                <property id="bit-score" value=""/>
                <property id="length"    value="100"/>
                <property id="mismatch"  value="4"/>
                <raw></raw>
        </hit>
    </feature>


=cut

=cut


=head1 PUBLIC METHODS


=pod 


=head3 function New

 Title        : New
 Usage        : New iANT::Feature::Hit::Property;
 Prerequisite : none
 Function     : Create an object iANT::Feature::Hit::Property
 Returns      : An object iANT::Feature::Hit::Property
 Args         : none
 Globals      : none

=cut

BEGIN
{
    our $VERSION = "0.1";

}

sub New
{
    my ($class, $xml_source) = (@_);

    my $self = {
                __id    => "",
                __value => ""
                };

    if (defined $xml_source)
    {
        $self = &Update($self, $xml_source);
    }
    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_property->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing iANT::Feature::Hit::Property object
 Returns      : An object iANT::Feature::Hit::Property
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_property;

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (-e $source)
    {
        $o_xml_property = iANT::feature::hit::property::property->new(filename => $source);
        $self = &__InitializeFromXml($self, $o_xml_property);
    }
    else    #raw XML data
    {
        $o_xml_property = iANT::feature::hit::property::property->new(xml => $source);
        $self = &__InitializeFromXml($self, $o_xml_property);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_property->GetXml();
 Prerequisite : none
 Function     : Retrieve a Property in xml format
 Returns      : An xml string
 Args         : none
 Globals      : none

=cut

sub GetXml
{
    my ($self) = @_;
    my $o_xml_property = iANT::feature::hit::property::property->new();

    $o_xml_property->id($self->{__id})       if (defined $self->{__id});
    $o_xml_property->value($self->{__value}) if (defined $self->{__value});

    return $o_xml_property;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_definition->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump a Property in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);
}

=item GetId

    Returns      : Property id

=cut

sub GetId
{
    my ($self) = (@_);
    $self->Get('id');
}

=item SetId

    Set property id

=cut

sub SetId
{
    my ($self, $id) = (@_);
    $self->Set('id', $id);
}

=item GetValue

    Returns      : Property value

=cut

sub GetValue
{
    my ($self) = (@_);
    $self->Get('value');
}

=item SetValue

    Set Property value

=cut

sub SetValue
{
    my ($self, $value) = (@_);
    $self->Set('value', $value);
}

=cut



=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_property,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing property object from a hash (attr => value, ...)
 Returns      : An object iANT::Feature::Hit::Property
 Args         : $o_property
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    $self->{__id}    = $rh_attributes->{id}    if (defined $rh_attributes->{id});
    $self->{__value} = $rh_attributes->{value} if (defined $rh_attributes->{value});
    return $self;
}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_property,$o_xml_property);
 Prerequisite : none
 Function     : Fill attributes of an existing property object from a Class::XML object
 Returns      : An object iANT::Feature::Hit::Property
 Args         : $o_property
                $o_xml_property
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_xml_property) = (@_);

    $self->{__id}    = $o_xml_property->id    if (defined $o_xml_property->id);
    $self->{__value} = $o_xml_property->value if (defined $o_xml_property->value);
    return $self;

}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Feature
    iANT::Feature::Software
    iANT::Feature::Database
    iANT::Feature::Sequence
    iANT::Feature::Hit

=cut

1;
