#!/usr/bin/env perl

package iANT::Organism::Organelle;

use strict;
use warnings;
use XML::Twig;
use iANT::Common;
use iANT::organism::organelle::organelle;
use HTML::Entities;

=head1 NAME

iANT::Organism::Organelle - Organism Organelle object


=head1 SYNOPSIS



=head1 DESCRIPTION

    iANT::Organism::Organelle is designed to make it reasonably easy to create, consume or modify iANT or MENS organism organelle following the XML syntax:
    
    <organism>
        <species code="rsoa">Ralstonia solanacearum</species>
        <taxonomy ontology="NCBI" id="taxid">dfsgjklsdflgjsdfklg</taxonomy>
        <organelle>Plasmid</organelle>
    </organism>


=cut

=cut


=head1 PUBLIC METHODS


=pod 


=head3 function New

 Title        : New
 Usage        : New iANT::Organism::Organelle;
 Prerequisite : none
 Function     : Create an object iANT::Organism::Organelle
 Returns      : An object iANT::Organism::Organelle
 Args         : none
 Globals      : none

=cut

BEGIN
{
    our $VERSION = "0.1";
}

sub New
{
    my ($class, $xml_source) = (@_);

    my $self = {__content => ""};

    if (defined $xml_source)
    {
        $self = &Update($self, $xml_source);
    }
    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_organelle->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing iANT::Organism::Organelleobject
 Returns      : An object iANT::Organism::Organelle
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_organelle;
    chomp $source;

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (($source !~ /\n/) && (-e $source))
    {
        $o_xml_organelle = iANT::organism::organelle::organelle->new(filename => $source);
        $self = &__InitializeFromXml($self, $o_xml_organelle);
    }
    else    #raw XML data
    {
        $o_xml_organelle = iANT::organism::organelle::organelle->new(xml => $source);
        $self = &__InitializeFromXml($self, $o_xml_organelle);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_organelle->GetXml();
 Prerequisite : none
 Function     : Retrieve an Organelle in xml format
 Returns      : An xml string
 Args         : none
 Globals      : none

=cut

sub GetXml
{
    my ($self) = @_;
    my $o_xml_organelle = iANT::organism::organelle::organelle->new();

    my $o_twig = XML::Twig->new();
    $o_twig->parse($o_xml_organelle);
    my $root            = $o_twig->root;
    my $encoded_content = HTML::Entities::encode($self->GetContent());
    $root->set_text($encoded_content);
    $o_xml_organelle = iANT::organism::organelle::organelle->new(xml => $o_twig->sprint());

    return $o_xml_organelle;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_definition->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump a an Organelle in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);
}

=item GetContent

    Returns      : Organelle content

=cut

sub GetContent
{
    my ($self) = @_;
    $self->Get('content');
}

=item SetContent

    Set Organelle content

=cut

sub SetContent
{
    my ($self, $content) = @_;
    $self->Set('content', $content);
}

=cut



=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_raw,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing organelle object from a hash (attr => value, ...)
 Returns      : An object iANT::Organism::Organelle
 Args         : $o_organelle
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    $self->{__content} = $rh_attributes->{content} if (defined $rh_attributes->{content});
    return $self;
}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_raw,$o_xml_raw);
 Prerequisite : none
 Function     : Fill attributes of an existing organelle object from a Class::XML object
 Returns      : An object iANT::Organism::Organelle
 Args         : $o_organelle
                $o_xml_organelle
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_xml_organelle) = (@_);
    my $o_twig = XML::Twig->new();
    $o_twig->parse($o_xml_organelle);
    my $root = $o_twig->root;
    $self->{__content} = HTML::Entities::decode($root->text);
    return $self;
}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Feature
    iANT::Entry
    

=cut

1;
