#!/usr/bin/env perl

package iANT::Pattern;

use strict;
use warnings;
use Carp;
use XML::Twig;
use iANT::Common;
use iANT::pattern::pattern;
use HTML::Entities;

BEGIN
{
    our $VERSION = "0.1";
}

=head1 NAME

iANT::Pattern - iANT/MENS Pattern object


=head1 SYNOPSIS


example: 

    my %h_attributes = (type => 'transcriptomic_array', definition => 'GMI1000 vs. 1609');
    my $o_pattern = New iANT::Pattern(\%h_attributes);
    $o_pattern->SetContent('2.5');
    print "Pattern ", $o_pattern->Get('type'), ":", $o_pattern->Get('definition'), "valeur:", $o_pattern->Get('content'),"\n";
    

=head1 DESCRIPTION

    iANT::Pattern is designed to make it reasonably easy to create, consume or modify iANT or MENS pattern following the XML syntax:
    <pattern type="transcriptomic_array" definition="mutant1 vs wt">-0.35</pattern>

=cut

=cut



=head1 PUBLIC METHODS

=pod 

=head3 function New

 Title        : New
 Usage        : New iANT::Pattern;
 Prerequisite : none
 Function     : Create an object iANT::Pattern
 Returns      : An object iANT::Pattern
 Args         : none
 Globals      : none

=cut

sub New
{
    my ($class, $source) = (@_);

    my $self = {
                __type       => "",
                __definition => "",
                __content    => ""
                };

    if (defined $source)
    {
        $self = &Update($self, $source);
    }

    bless($self, $class);

    return ($self);
}

=pod

=head3 function Update

 Title        : Update
 Usage        : $o_pattern->Update($source);
 Prerequisite : none
 Function     : Update attributes of an existing Pattern object
 Returns      : An object iANT::Pattern
 Args         : $source: xml file | xml string | hashref
 Globals      : none

=cut

sub Update
{
    my ($self, $source) = (@_);
    my $o_xml_pattern;

    if (ref($source) =~ /hash/i)
    {
        $self = &__InitializeFromHash($self, $source);
    }
    elsif (-e $source)
    {
        $o_xml_pattern = iANT::pattern::pattern->new(filename => $source);
        $self = &__InitializeFromXml($self, $o_xml_pattern);
    }
    else    #raw XML data
    {
        $o_xml_pattern = iANT::pattern::pattern->new(xml => $source);
        $self = &__InitializeFromXml($self, $o_xml_pattern);
    }
    return $self;
}

=pod

=head3 function GetXml

 Title        : GetXml
 Usage        : $o_pattern->GetXml($target_element);
 Prerequisite : none
 Function     : Retrieve a pattern (or part of) in xml format
 Returns      : An xml string
 Args         : $target_element: optional; if empty or undef, retrieve all
                xml elements.
 Globals      : none

=cut

sub GetXml
{
    my ($self, $target_element) = @_;
    my $o_xml_pattern = iANT::pattern::pattern->new();
    if (   (!defined $target_element)
        || ($target_element eq 'pattern')
        || ($target_element eq '')
        || ($target_element eq 'all'))
    {
        $o_xml_pattern->type($self->{__type});
        $o_xml_pattern->definition($self->{__definition});
        my $o_twig = XML::Twig->new();
        $o_twig->parse($o_xml_pattern);
        my $root            = $o_twig->root;
        my $encoded_content = HTML::Entities::encode($self->GetContent());
        $root->set_text($encoded_content);
        $o_xml_pattern = iANT::pattern::pattern->new(xml => $o_twig->sprint());
    }
    return $o_xml_pattern;
}

=pod

=head3 procedure Dump

 Title        : Dump
 Usage        : $o_pattern->Dump();
 Prerequisite : uses iANT::Common::Dump method
 Function     : Dump a Pattern in xml format
 Returns      : none
 Args         : $target:-optional; if empty or undef, print on STDOUT
                        -if defined, print in the specified file (unix-like mode is mandatory)
 Globals      : none

=cut

sub Dump
{
    my ($self, $target) = (@_);
    &iANT::Common::Dump(@_);
}

=pod
 
=head3 Getters and Setters

=item Get

    Function     : Generic Getter
    Args         : $attribute
    Prerequisite : uses iANT::Common::Get method

=cut

sub Get
{
    my ($self, $attribut) = @_;
    &iANT::Common::Get(@_);
}

=pod

=item Set

    Function    : Generic Setter
    Args        : $attribute
                  $value
    Prerequisite : uses iANT::Common::Set method

=cut

sub Set
{
    my ($self, $attribut, $value) = @_;
    &iANT::Common::Set(@_);

}

=item GetType

    Returns      : Pattern Type

=cut

sub GetType
{
    my ($self) = @_;
    $self->Get('type');
}

=item SetType

    Set Pattern Type

=cut

sub SetType
{
    my ($self, $type) = @_;
    $self->Set('type', $type);
}

=item GetDefinition

    Returns      : Pattern definition

=cut

sub GetDefinition
{
    my ($self) = @_;
    $self->Get('definition');
}

=item SetDefinition

    Set Pattern definition

=cut

sub SetDefinition
{
    my ($self, $definition) = @_;
    $self->Set('definition', $definition);
}

=item GetContent

    Returns      : Pattern Content

=cut

sub GetContent
{
    my ($self) = @_;
    $self->Get('content');
}

=item SetContent

    Set Pattern Content

=cut

sub SetContent
{
    my ($self, $content) = @_;
    $self->Set('content', $content);
}

=head1 PRIVATE METHODS

=pod

=head3 function __InitializeFromHash

 Title        : __InitializeFromHash
 Usage        : __InitializeFromHash($o_pattern,$rh_attributes);
 Prerequisite : none
 Function     : Fill attributes of an existing Pattern object from a hash (attr => value, ...)
 Returns      : An object Pattern
 Args         : $o_pattern:
                $rh_attributes
 Globals      : none

=cut

sub __InitializeFromHash
{
    my ($self, $rh_attributes) = (@_);
    $self->{__type}       = $rh_attributes->{type}       if (defined $rh_attributes->{type});
    $self->{__definition} = $rh_attributes->{definition} if (defined $rh_attributes->{definition});
    $self->{__content}    = $rh_attributes->{content}    if (defined $rh_attributes->{content});
    return $self;
}

=pod

=head3 function __InitializeFromXml

 Title        : __InitializeFromXml
 Usage        : __InitializeFromXml($o_pattern,$o_xml_pattern);
 Prerequisite : none
 Function     : Fill attributes of an existing Pattern object from a Class::XML object
 Returns      : An object Pattern
 Args         : $o_pattern
                $o_xml_pattern
 Globals      : none

=cut

sub __InitializeFromXml
{
    my ($self, $o_xml_pattern) = (@_);
    $self->{__type}       = $o_xml_pattern->type       if (defined $o_xml_pattern->type);
    $self->{__definition} = $o_xml_pattern->definition if (defined $o_xml_pattern->definition);

    my $o_twig = XML::Twig->new();
    $o_twig->parse($o_xml_pattern);
    my $root = $o_twig->root;
    $self->{__content} = HTML::Entities::decode($root->text);
    return $self;
}

=head1 AUTHOR

    Sebastien Carrere (Sebastien.Carrere@toulouse.inra.fr)
    Jerome Gouzy (Jerome.Gouzy@toulouse.inra.fr)

=head1 MANDATORY PERL PACKAGES

    Class::XML
    XML::Twig
    Carp


=head1 SEE ALSO

    iANT::Entry
    iANT::Feature
    iANT::DB::Feature

=cut

1;
