#!/usr/bin/perl -I perl

# $Id$

=pod
=head1 Glintfreq.pl

Glintfreq.pl - 
GLINTFREQ = Create unique pcr probes to identify a specific genome region

This program launch glintFreq_exe building suitable subregion from a genomic region
then it launch Primer3_exe creating a bunch of probes foreach suitable subregion
finally two checkings eliminates unvalid probes (ex : matching more than once in the whole genome)

=head1  SYNOPSIS

perl glintfreq.pl 

--genome genomeFilePath 
--region chr:start-end 
--target path directory                                                                OPTIONAL

--count_cutoff GLINTFREQ OPTION increase the number of results                         OPTIONAL
--suitable_min_size GLINTFREQ OPTION size of the suitable subregion created            OPTIONAL
--product_size_range PRIMER3 OPTION size range of the pcr product                      OPTIONAL

--select the valid probes are not re-search by the soft the probe selection is ~direct OPTIONAL
         sets by default --debug at 1
--probe_nb number of probe in the output selection                                     OPTIONAL
--selection_threshold probe penalty threshold for the output selection                 OPTIONAL

--force forces the software to recreate the query sequence file                        OPTIONAL

--debug if defined erase all the tmp file                                              OPTIONAL

=head1 DESCRIPTION
GLINTFREQ = Create unique pcr probes to identify a specific genome region

This program launch glintFreq_exe building suitable subregion from a genomic region
then it launch Primer3_exe creating a bunch of probes foreach suitable subregion
finally two checkings eliminates unvalid probes (ex : matching more than once in the whole genome)

=cut
=head1 SUBROUTINES

MAIN
DoYourJob
GetRegionInfos
GetParameters
FindCfgFile

=cut

use strict;
use warnings;

use Cwd qw( abs_path getcwd );
use Carp;
use Data::Dumper;

use File::Spec;
use Error qw( :try );
use File::Temp qw( tempdir );
use Pod::Usage;
use FindBin qw( $Bin );

use Bio::SeqIO;

use lib "$Bin/../lib";
use lib "$Bin/../lib/lipmutils";
use lib "/mnt/data4/www-narcisse/Dev/audrey22/animals/Core/lib";
use lib "/mnt/data4/www-narcisse/Dev/audrey22/animals/Core/API";

# nos modules, nos classes
use LipmError;
use General;
use ParamParser;
use Logger;
use Configuration::InstanceObserver;
use SpeciesObserver;

use Region;

use constant SPECIES => 'species';
use constant REGION  => 'region' ;		# $chrom : $start - $end
use constant TARGET  => 'target' ;
use constant PARAMS  => 'params' ;

MAIN:
{
	my $rh_params = &GetParameters();
	my ( $species, $region, $target, $rh_param ) = @{ $rh_params }{ ( &SPECIES, &REGION, &TARGET, &PARAMS ) };
	
	print "\n";

	&DoYourJob(  &SPECIES => $species, &REGION => $region, &TARGET => $target, &PARAMS => $rh_params );

	print "\n";
}

sub DoYourJob
{
	my %h_args = @_;
	
	my ( $species, $region, $target_dir, $rh_exeParam ) = @h_args{ ( &SPECIES, &REGION, &TARGET, &PARAMS ) }; 
	
	my $rh_region = &GetRegionInfos( $region );
	my ( $chrom, $start, $end ) = @{$rh_region}{ qw( chrom start end )};
	
	my $o_instanceObserver = New InstanceObserver( -cfg => "/mnt/data4/www-narcisse/Dev/audrey22/animals/Site/cfg/narcisse.cfg" );
	
	my $o_speciesObserver =  $o_instanceObserver->GetSpeciesObserver( $species );
	
	my $rh_chrominfo = $o_speciesObserver->GetChromsInfo();
	
	my $chrom_file    = $o_speciesObserver->GetChromFastaFile( chrom => $chrom );
	my $ra_index_file = $o_speciesObserver->GetChromIndexFilepaths();
	
	
	my $o_region = New Region(	chr        => $chrom, 
								start      => $start, 
								end        => $end,
								species    => $species,
								chrom_file => $chrom_file,
								index_file => $ra_index_file,
								target     => $target_dir,
								param      => $rh_exeParam,
								);
	$o_region->Run();
}

sub GetRegionInfos
{
	my ( $region ) = @_;
	
	my %h_region = ();
	my $ok = 0;
	
	if ( $region =~ /^(\d+):(\d+)\-(\d+)$/ )
	{
		@h_region{qw( chrom start end)} = ($1,$2,$3) ;
		$ok = 1 if ( $h_region{ start } < $h_region{ end } );
	}
	
	$ok or die 'Wrong region format, please see usage';
	
	return \%h_region;
}

sub GetParameters
{
	&ParamParser::SetDefaultBehaviour( 'use_substitution_table' );
	&ParamParser::SetDefaultUsage( sub { pod2usage(1) } );
	
	my $o_param =New ParamParser;
	
	my $cfg_file = &FindCfgFile();
	
	$o_param->Update( $cfg_file, 'A' );
	
	my $binpath =  abs_path( "$Bin/../.." );
	
	$o_param->SetSubstitution( '%p', $binpath );
	$o_param->Update($cfg_file, 'O' );
	
	my @a_keys   = qw( help );
	push( @a_keys, &SPECIES . '=s', &REGION . '=s', &TARGET . '=s', 'count_cutoff=i', 'suitable_min_size=i', 'product_size_range=s', 'probe_nb=i', 'select_threshold=i', 'force=i', 'select=i', 'versus=i', 'verbose=i' );
	$o_param->Update( 'GETOPTLONG', "O", 'debug=i', @a_keys );
	
	$o_param->AssertDefined( &SPECIES, &REGION );
	
	
	if ( $o_param->IsDefined( &TARGET ) )
	{
		$o_param->AssertDirExists( &TARGET );
	}
	
	#Option commandline
	if ( $o_param->IsDefined( 'count_cutoff'       ) )
	{
		my $count_cutoff = $o_param->Get( 'count_cutoff' );
		$o_param->Set( 'COUNT_CUTOFF', $count_cutoff );
	}
	if ( $o_param->IsDefined( 'suitable_min_size'  ) )
	{
		my $suitable_min_size = $o_param->Get( 'suitable_min_size' );
		$o_param->Set( 'SUITABLE_MIN_SIZE', $suitable_min_size );
	}
	if ( $o_param->IsDefined( 'product_size_range' ) )
	{
		my $product_size_range = $o_param->Get( 'product_size_range' );
		$o_param->Set( 'PRIMER_PRODUCT_SIZE_RANGE', $product_size_range );
	}
	if ( $o_param->IsDefined( 'probe_nb'           ) )
	{
		my $probe_nb = $o_param->Get( 'probe_nb' );
		$o_param->Set( 'TOTAL_PROBE_NUMBER', $probe_nb );
	}
	if ( $o_param->IsDefined( 'selection_threshold' ) )
	{
		my $selection_threshold = $o_param->Get( 'selection_threshold' );
		$o_param->Set( 'SCORING_THRESHOLD', $selection_threshold );
	}
	if ( $o_param->IsDefined( 'force'  ) )
	{
		my $force = $o_param->Get( 'force' );
		$o_param->Set( 'FORCE', $force );
	}
	if ( $o_param->IsDefined( 'select' ) )
	{
		my $select= $o_param->Get( 'select' );
		$o_param->Set( 'SELECT', $select );
	}
	if ( $o_param->IsDefined( 'debug'  ) )
	{
		my $debug= $o_param->Get( 'debug' );
		$o_param->Set( 'DEBUG', $debug );
	}
	if ( $o_param->IsDefined( 'versus' ) )
	{
		my $versus= $o_param->Get( 'versus' );
		$o_param->Set( 'VERSUS', $versus );
	}
	if ( $o_param->IsDefined( 'verbose' ) )
	{
		my $verbose= $o_param->Get( 'verbose' );
		$o_param->Set( 'VERBOSE', $verbose );
	}
	
	#Returning a hash
	my $rh_params = {};
	$o_param->Dump( 'HASH', $rh_params );
	
	return $rh_params;
}

sub FindCfgFile
{
	my $cfg_file = "$Bin/../../cfg/parameters.cfg";
	
	( -e $cfg_file )
		or die 'Configuration file not available';
	
	return $cfg_file;
}
