/*
   $Id: CommandChain.cpp 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <iostream>
#include <fstream>

#include "ParametersChain.h"
#include "glint.h"
#include "chain.h"
#include "utils.h"
#include "ParametersChain.h"
#include "CommandChain.h"

#ifndef __LP64__
    #error "GLINT MAY BE COMPILED ONLY ON A 64 BITS PLATFORM"
#endif

using namespace std;

void Usage();
void FirstClustering(AnchorVect& A, AnchorVect& B );
void ChainLowerLevel(AnchorVect& A, AnchorVect& B, uint32_t md, uint32_t mc, float mcv );
void GetOptions(int argc, char** argv, void(*help)());
void LogParameters(string file, int argc,char** argv);
void __OptionsValidation();
extern CSimpleOpt::SOption g_GlintOptions[];

CompressedFileName g_GlintOutputFilename;
CompressedFileName g_BlastpOutputFilename;
CompressedFileName g_AnchorsOutputFilename;

string g_OutputFilename;
string g_cutoffs = UNDEFINED_STRING;
string g_targetdir="";
uint32_t  g_maxoverlap = DEFAULT_MAX_CHAIN_OVERLAP;
bool g_RBH=true;
bool g_SELF=false;

int CommandChain::Main(int argc,char** argv)
{
	Parameters::Init();
	CompressionInfo::Init();

	Parameters& parameters = Parameters::Instance();
	ParametersInitializer& prm_init = _GetInitializer();
	prm_init.Init(argc, argv, *this);

	// OBSOLETE - A JETER - UTILISER L'OBJET parameters A LA PLACE !!!
	g_GlintOutputFilename   = parameters.GetGlintOutputFilename();
	g_BlastpOutputFilename  = parameters.GetBlastpOutputFilename();
	g_AnchorsOutputFilename = parameters.GetAnchorsOutputFilename();
	g_OutputFilename        = parameters.GetOutputFilename();
	g_cutoffs               = parameters.GetChainCutoffs();
	g_targetdir             = parameters.GetTargetdir();
	g_maxoverlap            = parameters.GetMaxOverlap();
	g_RBH                   = parameters.IsRbh();
	g_SELF                  = parameters.IsSelf();


	if ( g_targetdir != "" )
	{
		struct stat s_dir;
		
		// If there is some stuff with the good name...
		if ( stat (g_targetdir.c_str(), &s_dir )==0 )
		{
			// ... but it is NOT a directory
			if ( S_ISDIR(s_dir.st_mode)==0 )
			{
				cerr << "ERROR - " << g_targetdir << " is NOT a directory\n";
				exit(1);
			}
			
			// .. or it is not writable
			else if ( (s_dir.st_mode & S_IWUSR) == 0 )
			{
				cerr << "ERROR - " << g_targetdir << " is NOT a writable\n";
				exit(1);
			}
		}
		
		// If there is no such stuff, try to create it
		else
		{
			if (mkdir(g_targetdir.c_str(),0777)!=0)
			{
				cerr << "ERROR - Could not create " << g_targetdir << "\n";
				exit(1);
			}
		}
	}				
			
		
	string baseoutputfile = (  g_targetdir == ""  ) ? "output_chain" :  g_targetdir + "/" + "chain";

	if ( g_SELF == true && !parameters.IsSilent())
	{
		cerr<<"Self chaining has been asked"<<endl;
	}

	Syntenies synt( g_cutoffs, g_GlintOutputFilename, g_BlastpOutputFilename, g_AnchorsOutputFilename, baseoutputfile, g_RBH, g_maxoverlap);

	synt.LoadAlignments();
	synt.ConstructSyntenyLevels();

	synt.Print();

	string baselogfile = baseoutputfile + ".log";
	LogParameters( baselogfile, argc, argv );

	return 0;
}

void LogParameters(string file_name, int argc,char** argv)

{
	ofstream out(file_name.c_str());

	out << "Command: ";
    for (int p=0; p<argc;p++)
        out << argv[p] << ' ';
    out << '\n';

	if ( g_GlintOutputFilename )
		out << "glintout: "<< g_GlintOutputFilename <<endl;
	if ( g_BlastpOutputFilename )
		out << "blaspout: "<<  g_BlastpOutputFilename <<endl;
	if ( g_AnchorsOutputFilename )
		out << "anchors: "<<  g_AnchorsOutputFilename <<endl;
	out<<"Rbh: "<<g_RBH<<endl;
	out<<"Self: "<<g_SELF<<endl;
	out<<"targetdir: "<<g_targetdir<<endl;
	out<<"cutoffs: "<<g_cutoffs<<endl;

	out.close();
}



/*Utilisation du programme */
void CommandChain::Usage() const
{
	cerr << "glint chain: chaining program for glint hsps\n";
	cerr << "Usage: glint chain --glintout glint_output --blastpout blastp_output --cutoffs CMD-CS-CCV\n";
	cerr << "    or glint chain --anchors anchors_output --cutoffs CMD-CS-CCV\n\n";
	cerr << "\tOptions:\n";
	cerr << "\t\t --glintout glint.out   : The output from a previous glint invocation, in -m 2 format \n";
	cerr << "\t\t --blastpout blastp.out : The output from a previous blastp (protein) invocation, in ??? format\n";
	cerr << "\t\t --anchors anchors.out  : ???\n";
	cerr << "\t\t --targetdir dir : directory for the results files (default local directory)\n";
	cerr << "\t\t --cutoffs CMD-CS-CCV : the chaining parameters see below \n";
	cerr << "\t\t   The cutoff values are given for each level as 3 values using the following format CMD-CS-CCV : \n";
	cerr << "\t\t\tCMD : cutoff value for the manhattan Distance separating anchors (integer, a strict maximum value)\n";
	cerr << "\t\t\t\tThis parameter will be used for the construction of connected components.\n";
	cerr << "\t\t\tCS  : cutoff score for a cluster to be part of the cluster partition (integer, a strict minimum value)\n";
	cerr << "\t\t\tCCV : cutoff score for the covariation of anchors within a cluster (float (0<CCV<1), a strict minimum value)\n";
	cerr << "\tMore switches:\n";
	cerr << "\t\t --rbh : ????\n";
	cerr << "\t\t --self: ask self-chaining\n";
	cerr << "\t\t --silent : less warnings\n";
	cerr << "\t\t --verbose: more blabla\n";
}
