/*
   $Id: GlintEngine.cpp 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <stdio.h>
#include <stdlib.h>

#ifdef _OPENMP
	#include <omp.h>
#endif

#include "GlintEngine.h"
#include "Display.h"
#include "DisplayM1.h"
#include "DisplaySam.h"

#include "ptypes.h"
#include "CompactTree.h"
#include "alignment.h"
#include "utils.h"
#include "dust.h"
#include "QueryBuffer.h"
#include "QueryBufferDouble.h"
#include "bitcount.h"
#include "iqsort.h"
#include "Bank.h"
#include "time.h"

extern RealLogger* logger;

/**
 * \brief GlintEngineBase constructor
 */
GlintEngineBase::GlintEngineBase(QueryBuffer* q, Display& d):
	_parameters(Parameters::Instance()),
	_mask(Mask::Instance()),
	_ctree (CompactTree::Instance()),
	_display(d),
	_query_ptr(q),
	__query_aptr(NULL) 	
{
	for (uint16_t i = 1; i != 256; ++i) 
		_g_log_n[i] = (int)(4.343 * log(i) + 0.5);		
}

/**
 * \brief GlintEngineBase copy constructor
 */

GlintEngineBase::GlintEngineBase(const GlintEngineBase& eng):
	_parameters(Parameters::Instance()),
	_mask(Mask::Instance()),
	_ctree (CompactTree::Instance()),
	_display(eng._display),
	_query_ptr(eng._query_ptr->Clone()),
	__query_aptr(_query_ptr) {cerr<<"Now calling copy for GlintEngineBase " << this << endl;}

/**
 * \brief GlintEngine constructor
 */
GlintEngine::GlintEngine(QueryBuffer* q, Display& d):
	GlintEngineBase(q,d),
	__plus  (PLUS, _display),
	__minus (MINUS, _display) {}

/**
 * \brief GlintEngine copy constructor
 */

GlintEngine::GlintEngine(const GlintEngine& eng):
	GlintEngineBase(eng),
	__plus  ( PLUS, _display),	// NOTE - This is NOT a REAL copy 
	__minus (MINUS, _display) {}
	

bool GlintEngineBase::ReadNextSeq() { return _query_ptr -> ReadNextSeq(); }

/**
 * \brief Run: Running the entire glint engine process on the query
 */
void GlintEngineBase::Run ()
{
	if ( !_parameters.IsSilent() || _parameters.IsHspOnTheFly())
	{
			if ( !_parameters.IsSilent()) 
			{
				uint8_t msk_length 				= _mask.GetLength();
				size_t seq_length 				= _query_ptr->GetSeqLength();
				if ( seq_length < msk_length ) {
					ostringstream data;
					data << "SEQUENCE: " << _query_ptr->GetSeqId() << " - LENGTH = " << seq_length;
					ostringstream msg;
					msg << "sequence length smaller than mask length (=" << (int)msk_length << ")";
					_parameters.Warn(msg.str(),data.str());
				}
			}
		
			// If not on the fly, we prefer to set the query at the very last moment, for parallelization
			if ( _parameters.IsHspOnTheFly())
			{
				//_display.SetQuery(_query_aptr.get());
				_display.SetQuery(_query_ptr);
			}
	}
	__Run();
}

/**
 * \brief Start the comparison between the sequence stored in the QueryBuffer and the bank singleton
 *         -# Mask low complexity sequences if asked ( -C > 0)
 *         -# Search for seeds
 *         -# Align the seeds and create the hsps
 * \param a reference on a QueryBuffer
 */
void GlintEngineBase::__Run ()
{	
	BuildHsps();
	if ( _parameters.GetOutputFormat() != 0 )
	{
		#pragma omp critical
		{
			DisplayHsps();	
			if ( GetNbHsp() == 0 && !_parameters.IsSilent() && !_query_ptr->IsUpper(_mask.GetLength()) )
			{
				string data = "SEQUENCE: " + _query_ptr->GetSeqId();
				string msg;
				if ( !_parameters.IsToUpper() )
				{
					msg = "ONLY LOWER CASE SEQUENCE: may be you should try the --to-upper switch ?";
				}
				else
				{
					msg = "SKIPPED SEQUENCE because low complexity: may be you should try the --no_lc_filtering switch ?";
				}
				_parameters.Warn(msg,data);
			}
			HspReset();
		}
	}
	else
	{
		HspReset();
	}
}

/**
 * \brief Display the Hsps if any
 *         -# Print the hsps, unless already done
 *         -# Reset the hsp-related structures
 */
void GlintEngineBase::DisplayHsps ()
{		
	if ( !_parameters.IsHspOnTheFly() && GetNbHsp()>0 )
	{	
		__DisplayAll();
		return ;			
	} 	
}

/**
 * \brief Start the comparison between the sequence stored in the QueryBuffer and the bank singleton
 *         -# Mask low complexity sequences if asked ( -C > 0)
 *         -# Search for seeds on the plus and on the minus strands
 *         -# Align the seeds and create the hsps on the plus and on the minus strands
 */
void GlintEngine::BuildHsps ()
{	
	// Nothing to mask if self, as the masking was already done while indexing !
	if ( !_parameters.IsSelf() )	
	{
		__MaskLowComplexity();		
	}
	__SearchingForSeeds();
	__ComputeHsp();	
	//cerr << "ici GlintEngine::BuildHsps() " << GetNbHsp() << " " << _query_ptr -> GetSeqLength() << "\n";
}

/** 
 * \brief Computing the hsp
 *        - Calling ComputeSimpleMatches for each strand
*/
void GlintEngine::__ComputeHsp()
{
	QueryBuffer& query = *_query_ptr;
	__plus.ComputeSimpleMatches(query);
	__minus.ComputeSimpleMatches(query);
}

/** 
 * \brief Searching for seeds on a single strand
 *        - The sequence is read using the winfow object
 *        - For each valid window (containing only valid characters) 
 *          the corresponding word is searched in the compatct index
 * \param a reference on a GlintEngineSingleStrand
*/
void GlintEngine::__SearchingForSeeds()
{
	size_t seq_length 				= _query_ptr -> GetSeqLength();
	const unsigned char * seq_base 	= _query_ptr -> GetBase();
	
	Window window(_mask);
	uint8_t msk_len = _mask.GetLength();
	
    // The pos variable corresponds to the first base of the sequence (absolute position)
	for (uint32_t pos=1; pos <= seq_length; pos++)
    {
        char c=*(seq_base + pos);

        if (!Word::IsCode(c)) {     // Masked region
            window.Reset();
        }
        else
        {
            window.Shift(c);        // The complexity is filtered out on the request
            if (window.IsFull()) 
            {
				if ( pos % _parameters.GetStep() == 0 )
				{
					_ctree.CompareWindow(window,pos-msk_len+1,__plus,  *_query_ptr);
					_ctree.CompareWindow(window,pos-msk_len+1,__minus, *_query_ptr);
				}
            }
        }
    }
}

/**
 * \brief Among all the hsps, count best hits and suboptimal hits
 *        and identify the best hsp
 * \pre   The hsp vector must have been recorder first
 */
void GlintEngine::__GetMappingInfos()
{							
	if ( __hspl.empty() )
		return;
						
	Hsp* besthsp = GetBestHsp();
	uint32_t BestScore = besthsp->GetScore();	
		
	_nb_besthits = 0;
	_nb_subopthits = 0;
	
	HspIterator i = __hspl.begin();
	
	while ( i !=  __hspl.end() && ((*i)->GetScore()) == BestScore )
	{
		i++;
		_nb_besthits++;
	}	
	
	uint32_t SubOptimalScore = 0;
	Hsp* secondbest = NULL;
	if ( i != __hspl.end() )
	{
		SubOptimalScore = (*i)->GetScore();	
		secondbest = *i ;
	}
	while ( i !=  __hspl.end() && ((*i)->GetScore()) == SubOptimalScore )
	{
		i++;
		_nb_subopthits++;
	}	
	
	uint16_t mapqual = __BwaLikeApproxMappingQual();
	besthsp->SetMapQuality(	mapqual );
	//_nb_subopthits = __hspl.size() - _nb_besthits;
	
	besthsp->AddMapInfo( Tag(NB_BEST_HITS , _nb_besthits) );
	besthsp->AddMapInfo( Tag(NB_SUBOPT_HITS , _nb_subopthits) );
	besthsp->AddMapInfo( Tag(SUBOPTALIGN_SCORE, SubOptimalScore) );
	
}

/**
 * \brief Computes the mapping quality ot the alignment
 *        
 * \param h       The current hsp
 */
uint16_t GlintEngine::__ComputeMappingQual ( Hsp* first, Hsp* second, uint32_t n2 )
{	
	if ( second == NULL )
		return 99;
	
	uint32_t q1, q2;	
	if ( _query_ptr -> GetQual() == NULL )
	{
		q1 = DEFAULT_QUALITY * first->GetMismatch().GetSize();
		q2 = DEFAULT_QUALITY * second->GetMismatch().GetSize();
	}
	else
	{
		const char* qual = _query_ptr -> GetQual();
		GapPile_const_iterator mismatch;
		
		q1 = 0;
		mismatch = first->GetMismatch().begin();
		for ( uint32_t l=0; l < first->GetSize(); ++l )
		{
			if ( mismatch != first->GetMismatch().end() && *mismatch==l ) 
			{
				q1+=(uint16_t)qual[l];	
				++mismatch;
			}
		}
		q2 = 0;
		mismatch = second->GetMismatch().begin();
		for ( uint32_t l=0; l < second->GetSize(); ++l )
		{
			if ( mismatch != second->GetMismatch().end() && *mismatch==l ) 
			{
				q2+=(uint16_t)qual[l];	
				++mismatch;
			}
		}			
	}
			
	return 	(uint16_t) floor(q2 - q1- 4.343*log(n2));
}

uint16_t GlintEngine::__BwaLikeApproxMappingQual (  )
{	
	if ( _nb_besthits == 0 ) return 23;
	if ( _nb_besthits > 1) return 0;
	if ( _nb_subopthits == 0 ) return 37;
	uint16_t n = ( _nb_subopthits >= 255) ? 255 : _nb_subopthits;
	
	return (23 < _g_log_n[n])? 0 : 23 - _g_log_n[n];	
}

/**
 * \brief Display the Hsps if any
 *         -# Print the hsps, unless already done
 *         -# Reset the hsp-related structures
 * 
 * \param a reference on a QueryBuffer
 * 
 */
void GlintEngine::DisplayHsps ()
{		
	if ( !_parameters.IsHspOnTheFly() && GetNbHsp()>0 )
	{
		__DisplayAll();
		return ;			
	} 	
}

/** 
 * \brief Masking low complexity subsequences
 *        - Calling the ComplexityMasker Run method
 * \param none
*/
void GlintEngine::__MaskLowComplexity()
{
	QueryBuffer& query = *_query_ptr;
	size_t seq_length 				= query.GetSeqLength();
	const unsigned char * seq_base 	= query.GetBase();
	__lowcomplexity_masker.Run( (unsigned char*)(seq_base+sizeof(unsigned char)), seq_length, _parameters.GetComplexity());  // We remove the first $
}

/**
 * \brief Display the hsps of both strands
 *     - Collect the hsp of the two strands
 *     - Give the display object a reference on the query
 *     - Call the Display DisplayHits method
*/
void GlintEngine::__DisplayAll()
{
	RecordHsps();
	
   _display.SetQuery( _query_ptr );
   _display.DisplayHits( __hspl );
}

/**
 * \brief Sort the hsp of both strands
 *     - Collect the hsp of the two strands
 *     - Sort them according to the sorting function given in parameters
 * \param a pointer on a function to be passed to the std::sort method
*/
void GlintEngine::SortingHsps(pt2CmpFunc cmp_fn_ptr)
{
	sort(__hspl.begin(),__hspl.end(),cmp_fn_ptr);	
}

/**
 * \brief Sort the hsp of both strands
 *     - Collect the hsp of the two strands
 *     - Sort them according to the sorting function given in parameters
 * \param a pointer on a function to be passed to the std::sort method
*/
void GlintEngine::RecordHsps()
{
	for (int i=0; i<2; i++) {
        GlintEngineSingleStrand& s_glengine = (i==0?__plus:__minus);
        for (HspIterator k = s_glengine.begin(); k !=  s_glengine.end(); ++k) {
            __hspl.push_back(*k);
        }
    }
    SortingHsps(CompHspScorePtr);
    __GetMappingInfos();   
}

/**
 * \brief Building the hsps for both the __fwd and the __bwd sequence
 *     - If the bestcore flag is set (only the best scored hsps will be considered)
 * 			- Collect the hsps and sort them accoring to their score
 *          - Check the pairing condition for the two best hsp
 *          - Declare the best score as Paired if __BestPaired() 
 *          - remove otherwise the hsps and in anycase all other hsps
*/
void GlintEngineDouble::BuildHsps()
{		
	__BuildHsps();
	__RecordHsps();	
	__IdentifyPairedHsps();
}

/**
 * \brief Calling SortingHsps for __fwd and __bwd
 * \param a pointer on a function to be passed to the std::sort method
*/
void GlintEngineDouble::__RecordHsps()
{	
	__fwd.RecordHsps();
	__bwd.RecordHsps();
}

/**
 * \brief Calling SortingHsps for __fwd and __bwd
 * \param a pointer on a function to be passed to the std::sort method
*/
void GlintEngineDouble::__SortingDoubleHsps(pt2CmpFunc cmp_fn_ptr)
{	
	__fwd.SortingHsps(cmp_fn_ptr);
	__bwd.SortingHsps(cmp_fn_ptr);
}

/**
 * \brief Checks if two hsps are properly paired
 * 		- on the same chromosome, less than MaxDistMate apart and in a reverse orientation
*/
bool GlintEngineDouble::__ArePaired(const Hsp* i, const Hsp* j )
{
	uint32_t distance = min(abs((long)j->s_start - i->s_end), abs((long)i->s_start - j->s_end));
	
	return ( i->Subject.Name == j->Subject.Name &&  distance < _parameters.GetMaxDistMate() && i->GetStrand() != j->GetStrand() ); 	
} 

/**
 * \brief For each hsp pair determines if they form a correct hsp pair
 * 		
*/
void GlintEngineDouble::__IdentifyPairedHsps()
{
	//Sorting the hsps according to their position on the subject
	__SortingDoubleHsps(CompHspSubjPosPtr);
		
	vector<Hsp*>& __fwd_hspl = __fwd.GetHsps();
	vector<Hsp*>& __bwd_hspl = __bwd.GetHsps();
			
	if ( __fwd_hspl.empty() ||  __bwd_hspl.empty() )
		return;
				
	HspIterator j = __bwd_hspl.begin();		
						
	for (HspIterator i = __fwd_hspl.begin(); i !=  __fwd_hspl.end(); ++i)
	{
		uint32_t i_rightneighborhood;
		uint32_t i_leftneighborhood;
		
		i_leftneighborhood = ( (long)(*i)->s_start - _parameters.GetMaxDistMate() > 0 ) ? (long)(*i)->s_start - _parameters.GetMaxDistMate() : 0;
		i_rightneighborhood = (*i)->s_end + _parameters.GetMaxDistMate();		
		
		//Going backwards
		while ( j != __bwd_hspl.begin() &&  ( (*j)->Subject.Name > (*i)->Subject.Name || (*j)->s_end >  i_leftneighborhood ) )
			j--;
		//Going forwards
		while ( j != __bwd_hspl.end() &&  ( (*j)->Subject.Name < (*i)->Subject.Name || (*j)->s_end < i_leftneighborhood ) )
			j++;
		//While still within the _seq neighborhood
		while ( j !=  __bwd_hspl.end() &&  (*j)->Subject.Name == (*i)->Subject.Name && (*j)->s_start <  i_rightneighborhood )
		{	
			if ( (*i)->GetStrand() != (*j)->GetStrand() )
			{					
				MatePair mp(*i,*j);
				__matepairs.push_back(mp);
			}	
			j++;
		}
		
		if ( j ==  __bwd_hspl.end())
			j--;
	}
	
	//Sorting the mate pairs according to the score
	sort(__matepairs.begin(),__matepairs.end(),CompPairHsp);
	
	//Sorting back the hsps according to their score
	//TODO does it slow down the whole procedure ?
	__SortingDoubleHsps(CompHspScorePtr);
		
}

/**
 * \brief Display all the hsps, starting with the pairs
 *        If output format is bed (in fact will be bedpe), call display only for the seq
 *        Else, call display for seq AND for mate
*/
void GlintEngineDouble::__DisplayAll()
{
	
	if ( __fwd.HspEmpty() && __bwd.HspEmpty() )
	{
		return;
	}
	
	__GetMappingInfos();
				
	if ( _parameters.IsBestScore() )
	{

		__DisplayBestHspPair();			
		
		 /// Displaying dangling hsps (meaningfull only if not the bed format)
		if ( _parameters.GetOutputFormat() != OUTPUT_FORMAT_BED ) /// Displaying dangling hsps (meaningfull only if not the bed format)
		{		
			if ( ! __fwd.HspEmpty() )
			{
				Hsp* fwd_besthsp =__fwd.GetBestHsp();
				_display.SetQuery  ( __fwd.GetQueryPtr() );
				_display.DisplayHsp( *fwd_besthsp );
			}
			
			if ( ! __bwd.HspEmpty() )
			{
				Hsp* bwd_besthsp =__bwd.GetBestHsp();
				_display.SetQuery  ( __bwd.GetQueryPtr() );
				_display.DisplayHsp( *bwd_besthsp );
			}
		}									
	}
	else
	{		
		__DisplayAllHspPairs();		
		
		/// Displaying dangling hsps (meaningfull only if not the bed format)
		if ( _parameters.GetOutputFormat() != OUTPUT_FORMAT_BED ) /// Displaying dangling hsps (meaningfull only if not the bed format)
		{		
		
			vector<Hsp*>& __fwd_hspl = __fwd.GetHsps();
			vector<Hsp*>& __bwd_hspl = __bwd.GetHsps();
			
			_display.SetQuery( __fwd.GetQueryPtr() );
			for (HspIterator i = __fwd_hspl.begin(); i !=  __fwd_hspl.end(); ++i)
			{
				if ( !(*i)->IsPaired() )
					_display.DisplayHsp(**i);	
			}
			
			_display.SetQuery( __bwd.GetQueryPtr() );
			for (HspIterator j = __bwd_hspl.begin(); j !=  __bwd_hspl.end(); ++j)
			{
				if ( !(*j)->IsPaired() )
					_display.DisplayHsp(**j);	
			}		
		}
	}			
}

/**
 * \brief Display the best hsp Pair
 *        If output format is bed (in fact will be bedpe), call display only for the seq
 *        Else, call display for seq AND for mate
*/
void GlintEngineDouble::__DisplayBestHspPair()
{
	if ( __matepairs.empty() )
	{
		return;	
	}	
	
	MatePair bestMateHsp = __matepairs[0];	  // mate pairs are sorted according to their score	
	Hsp* fwd_besthsp = bestMateHsp.first;
	Hsp* bwd_besthsp = bestMateHsp.second;
			
	Pair( fwd_besthsp, bwd_besthsp );	
	_display.SetQuery  ( __fwd.GetQueryPtr() );
	_display.DisplayHsp( *fwd_besthsp );
	
	if ( _parameters.GetOutputFormat() != OUTPUT_FORMAT_BED )
	{
		_display.SetQuery( __bwd.GetQueryPtr() );
		_display.DisplayHsp( *bwd_besthsp );	
	}		
}

/**
 * \brief Display all hsp Pairs
 *        If output format is bed (in fact will be bedpe), call display only for the seq
 *        Else, call display for seq AND for mate
 *  \side Warning : side effect, Pair( , ) pairs the hsps (the mate hsp can potentially change for different pairs involving a particular hsp) 
 *                  The fact that an hsp has been paired, irrespective of who was the mate, will be checked prior the display of dangling hsps (selbsvertändlich)
*/
void GlintEngineDouble::__DisplayAllHspPairs()
{
	if ( __matepairs.empty() )
	{
		return;	
	}	
	
	_display.SetQuery( __fwd.GetQueryPtr() );
	for ( MatePairIterator i = 	__matepairs.begin(); i != __matepairs.end() ; i++ ) 
	{
		Pair( i->first, i->second);
		_display.SetQuery( __fwd.GetQueryPtr() );
		_display.DisplayHsp(*(i->first));
		if ( _parameters.GetOutputFormat() != OUTPUT_FORMAT_BED )
		{
			_display.SetQuery( __bwd.GetQueryPtr() );
			_display.DisplayHsp(*(i->second));
		}
	}		
}

/**
 * \todo some comment
 */

void GlintEngineDouble::__GetMappingInfos()
{				
														
	if ( __matepairs.size() > 0 )
	{	
		MatePair bestMatePair = __matepairs[0];
		uint32_t BestPairScore = bestMatePair.first->GetScore()+bestMatePair.second->GetScore();	
				
		MatePairIterator i = __matepairs.begin();	

		_nb_besthits = 0;
		_nb_subopthits = 0;
		while ( i !=  __matepairs.end() && (i->first->GetScore()+i->second->GetScore()) == BestPairScore )
		{
			_nb_besthits++;
			i++;
		}
				
		uint32_t SubOptimalScore = 0;
		if ( i != __matepairs.end() )
		{
			SubOptimalScore = i->first->GetScore() + i->second->GetScore();	
		}
		while ( i !=  __matepairs.end() && (i->first->GetScore()+i->second->GetScore())  == SubOptimalScore )
		{
			_nb_subopthits++;
			i++;
		}	
		//cout<<"size "<<__matepairs.size()<<endl;
		//_nb_subopthits = __matepairs.size() - _nb_besthits;		

		Hsp* fwd_besthsp = bestMatePair.first;
		Hsp* bwd_besthsp = bestMatePair.second;
		
		uint16_t mapqual = ( fwd_besthsp->GetMapQuality() > bwd_besthsp->GetMapQuality() ) ? fwd_besthsp->GetMapQuality() : bwd_besthsp->GetMapQuality();
		
		fwd_besthsp->SetMapQuality(	mapqual );
		bwd_besthsp->SetMapQuality(	mapqual );
							
		if ( fwd_besthsp == __fwd.GetBestHsp() && bwd_besthsp == __bwd.GetBestHsp() )
		{	
			Pair( fwd_besthsp, bwd_besthsp );
			
			fwd_besthsp->AddMapInfo( Tag( NB_BEST_PAIRHITS, _nb_besthits ) );
			fwd_besthsp->AddMapInfo( Tag( NB_SUBOPT_PAIRHITS, _nb_subopthits ) );

			bwd_besthsp->AddMapInfo( Tag( NB_BEST_PAIRHITS, _nb_besthits ) );
			bwd_besthsp->AddMapInfo( Tag( NB_SUBOPT_PAIRHITS, _nb_subopthits ) );
		}	
		
	}
			
}

/**
 * \brief For each hsp pair determines if they form a correct hsp pair
 * 		
*/
void GlintEngineDoublePCR::__IdentifyPairedHsps()
{
	//Sorting the hsps according to their position on the subject
	__SortingDoubleHsps(CompHspSubjPosPtr);
		
	vector<Hsp*>& __fwd_hspl = __fwd.GetHsps();
	vector<Hsp*>& __bwd_hspl = __bwd.GetHsps();
			
	if ( __fwd_hspl.empty() ||  __bwd_hspl.empty() )
		return;
				
	HspIterator j = __bwd_hspl.begin();		
						
	for (HspIterator i = __fwd_hspl.begin(); i !=  __fwd_hspl.end(); ++i)
	{
		uint32_t i_rightneighborhood;
		uint32_t i_leftneighborhood;
		
		if ( (*i)->GetStrand() == 1 )
		{			
			i_leftneighborhood = (long)(*i)->s_end;
			i_rightneighborhood = (*i)->s_end + _parameters.GetMaxDistMate();				
		}
		else
		{
			i_leftneighborhood = ( (long)(*i)->s_start - _parameters.GetMaxDistMate() > 0 ) ? (long)(*i)->s_start - _parameters.GetMaxDistMate() : 0;
			i_rightneighborhood = (*i)->s_start;	
		}

		//Going backwards
		while ( j != __bwd_hspl.begin() &&  ( (*j)->Subject.Name > (*i)->Subject.Name || (*j)->s_end >  i_leftneighborhood ) )
			j--;
		//Going forwards
		while ( j != __bwd_hspl.end() &&  ( (*j)->Subject.Name < (*i)->Subject.Name || (*j)->s_end < i_leftneighborhood ) )
			j++;
		//While still within the _seq neighborhood
		while ( j !=  __bwd_hspl.end() &&  (*j)->Subject.Name == (*i)->Subject.Name && (*j)->s_start <  i_rightneighborhood )
		{	
			if ( (*i)->GetStrand() != (*j)->GetStrand() && !_OverlappingHsps( *i, *j) )
			{					
				MatePair mp(*i,*j);
				__matepairs.push_back(mp);
			}	
			j++;
		}
		
		if ( j ==  __bwd_hspl.end())
			j--;
	}
	
	//Sorting the mate pairs according to the score
	sort(__matepairs.begin(),__matepairs.end(),CompPairHsp);
	
	//Sorting back the hsps according to their score (is not necessary anymore)
	//TODO does it slow the whole procedure
	//__SortingDoubleHsps(CompHspScorePtr);	
}

bool GlintEngineDoublePCR::_OverlappingHsps(Hsp* i,Hsp* j) 
{		
	if ( j->s_start <= i->s_end && j->s_end >= i->s_start )
			return true;	
	return false;
}

/**
 * \brief GlintEngineSingleStrand constructor
 * \param The strand
 * \param A reference on the display object (property of the GlintEngine object)
 */
GlintEngineSingleStrand::GlintEngineSingleStrand(int st,const Display& d):
		__parameters(Parameters::Instance()),
		//__bank(Bank::Instance()),
		__strand(st),
		__top(0),
		__HP(d),
		__hsp_cnt(0),
		__align_engine_aptr(__InitAlignEnginePtr()),
		__align_engine(*__align_engine_aptr),
		__display(d),
		__match_scoring(MatchConstants::Instance()),
		__bit_count( BitCount::Instance() )
{
    p = new match[MATCH_PILE_SIZE];
    for (uint32_t i=0; i < MATCH_PILE_SIZE;i++)
    {
        l[i] = &(p[i]);
    }
}

/* Some iterators, Reset etc */

HspIterator GlintEngineSingleStrand::begin()  {return __HP.begin();}
HspIterator GlintEngineSingleStrand::end()    {return __HP.end();}
void GlintEngineSingleStrand::HspReset()      {__HP.Reset(); __hsp_cnt = 0; }


/**
 * \brief Adds an element to the seed pile, compute the matches is the pile is full
 * \param spos the position on the subject of the seed
 * \param qpos the position on the query of the seed 
 * \param the hit value of the seed (depending on the level : equals the number of matching position of the mask)
 * \param a reference on the QueryBuffer
 * \return the current size of the seed pile
 */
uint32_t GlintEngineSingleStrand::Push(uint32_t spos,uint32_t qpos, uint8_t hit, const QueryBuffer& query)
{
    if ( __top == MATCH_PILE_SIZE )
    	ComputeSimpleMatches(query);
 
    match *m = l[__top++];
    m->spos=spos;
	m->hit=hit;
	m->qpos = qpos;
	m->score = __match_scoring.score_matrix[hit];
	m->diag  = (DiagInt)m->spos-__strand*(DiagInt)m->qpos;
    
    return __top;
}

/**
 * \brief mark the seed pile as "empty"
 */
void GlintEngineSingleStrand::MatchReset()
{
    __top=0;
}

/**
 * \brief Initialize the AlignEngine object according to the selected behavior (Self or not)
 */
auto_ptr<AlignEngine> GlintEngineSingleStrand::__InitAlignEnginePtr()
{
	return (__parameters.IsSelf()) ? auto_ptr<AlignEngine> (new AlignEngineSelf( __strand )) : auto_ptr<AlignEngine> (new AlignEngine( __strand ));
}
	

/**
 * \brief Computes the hsp
 * 		(i)  sort the matches, according to diagonals first and then subject
 * 		(ii) slide across the diagonals to identify regions with a concentration of seeds
 *      (ii) Reset the seed pile
 * \param a reference on the QueryBuffer
 */
void GlintEngineSingleStrand::ComputeSimpleMatches(const QueryBuffer& query)
{	
	if ( __IsNotEmpty() )
	{
		__QSortSeeds();
		__ScoreSlidingDiagonal(query);
		MatchReset();
	}
}

/**
 * \brief Slide across the diagonals to identify regions with a concentration of seeds
 *				(i) the score of overlapping seeds are calculated using bitwise operations 
 *					__bit_count.GetBitCount() gives the number of 1s in the bscore bit array
 *				(ii) when the overlapping seeds extend the initial window size, bitwise operation enable to keep track of previous seed scores
 *				(iii) when they are not overlapping but close, we performe a temptative alignment and if unsuccessful we keep track of the score of the previous seeds
 *				(iv)  when they are not overlapping and far away, we performe a temptative alignment and the score is reset to zero
 *				(v)   when a new diagonal is visited, we perform a temptative alignment and the score is reset to zero
 * \param a reference on the QueryBuffer used
*/
void GlintEngineSingleStrand::__ScoreSlidingDiagonal(const QueryBuffer& query)
{
	match **m=l;
	uint32_t n=__top;
	
	const Mask& M= Mask::Instance();
	uint8_t Window_Size=M.GetLength();

	// Initialization of the variables LastDiag, SubjectStart and SubjectEnd
	LastDiag     = numeric_limits<DiagInt>::min();
	SubjectEnd   = 0;
    SubjectStart = numeric_limits<uint32_t>::max();

	uint32_t start		 = 0;
	uint32_t ChunkStart = start;

	uint64_t bscore     = M.bm[m[start]->hit] << Window_Size;
	DiagInt	CurrentDiag = m[start]->diag;
	uint32_t	score=0;

	int offset	= 0;
	uint32_t j		= 0;
	bool found	= 0;	
	
	while ( start <= n-1 ) 
	{
		j=start+1;
		found=0;
		
		// Adding all overlapping seeds to bscore (i)
		while ( j < n && m[j]->diag == CurrentDiag && (( offset=m[j]->spos - m[start]->spos ) < Window_Size)) 
		{
				bscore |= ( M.bm[m[j]->hit]<<(Window_Size-offset) );
				j++;
		};
		// Transforming the bscore into a score
		//score += precomputed16_bitcount( bscore );
		score += __bit_count.GetBitCount( bscore );
		
		if ( j == n )
		{		
			if ( score >= __parameters.GetCutoffSlide() )
			{ 
				found=__LocateMatchesChunk( m, ChunkStart , j-1 , score, query);
			}
			break;
		}
		
		if (  m[j]->diag != CurrentDiag )// (v)
		{
			if ( score >= __parameters.GetCutoffSlide() ) 
			{	
				found=__LocateMatchesChunk( m, ChunkStart, j-1, score, query);
			}
			
			if ( found ) {
				while ( j < n  &&  m[j]->diag <= LastDiag  && m[j]->spos >= SubjectStart && m[j]->spos <= SubjectEnd ) 
				{   /* Jumping to the next chunk  */
					j++;
				}
				if (j==n) break; // There is no further chunk !!
			}
			start = j;
			ChunkStart = start;
			score = 0;
			bscore = M.bm[m[start]->hit] << Window_Size;
			CurrentDiag = m[start]->diag;
		}
		else if ( m[j]->spos - m[j-1]->spos >= Window_Size )
		{
			if ( score >= __parameters.GetCutoffSlide() ) 
			{
				found=__LocateMatchesChunk( m, ChunkStart, j-1, score, query);
			}

			if ( found ) 
			{
				while ( j < n  &&  m[j]->diag <= LastDiag  && m[j]->spos >= SubjectStart &&  m[j]->spos <= SubjectEnd ) 
				{     /* Jumping to the next chunk  */		
					j++;
				}
				if (j==n) break;    // There is no further chunk !!

				start = j;
				ChunkStart = start;
				score=0;			
			} else {
				if ( m[j]->spos - m[j-1]->spos > 100 ) //(iv) 
				{
					start = j;
					ChunkStart = start;
					score=0;
				} else //(iii) 
				{
					ChunkStart = start;
					start = j;
				}
			}

			bscore = M.bm[m[start]->hit] << Window_Size;
		}
		else  //(ii)
		{	
			// We keep track of the score of the previous window except the one overlapping the current window 
			//score += precomputed16_bitcount( bscore >> Window_Size );
			score += __bit_count.GetBitCount( bscore >> Window_Size );
			ChunkStart = start;
			start = j-1;
			bscore <<= Window_Size;
		
		}
	}

}

/**
 * \brief Given a set of seeds in a defined region, try to compute a local alignment
	On a segment of a diagonal, tries to identify an aligment in several steps
		i) Identify the match (seed) with the best score 
		ii) Starting from this word (seed) a check is made if it doesn't correspond to a previous alignment (__HP.Lookup)
		iii) If ok extend on the right (look_right) and on the left (look_left)
		iv) If the resulting ungapped alignment has a score > __parameters.Cutoff_Dyn, a dynamic programming alignment is tried on the right and on the left
		v) If the resulting possibly gapped alignment has a score > __parameters.Cutoff_Hsp one checks if it is not a previous recorded alignment (__HP.Lookup again)
		vi) If ok record it or print it depending on IsHspOnTheFly
*/

bool GlintEngineSingleStrand::__LocateMatchesChunk( match **m, int min, int max, int16_t score, const QueryBuffer& query)
{
	bool found = false;
	char sign;
	unsigned char bstrand;

	if ( __strand > 0 ) 
	{
		sign='+';
		bstrand=0;
	} else {
		sign='-';
		bstrand=1;
	}

	/* step i) Selecting the match with the best score */
	int max_score=0;
	int ind_maxscore=0;
	for (int i=min;i<=max;i++) 
	{
		if ( m[i]->hit > max_score ) 
		{
			max_score=m[i]->hit;
			ind_maxscore=i;
		}
	}
	match* best_match = m[ind_maxscore];
	

#ifdef DEBUGLOG_REMOVED

	_DebugPrintSelMatches( m, min, max , score);
#endif

	/* step ii)   Checking if best_match is not already part of an existing hsp*/
	if (__HP.Lookup(*best_match))    // Return false if we detect an overlap with another hsp
	{	
#ifdef DEBUGLOG2
		logger -> SetLogType( "MV" ) << "*** This seed is already included in a hsp\n";
#endif
		return false;
	}

#ifdef DEBUGLOG_REMOVED
	_DebugPrintTopMatch( m, ind_maxscore);
#endif

	/* Get an hsp from the HspPile */
	Hsp& h    = __HP.GetTop();

	/* step iii), iv), v) Performing an ungapped, then gapped alignment, and filtering the returned hsp */

	DiagInt max_diago;  // Will be initialized by __align_engine
	if ( __align_engine.Align(best_match, h, max_diago, query) )
	{
		/* step vi) Insert this hsp in the pile, unless it overlaps with a previous one */
		if (__HP.Lookup(h))  // Checking if the hsp is not already part of an existing hsp
		{
			found = false;
		}
		else
		{
			// Display the hsp if necessary
			if (__parameters.IsHspOnTheFly())
			{
				__display.DisplayHsp(h);
			}
			
			/*    TODO à déplacer dans ScoreSlidingDiagonal */
			LastDiag     = ( max_diago > LastDiag ) ? 	max_diago	:	LastDiag;
			SubjectStart = __align_engine.GetSubjectStart();
			SubjectEnd   = __align_engine.GetSubjectEnd();

			__HP.Commit();

			found = true;
			
			__hsp_cnt++;

		}

#ifdef DEBUGLOG
		h.LogHsp( "M", "hsp    " );
		__HP.LogPile ( "MV" );
#endif	
	}
	return found;
}

/**
 * \brief Sorting seeds using an in-line qsort implementation ( iqsort.h )
 * 
 */

void GlintEngineSingleStrand::__QSortSeeds()
{
#define elt_sort(a,b) ((*a)->diag)==((*b)->diag) ? (((*a)->spos)<((*b)->spos)) : (((*a)->diag)<((*b)->diag))
 QSORT(struct match*,l,__top,elt_sort);
}


void GlintEngineSingleStrand::DisplaySeeds()
{
#define voir(a,b) printf("%d\t%u\t%u\t%u\n",b,a->qpos,a->spos,a->diag)
 uint32_t i;
 for (i=0;i<__top;i++) {
 	voir((int)l[i],i);
 };

}

#ifdef DEBUGLOG
void _DebugPrintSelMatches(match **m, int min, int max,int16_t score)
{
	for ( int i = min ; i <= max ; i++ )
	{
		logger -> SetLogType("MV") << "lookup     " << (int) m[i]->hit << '\t' <<  m[i]->diag << '\t' << m[i]->qpos << '\t' << m[i]->spos << '\n';
	}
}
void _DebugPrintTopMatch(match **m, int ind_max)
{
	logger -> SetLogType("M") << " lookup_max " << (int) m[ind_max]->hit << '\t' << m[ind_max]->diag << '\t' << m[ind_max]->qpos << '\t' << m[ind_max]->spos << '\n';
}
#endif
