/*
   $Id: alignment.cpp 649 2011-10-31 13:57:42Z manu $
*/

/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#include <ctype.h>
#include "utils.h"
#include "alignment.h"
#include "QueryBuffer.h"
#include "CompactTree.h"
#include "Comparison.h"

extern RealLogger* logger;

/*
     Constructors and other initialisation functions
*/

AlignEngine::AlignEngine(int strand):
	__strand(strand),
	__strand01(strand==1?0:1),
	__parameters(Parameters::Instance()),
	__bank(Bank::Instance()),
	__mask(Mask::Instance()),
	_bestscore(0),
	scoring(AlignConstants::Instance()),
	ctree(CompactTree::Instance())
{
	__InitScoreMatrix();
	__InitTraceMatrix();

	for (int i=0;i<256;i++) {
	   Rev[i] = i;
	}
	Rev['A'] = 'T';
	Rev['T'] = 'A';
	Rev['C'] = 'G';
	Rev['G'] = 'C';
	Rev['a'] = 't';
	Rev['t'] = 'a';
	Rev['c'] = 'g';
	Rev['g'] = 'c';

	Rev['N'] = 'N';
}

void AlignEngine::__InitScoreMatrix()
{
	uint32_t size = DYNPROG_MAT_SIZE;
	
	for (uint32_t i=1;i<=size;i++) {
		S[i][0] = __parameters.GetGapOpenPenality() + (i-1) * __parameters.GetGapExtenPenality();
		S[0][i] = __parameters.GetGapOpenPenality() + (i-1) * __parameters.GetGapExtenPenality();
	}

	S[0][0] = 0;
}


void AlignEngine::__InitTraceMatrix()
{
	memset(Trace  , 0, sizeof(Trace));
	memset(Trace_E, 0, sizeof(Trace_E));
	memset(Trace_F, 0, sizeof(Trace_F));
		
	for (uint32_t i=0; i <= DYNPROG_MAT_SIZE;i++) {
		Trace   [0][i] = LEFT;
		Trace_E [0][i] = CONT_GQ;
		Trace   [i][0] = ABOVE;
		Trace_F [i][0] = CONT_GS;
	}

	Trace_E [0][1] = OPEN_Q;
	Trace_F [1][0] = OPEN_S;

	Trace   [0][0] = DONE;
	Trace_E [0][0] = DONE;
	Trace_F [0][0] = DONE;
}

/**
 * \brief Fill the Hsp passed by argument, using the data found in AlignEngine
 * \param h [out] The hsp to fill
 * \param query The QueryBuffer
 * \return The largest diagonal implied in the alignment, is returned
*/

DiagInt AlignEngine::__ToHsp(Hsp & h, const QueryBuffer& query)
{
	uint8_t msk_length = __mask.GetLength();
	
    h.q_start= _q_start;
    h.q_end  = _q_end;
    h.__size   = h.q_end-h.q_start+1+h.q_gap.GetSize();

    h.s_start= _s_start;
    h.s_end  = _s_end;
    h.__score  = _a_score;
	h.__strand = __strand;
#ifdef DEBUGLOG
   logger->SetLogType("A") << "Storing hsp h : ["<<h.q_start <<"-"<<h.q_end<<"]->["<<h.s_start<<"-"<<h.s_end<<"]\n";
#endif

	 /*      Display align_results. and alignment      */

    DiagInt diago1 = ( __strand >0 ) ? (long) _s_start-_q_start : _s_start+_q_end-msk_length+1;
    DiagInt diago2 = ( __strand >0 ) ? (long) _s_end-_q_end : _s_end+_q_start-msk_length+1;

    DiagInt max_diago;

    if ( diago1 > diago2 ) {
        h.min_diag = diago2 - h.s_gap.GetSize();
        h.max_diag = diago1 + h.q_gap.GetSize();
        max_diago  = diago1;
    } else {
        h.min_diag = diago1 - h.s_gap.GetSize();
        h.max_diag = diago2 + h.q_gap.GetSize();
        max_diago  = diago2;
    }

	//Getting the actual sequence using information from the .he file
    __GetSeqInfo(h,query);

    //Recording the hsp shortest edit script
    __RecordHspScript(h,query);    
    
    return max_diago;
}

/**
 * \brief Called by _DisplayHspInfo, records the smallest script for describing the hsp 
 * \param h an hsp pointer
 * \param query A query buffer
 * \return The size of the alignment
 * 
*/
uint16_t AlignEngine::__RecordHspScript(Hsp& h, const QueryBuffer& query) const
{
	uint32_t l = 0;
	const unsigned char *nQuery = query.GetBase()+h.q_start;
	const unsigned char *nBase = (h.__strand>0) ? ctree.GetSeq()+h.s_start : ctree.GetSeq()+h.s_end;
	char base;
	AlignStatus align_status = ALIGNED;

	//h.q_gap.Sort();
	//h.s_gap.Sort();
    GapPile_const_iterator qgap = h.q_gap.begin();
    GapPile_const_iterator sgap = h.s_gap.begin();

	while (  l < h.__size ) {
		if (qgap!=h.q_gap.end() && *qgap==l) {
			if (align_status == ALIGNED)
			{
				h.gap_opening.Push(l);
				align_status = GAPED;
			}
			nBase+=h.__strand;
			qgap++;

		} else if (sgap!=h.s_gap.end() && *sgap==l) {
			if (align_status == ALIGNED)
			{
				h.gap_opening.Push(l);
				align_status = GAPED;
			}
			nQuery++;
			sgap++;
		} else {
			base=(h.__strand>0) ? *nBase : Rev[*nBase];
			if (toupper(*nQuery)!=toupper(base)) h.mismatch.Push(l);
			nBase+=h.__strand;
			nQuery++;
			align_status = ALIGNED;
		}
		l++;
	}

	return h.__size;
}


/**
 * \brief Retrieve and keep to the hsp the names and positions of the sequences, from their position in the QueryBuffer
 * \param [out] h The hsp to populate
 * \param query The QueryBuffer
 */
void AlignEngine::__GetSeqInfo(Hsp & h, const QueryBuffer& query)
{
	h.Subject     = __bank.GetTrueSeqPos(h.s_start);
	h.Query.Name  = (char*)query.GetSeqId().c_str();
	h.Query.start = 1;
	h.Query.end   = query.GetSeqLength();		
}

/**
 * \brief Redefinition of __GetSeqInfo for self comparison
 * \param The hsp for which the Query positions have to be set
 */
void AlignEngineSelf::__GetSeqInfo(Hsp & h, const QueryBuffer& query)
{
	h.Subject  = __bank.GetTrueSeqPos(h.s_start);
	h.Query    = __bank.GetTrueSeqPos(h.q_start);
		
#ifdef DEBUGLOG		
		if (h.q_end==h.s_end || h.q_start==h.s_start) 
		{
				//fprintf(stderr,"Warning Here main diagonal [%u-%u]-->[%u-%u]\n",h.q_start,h.q_end,h.s_start,h.s_end);
		}
#endif

}


/**
 * \brief Check the hsp against several parameters
 * \param h The hsp to check
 * \return true is OK, fals if not
 * \todo METTRE CETTE FONCTION EN TANT QUE METHODE DE Hsp ????
 */

bool AlignEngine::__ValidHsp(Hsp& h, const QueryBuffer& query)
{
	bool IsValid = true;

	if ( __parameters.GetHspMaxMismatch() >=0 && h.mismatch.GetSize()>__parameters.GetHspMaxMismatch() )
		IsValid = false;
	if ( h.__size < __parameters.GetHspMinSize() )
		IsValid = false;
	if ( __parameters.GetHspMaxGap() >=0 && (h.q_gap.GetSize()+h.s_gap.GetSize())>__parameters.GetHspMaxGap() )
		IsValid = false;
	// if ( __parameters.GetHspMinRelScore() >=0 &&  h.score < (__parameters.GetHspMinRelScore() * query.MaxScore( scoring.GetMatch() )))
	if ( __parameters.GetHspMinRelScore() >=0 &&  h.__score < (__parameters.GetHspMinRelScore() * query.MaxScore(__parameters.GetNMatchPenality())))
		IsValid = false;
	if ( __parameters.GetHspMinRelLength() >=0 &&  h.__size < (__parameters.GetHspMinRelLength() * query.GetSeqLength()))
		IsValid = false;

	return IsValid;
}

/**
 * \brief Perform an ungapped alignment, then if necessary a gapped alignment
 *        Return true if the hsp is valid and conserved, false if not (score too low, hsp not validated)
 * \param starting_match The match to start from
 * \param hsp [out] The resulting hsp, if any
 * \param max_diag [out] The largest diagonal implied in the alignment (see __ToHsp)
 * \param query The QueryBuffer
 * \return If true, a valid hsp and a valid diagonal are returned through hsp and max_diag
 */
bool AlignEngine::Align(match* starting_match, Hsp& hsp, DiagInt& max_diag, const QueryBuffer& query)
{
	bool result  = false;	// will be returned by the function
	
	// Init data
	_a_size = 0;
		
	int query_offset = ( _GetStrand() > 0 ) ? 1 : -1;	
	// Look at each side of starting match
	_a_score  = __LookRight (starting_match -> qpos,           starting_match -> spos,   query);
	_a_score += __LookLeft  (starting_match -> qpos - query_offset, starting_match -> spos - 1 , query );

#ifdef DEBUGLOG
	LogScore( "M" );
#endif		
	
	// If we want dynamic programming and if the score is high enough, we go...
	if ( __parameters.IsDynProgramming() &&  _a_score >= __parameters.GetCutoffDyn() )
	{

#ifdef DEBUGLOG
		LogAlign( "M", " before     " );
#endif		
	
		/* Alignment on the left side    */
		__SetDirection(UPSTREAM);
		__LocalAlign(hsp,query);
	
#ifdef DEBUGLOG
		LogAlign( "M", " left     " );
#endif	
		_a_size += bestSi_right - bestSi_left + 1;
	
		/* Alignment on the right side    */
		__SetDirection(DOWNSTREAM);
		__LocalAlign(hsp,query);			
#ifdef DEBUGLOG
		LogAlign( "M", " right     " );
#endif
	}
		
	/* If the score is good enough keep this alignment in an hsp */
	if ( _a_score >=  __parameters.GetCutoffHsp() )
	{
		max_diag = __ToHsp(hsp,query);
	
		/* Validate the hsp */	
		if ( __ValidHsp(hsp,query) )
		{
			result = true;
		}
	}
	
	return result;
}

uint32_t AlignEngine::__LocalAlign(Hsp& hsp, const QueryBuffer& query)
{
	
	__InitDPparameters(query);
			
	int max_line_score=0;
	int query_move = __GetDPMatrixQueryMove();
		
	I=1;
	while ( max_line_score > dropoff && I < max_query_size )  {
		   		 				     
		max_line_score = _ComputeDPMatrixLine();		
		
		if ( ! _ValidDPMatrixline() )
			break;
		
		nQuery += query_move;
		I++;
	}
	
	if ( _bestscore > 0 ) 
	{										
		_a_size   = __RecordAlignment(hsp);
		_a_score  += _bestscore;
		
		if ( Jrange.max == DYNPROG_MAT_SIZE ) 
		{			
			_bestscore += __LocalAlign(hsp,query);
		}
	}

	return _a_score;
}

void AlignEngine::__InitDPparameters(const QueryBuffer& query)
{
	Jrange.min = 1;
	Jrange.max = 0;
			
	// The scoring matrix depends on the strand
	//scoring.score= ( _GetStrand() > 0 ) ? scoring.score_fwd : scoring.score_bwd;
	
	// Initialization of the bestscore and best cells
	_bestscore = 0;
	bestSi    = 0;
	bestSj    = 0;
	dropoff   = _bestscore - __parameters.GetGapDropoff();
	
	// Initialization of the pointers on the DP matrix
	sdiag     = S[0];
	sleft     = S[1];
	sabove    = sdiag + 1;
	scurrent  = sleft + 1;
			
	// Setting Jrange.max
	for ( int i = 0; S[0][i] > dropoff-1; i++) Jrange.max++;
	
	// Matrix (array) F Initialization
	for ( int i = Jrange.min ; i <= DYNPROG_MAT_SIZE; i++ ) F[i]=N_INFTY;

	//The Query sequence (vertical axis) indexed by I (the lines)
	nQuery=query.GetBase() + __GetDPMatrixQueryStart();	
	
	// The actual sequence begins at position 1 in the array just as I below so we have to add an extra 1
	// so that nQuery=__GetDPMatrixQueryStart()+(I-1) can visit first and last nucleotides
	max_query_size = ( __GetDPMatrixQueryMove() > 0 ) ? query.GetSeqLength() - __GetDPMatrixQueryStart() + 2 : __GetDPMatrixQueryStart() + 1;
	
	//The subject sequence (horizontal axis) indexed by J (the columns)
	uint8_t msk_length = __mask.GetLength();
	max_subject_size= ( __GetDPMatrixSubjectMove() > 0 ) ? msk_length - __GetDPMatrixSubjectStart() : __GetDPMatrixSubjectStart();
	
	if ( max_subject_size > DYNPROG_MAT_SIZE ) max_subject_size = DYNPROG_MAT_SIZE;
	if ( max_query_size > DYNPROG_MAT_SIZE ) max_query_size = DYNPROG_MAT_SIZE;

#ifdef DEBUGLOG
	logger -> SetLogType( "A" ) << __GetDPMatrixQueryStart()<< "  " << __GetDPMatrixSubjectStart() <<" "<< max_subject_size<< " "<< max_subject_size<<"\n";
#endif			
}


int AlignEngine::_ComputeDPMatrixLine()
{
	int max_line_score=dropoff-1;

	sdiag=S[I-1]+Jrange.min-1;
	sleft=S[I]+Jrange.min-1;
	sabove=sdiag+1;
	scurrent=sleft+1;  	

	fabove = &F[Jrange.min];
	E=N_INFTY;
	
	const unsigned char QueryNuc = *nQuery;
	nSubject = ctree.GetSeq() + __GetDPMatrixSubjectStart() + __GetDPMatrixSubjectMove()*(Jrange.min-1);
	int subject_move = 	__GetDPMatrixSubjectMove();	
			
	J=Jrange.min;  		
	do
	{
		if ( J > Jrange.max ) *sabove=N_INFTY;
						
		// The recurrence formulae
		// E         = max2_E( *sleft + scoring.alpha, E + scoring.beta );
		//*fabove   = max2_F( *sabove + scoring.alpha , *fabove + scoring.beta);
		E         = max2_E( *sleft  + __parameters.GetGapOpenPenality(), E       + __parameters.GetGapExtenPenality() );
		*fabove   = max2_F( *sabove + __parameters.GetGapOpenPenality(), *fabove + __parameters.GetGapExtenPenality() );
		//*scurrent = max3( *sdiag + scoring.score[ QueryNuc ][ *nSubject ], E, *fabove);
		*scurrent = max3( *sdiag + scoring.GetScore( _GetStrand01(),QueryNuc,*nSubject), E, *fabove);
													
		if ( *scurrent < dropoff ) 
		{
			*scurrent=N_INFTY;
		} 
		else if ( *scurrent >= _bestscore ) 
		{
			_bestscore = *scurrent;
			bestSi    = I;
			bestSj    = J;
			max_line_score = _bestscore;
			dropoff        = _bestscore - __parameters.GetGapDropoff();
		}
		else if (*scurrent > max_line_score) 
		{
			max_line_score = *scurrent;
		}
			
		nSubject += subject_move;
		J++,sdiag++,sabove++,sleft++,scurrent++,fabove++;
	} 
	while ( (*(scurrent-1) > dropoff || J <= Jrange.max ) && J < max_subject_size );
	
	// Updating Jrange.max
	Jrange.max= ( J < max_subject_size ) ?  J : max_subject_size;

	//E= max2_E( *sleft+scoring.alpha, E+scoring.beta );
	E = max2_E( *sleft + __parameters.GetGapOpenPenality(), E + __parameters.GetGapExtenPenality() );
	F[ Jrange.max ] = N_INFTY;		
	//*scurrent = max3( *sdiag + scoring.score[*nQuery][*nSubject] , E, *fabove);
	*scurrent = max3( *sdiag + scoring.GetScore( _GetStrand01() ,*nQuery,*nSubject), E, *fabove);
		
	if ( *scurrent < dropoff ) *scurrent = N_INFTY;
		
	//Updating Jrange.min
	if ( S[I][Jrange.min] < dropoff ) 
	{
		S[I+1][Jrange.min] = N_INFTY;
		Jrange.min++;
	}
	if ( Jrange.min > 1 ) 
	{
		S[I+1][Jrange.min-1] = N_INFTY;
	}
	
	return max_line_score;
	
}

bool AlignEngine::_ValidDPMatrixline( )
{
	return true;
}

bool AlignEngineSelf::_ValidDPMatrixline( )
{
	bool IsValid = true;
	
	uint32_t BestScoreSubjectIndex = __GetDPMatrixSubjectStart() + __GetDPMatrixSubjectMove() * (bestSj - 1);	
	uint32_t BestScoreQueryIndex   = __GetDPMatrixQueryStart() + __GetDPMatrixQueryMove() * (bestSi - 1);

	if ( BestScoreQueryIndex == BestScoreSubjectIndex ) //Here the main diagonal
	{
		IsValid = false;
	}
			
	uint32_t MinSubject = ( __GetDPMatrixSubjectMove() == DOWNSTREAM ) ? Jrange.min - 1 : 1 - Jrange.max; 		
	MinSubject += __GetDPMatrixSubjectStart();

	uint32_t MaxQuery = ( __GetDPMatrixQueryMove() > 0 ) ? I -1 : 0;
	MaxQuery += __GetDPMatrixQueryStart() 	;

	if ( MinSubject - 2 <= _q_end || MaxQuery + 2 >= _s_start ) //Overlapping alignment +1 and -1 in order to be strict (not even a single nucleotide overlap)
	{			
		IsValid = false;	
	}	
	
	return 	IsValid;	
}

/*
     Trying to align on the right side, not allowing gaps, using a dropoff principle 
     (continue while the score does not drop X away from the current best score).
        - Version AlignEngine ==> NOT USED FOR SELF COMPARISON
*/
int AlignEngine::__LookRight(uint32_t i, uint32_t j, const QueryBuffer& query)
{
	uint8_t msk_length = __mask.GetLength();

	int16_t  score = 0;
    int   X     = __parameters.GetUnGapDropoff();

	_bestscore=0;

	if ( _GetStrand() > 0 ) 
	{
		bestSi_right=i;
		bestSj_right=j;

		while(score>_bestscore-X) 
		{
			if (score>=_bestscore) {
				_bestscore=score;
				bestSi_right=i-1;
				bestSj_right=j-1;
			}
			//score+=scoring.score_fwd_severe[query.Getc(i++)][ctree.Getc(j++)];
			score += scoring.GetScoreSevere( _GetStrand01() ,query.Getc(i++),ctree.Getc(j++));
		}
		_q_start=0;
		_q_end=bestSi_right;
		_s_start=0;
		_s_end=bestSj_right;
	} 
	else 
	{
		i+=msk_length-1;
		bestSi_left=i;
		bestSj_right=j;

		while(score>_bestscore-X) 
		{
			if (score>=_bestscore) 
			{
				_bestscore=score;
				bestSi_left=i+1;
				bestSj_right=j-1;
			}
			//score+=scoring.score_bwd_severe[query.Getc(i--)][ctree.Getc(j++)];
			score += scoring.GetScoreSevere(  _GetStrand01(),query.Getc(i--),ctree.Getc(j++));
		}
		_q_start=bestSi_left;
		_q_end=0;
		_s_start=0;
		_s_end=bestSj_right;
	}

	return _bestscore;
}

/*

     Trying to align on the right side, not allowing gaps, using a dropoff principle 
     (continue while the score does not drop X away from the current best score).
        - Version AlignEngineSelf ==> USED ONLY FOR SELF COMPARISON

*/
int AlignEngineSelf::__LookRight(uint32_t i, uint32_t j, const QueryBuffer& query)
{
	uint8_t msk_length = __mask.GetLength();

	int16_t score=0;
    //int X = scoring.X1;
    int X = __parameters.GetUnGapDropoff();

	_bestscore=0;

	uint32_t start_j = j;		

	if ( _GetStrand() > 0 ) 
	{
		bestSi_right=i;
		bestSj_right=j;

		while(score>_bestscore-X) 
		{
			if ( i >= start_j - 1 ) //Stop before the overlap
			{
				score = -10000;			
			}
			if (score>=_bestscore) {
				_bestscore=score;
				bestSi_right=i-1;
				bestSj_right=j-1;
			}
			//score+=scoring.score_fwd_severe[query.Getc(i++)][query.Getc(j++)];
			score += scoring.GetScoreSevere( _GetStrand01(),query.Getc(i++),query.Getc(j++));
		}
		_q_start=0;
		_q_end=bestSi_right;
		_s_start=0;
		_s_end=bestSj_right;
	} 
	else 
	{
		i+=msk_length-1;
		bestSi_left=i;
		bestSj_right=j;

		while(score>_bestscore-X) {
			if (score>=_bestscore) {
				_bestscore=score;
				bestSi_left=i+1;
				bestSj_right=j-1;
			}
			//score+=scoring.score_bwd_severe[query.Getc(i--)][ctree.Getc(j++)];
			score += scoring.GetScoreSevere( _GetStrand01(),query.Getc(i--),ctree.Getc(j++));
		}
		_q_start=bestSi_left;
		_q_end=0;
		_s_start=0;
		_s_end=bestSj_right;
	}
	return _bestscore;
}

/*

 Trying to align on the left side, not allowing gaps, using a dropoff principle.
        - Version AlignEngine ==> NOT USED FOR SELF COMPARISON

*/

int AlignEngine::__LookLeft(uint32_t i, uint32_t j, const QueryBuffer& query)
{
	uint8_t msk_length = __mask.GetLength();

	int score = 0;
	//int X     = scoring.X1;
	int X     = __parameters.GetUnGapDropoff();

	_bestscore = 0;

	if ( _GetStrand() > 0 ) 
	{
		bestSi_left = i;
		bestSj_left = j;
		while( score > _bestscore-X ) 
		{
			if (score >= _bestscore) 
			{
				_bestscore = score;
				bestSi_left = i+1;
				bestSj_left = j+1;
			}
			//score+=scoring.score_fwd_severe[query.Getc(i--)][ctree.Getc(j--)];
			score+=scoring.GetScoreSevere( _GetStrand01(),query.Getc(i--),ctree.Getc(j--));
		}
		_q_start = bestSi_left;
		_s_start = bestSj_left;
	} 
	else 
	{
		i+=msk_length-1;
		bestSi_right = i;
		bestSj_left = j;
		while(score>_bestscore-X) 
		{
			if (score>=_bestscore) 
			{
				_bestscore = score;
				bestSi_right = i-1;
				bestSj_left = j+1;
			}

			//score+=scoring.score_bwd_severe[query.Getc(i++)][ctree.Getc(j--)];
			score += scoring.GetScoreSevere( _GetStrand01(),query.Getc(i++),ctree.Getc(j--));
		}
		_q_end = bestSi_right;
		_s_start = bestSj_left;
	}

	return _bestscore;
}

/*

 Trying to align on the left side, not allowing gaps, using a dropoff principle.
        - Version AlignEngineSelf ==> USED ONLY FOR SELF COMPARISON

*/

int AlignEngineSelf::__LookLeft(uint32_t i, uint32_t j, const QueryBuffer& query)
{
	uint8_t msk_length = __mask.GetLength();

	int score = 0;
    //int X = scoring.X1;
    int X = __parameters.GetUnGapDropoff();

	_bestscore = 0;

	if ( _GetStrand() > 0 ) 
	{
		bestSi_left = i;
		bestSj_left = j;
		while(score>_bestscore-X) 
		{
			if ( j <= _q_end + 1 )//  /Stop before the overlap
			{
				score = -10000;			
			}
			if (score >= _bestscore) 
			{
				_bestscore = score;
				bestSi_left = i+1;
				bestSj_left = j+1;
			}
			//score+=scoring.score_fwd_severe[query.Getc(i--)][ctree.Getc(j--)];
			score += scoring.GetScoreSevere(_GetStrand01(),query.Getc(i--),ctree.Getc(j--));
		}
		_q_start = bestSi_left;
		_s_start = bestSj_left;
	} 
	else 
	{
		i+=msk_length-1;
		bestSi_right = i;
		bestSj_left = j;
		while(score>_bestscore-X) 
		{
			if (score>=_bestscore) 
			{
				_bestscore = score;
				bestSi_right = i-1;
				bestSj_left = j+1;
			}
			//score+=scoring.score_bwd_severe[query.Getc(i++)][ctree.Getc(j--)];
			score += scoring.GetScoreSevere(_GetStrand01(),query.Getc(i++),ctree.Getc(j--));
		}
		_q_end = bestSi_right;
		_s_start = bestSj_left;
	}
	
	return _bestscore;
}

/**
 * \brief The dynamic programming allows to compute the best score. 
 *        Here the traceback is undertaken in order to recover the (one of the) alignemnt exhibiting the best score.
 * 
 * \param [in,out] hsp The hsp 
 * \return The new alignment size
 */

int AlignEngine::__RecordAlignment(Hsp& hsp)
{
	int i = bestSi;
	int j = bestSj;
				
	__UpdateAlignementStartEnd();		
				
	int previous_hsp_size = _a_size;
	
	int k = 0;				// Relative position of the gaps starting at the alignment end
	int move;               // The traceback atomic mouvement
	uint8_t (*t)[DYNPROG_MAT_SIZE+1];			// A pointer on the trace matrices
	
	GapPile q_gap;      // Local query and subject gap piles
	GapPile s_gap;
	
	t = Trace;
	move = t[i][j];
	
	while(move!=DONE) {
		switch(move) {
			case DIAG	: 	  /* Nothing to do */
							  i--; j--;  /* TODO should we check the mismatch here ? */
					 		  break;
			case ABOVE	:     s_gap.Push( k );
							  t = Trace_F;
				 			  break;
			case OPEN_S:      k--;
							  t = Trace;
							  i--;
				 			  break;
			case CONT_GS:     s_gap.Push( k );
							  i--;
				 			  break;
			case LEFT	:     q_gap.Push( k );
							  t = Trace_E;
				 			  break;
			case OPEN_Q:  	  k--;
							  t = Trace;
							  j--;
				 			  break;
			case CONT_GQ:     q_gap.Push( k );
							  j--;
				 			  break;
			default    : 	  fprintf(stderr,"It looks like there is a problem in here %d %d\n",i,j);
							  exit(1);
		}
		k++;
		move = t[i][j];
	}
	
	//k is now the size of the alignment
	int new_hsp_size = k;
	
	// We have now to correct the gap positions
	// Reporting the absolute positions of gaps
	if ( __GetDPMatrixQueryMove( ) == UPSTREAM  ) {
		
		// Previous reported gaps have to be incremented first by the size of the current hsp alignment
		hsp.s_gap.Increment( new_hsp_size );
		hsp.q_gap.Increment( new_hsp_size );
		
		for(uint16_t i=0;i<q_gap.GetSize();i++)
			hsp.q_gap.Push( q_gap.gaps[i] );
		for(uint16_t i=0;i<s_gap.GetSize();i++)
			hsp.s_gap.Push( s_gap.gaps[i] );		
	}
	else {
		// Newly reported gaps have to be adjusted according to the starting point of the alignment and its size
		
		int offset = previous_hsp_size + new_hsp_size + -1;
		
		for(uint16_t i=0;i<q_gap.GetSize();i++)
			hsp.q_gap.Push( offset - q_gap.gaps[i] );
		for(uint16_t i=0;i<s_gap.GetSize();i++)
			hsp.s_gap.Push( offset - s_gap.gaps[i] );
	}
	
	return (new_hsp_size + previous_hsp_size);
}


void AlignEngine::__UpdateAlignementStartEnd( )
{
	
	if ( __GetDPMatrixSubjectMove() == DOWNSTREAM )
	{
		_s_end = __GetDPMatrixSubjectStart() + bestSj - 1;	
	}
	else
	{
		_s_start = __GetDPMatrixSubjectStart() - bestSj + 1;		
	}
	
	if ( __GetDPMatrixQueryMove() ==  DOWNSTREAM )
	{
		_q_end = __GetDPMatrixQueryStart() + bestSi - 1;	
	}
	else
	{
		_q_start = __GetDPMatrixQueryStart() - bestSi + 1;		
	}
	
	
}



#ifdef DEBUGLOG
void AlignEngine::LogScore( const char* log_type)
{
	logger -> SetLogType(log_type) << " score      " << GetScore() << '\t' << bestSi_left << '\t' << bestSi_right << '\t' << bestSj_left << '\t' << bestSj_right << '\n';
}

void AlignEngine::LogAlign(const char* log_type, const char* header)
{
	logger -> SetLogType(log_type) << header << "\t" << GetQueryStart() << '\t' << GetQueryEnd() << '\t' << GetSubjectStart() << '\t' << GetSubjectEnd() << '\n';
	//logger -> SetLogType(log_type) << header << "\t" << align_direction.strand << "\t" << GetQueryStart() << '\t' << GetQueryEnd() << '\t' << GetSubjectStart() << '\t' << GetSubjectEnd() << '\n';
}

void AlignEngine::show_score_matrix(int l_inf, int l_max, int c_inf, int c_max, const char* log_type, const QueryBuffer& query)
{
	int i,j;
	char d;
	int query_move = __GetDPMatrixQueryMove();

	logger -> SetLogType(log_type)<<"   \t   \t";
	for (j=c_inf;j<=c_max;j++) {
		logger -> SetLogType(log_type)<<j<<"\t";
	}
	logger -> SetLogType(log_type)<<"\n";

	for (j=c_inf;j<=c_max;j++) {
		if (j==0) {
			logger -> SetLogType(log_type)<<"   \t - \t";
		} else if (j==c_inf) {
			logger -> SetLogType(log_type)<<"   \t   \t";
		}
		//fprintf(pf,"%3c\t",*(query.ctree.GetSeq()+align_direction.subject_start+align_direction.subject_move*(j-1)));
		logger -> SetLogType(log_type)<<*(ctree.GetSeq()+__GetDPMatrixSubjectStart()+__GetDPMatrixSubjectMove()*(j-1))<<"\t";
	}
	logger -> SetLogType(log_type)<<"\n";
	for (i=l_inf;i<=l_max;i++) {
		logger -> SetLogType(log_type)<<i<<"\t";
		d=(i==0) ? '-' : *(query.GetBase()+__GetDPMatrixQueryStart()+query_move*(i-1));
		logger -> SetLogType(log_type)<<d<<"\t";
		for (j=c_inf;j<=c_max;j++) {
			logger -> SetLogType(log_type)<<S[i][j]<<"\t";
		}
		logger -> SetLogType(log_type)<<"\n";
	}
}

#endif

