/*
   $Id: chain.h 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/

#ifndef __CHAIN_H__
#define __CHAIN_H__

#include <string>
#include <vector>

#include "anchor.h"
#include "ptypes.h"
#include "Parameters.h"

#define FACTICE_CUTOFFS "_FACTICE_CUTOFFS_"
#define INITIAL_LEVEL   0
#define INTERMEDIATE_LEVEL   1
#define LAST_LEVEL   2

// AnchorVectBase type is defined as being a vector of pointers on AnchorNode
// this type, especially AnchorVect an object deriving from it, will be used to
// manage all the hsps produced by glint
typedef std::vector<AnchorNode*> AnchorVectBase;

/*

    Classe AnchorVect,
	AnchorVect is a class for managing the vector of hsps implemented as a vector of AnchorNode pointers

*/

class AnchorVect : public AnchorVectBase
	{
public:
	AnchorVect(): _view_type(QUERY),level(0),MaxDist(MAX_DIST),MinScore(MIN_SCORE),MinCovar(MIN_COVAR) {};
private:
       // Not implemented; prevent use of copy c'tor and assignment.
	AnchorVect(const AnchorVect &);
	AnchorVect &operator=(const AnchorVect &);
public:
	//Methods to load data from file
	void LoadDataFromFile(const CompressedFileName&, MolType moltype=DNA);

	//Managing the different views
	void SetView(ViewType v);
	ViewType GetView() const { return _view_type; };

	//Managing the orders
	void SetAnchorsOrder();
	void OrderLabel();

	//The actual cluster (chain construction)

	uint32_t DetectChains();

	void InitGraphStructure();
	void InitGraphStructure_Self();
	void ConnectedComponents();
	void ConstructClustersInfo();
	void TrivialChaining();
	void ClusterChaining();

	uint32_t __ManhattanSegments(const Anchor& u, const Anchor& v);
	uint32_t __ManhattanPoints(Point& a, Point& b);

	uint32_t PartitionSize() { return partition.size(); };
	const vector<Cluster*>& GetPartition() { return partition; };
	AnchorVect& operator=(const vector<Cluster*>& part);

	void RemoveRepeats();
	void _RemoveRepeats();
	void SelectClusters();
	void SetClusteringParameters(uint32_t md, uint32_t mc, float mcv);
	void SetLevel(uint16_t l) { level = l; };

	void Rbh(uint32_t maxoverlap);
	void PartitionRbh();
	void __DetectAndRemoveSegmentOverlap(ViewType side, uint32_t maxoverlap);
	void __DetectAndRemoveOverlap(ViewType side);
	void PropagateSelection();
	void UpdateClusterProperties();

	//void CheckDiagonalClusters(); Onsolete

	//Printing the clusters and anchors
	void Print(const string& filename);
	void PrintClusters();
	void PrintPartition(const string& filename);

	//For debugging purpose
	void PrintDebug();

	friend ostream & operator<<(ostream& os,  AnchorVect& av);

private:
	std::map<AnchorNode*,Cluster*> clusters;
	vector<Cluster*> partition;
	ViewType _view_type;
	uint16_t level;
	uint32_t MaxDist;
	uint32_t MinScore;
	float MinCovar;
	};

class SyntenyLevel {
public:
	SyntenyLevel(string cutoff,uint8_t t, uint16_t l);
	void FirstClustering();
	void Clustering( );
	void LastClustering();
	void ChainingSegments();
	void Rbh(uint32_t maxoverlap);
	void GetSegmentsFromLowerLevel(SyntenyLevel* A );
	uint32_t GetSize() { return segments.size(); };
	uint16_t GetLevel() { return level; };
	string CutoffParameters();
	friend class Syntenies;
	friend ostream & operator<<(ostream& os,  SyntenyLevel& cl);
private:
	AnchorVect segments;
	uint32_t ManhattanCutoff;
	uint32_t ScoreCutoff;
	float CovarCutoff;
	uint16_t level;
	uint8_t type;
};

typedef vector<SyntenyLevel*>::iterator SyntenyLevelIterator;
typedef vector<SyntenyLevel*> SyntenyLevelVectBase;

class Syntenies: public SyntenyLevelVectBase
{
public:
	Syntenies(const string& chains, const CompressedFileName& glintoutput, const CompressedFileName& blastpoutput, const CompressedFileName& anchorsoutput, const string& basename, bool rbh,uint32_t maxoverlap);
	void InitialClustering();
	void LoadAlignments( );
	void ConstructSyntenyLevels();
	void Print();
	friend ostream & operator<<(ostream& os, Syntenies& s);
private:
	Parameters& __parameters;
	string BaseOutputFilename;
	const CompressedFileName GlintOutputFilename;
	const CompressedFileName BlastpOutputFilename;
	const CompressedFileName AnchorsOutputFilename;
	bool RBH;
	uint32_t MaxOverlap;
};

#endif


