#!/usr/bin/perl -I perl

# $Id: check.pl 302 2010-12-03 15:59:12Z manu $

use strict;
use warnings;
use Cwd;
use Error qw(:try);
use File::Basename;
use IO::File;
use File::Spec;
use ParamParser;

my $UNIX_RM="/bin/rm -f";

my $RH_PARAM;

MAIN:
{

ParamParser::SetDefaultUsage(\&Usage);
ParamParser::SetDefaultBehaviour('exit_on_getopt_error');

$RH_PARAM=New ParamParser('GETOPTLONG','no_compare','reference_dir=s','target_dir=s','index=s','data=s','index_opt=s','compare_opt=s','help');

$RH_PARAM->AssertDirExists('reference_dir');

my $no_compare_flg = ($RH_PARAM->IsDefined('no_compare')) ? $RH_PARAM->Get('no_compare') : 0;

my $ref_dir     = $RH_PARAM->Get('reference_dir');
$ref_dir = File::Spec->rel2abs($ref_dir);

my $target_dir  = ($RH_PARAM->IsDefined('target_dir')) ? $RH_PARAM->Get('target_dir') : "check";
$target_dir = File::Spec->rel2abs($target_dir);

my $index       = ($RH_PARAM->IsDefined('index')) ? $RH_PARAM->Get('index'):'bank.fasta';
my $data        = ($RH_PARAM->IsDefined('data')) ? $RH_PARAM->Get('data'):'request.fasta';


&launch_commands(ref_dir => $ref_dir, target_dir => $target_dir, bank => $index, query => $data);


unless ($no_compare_flg==1)
{
	my $error_code=&check_outputs(ref_dir => $ref_dir, target_dir => $target_dir, bank => $index, query => $data);

	if ($error_code) {
		exit(1);
	} else {	
	print STDERR "PASSED check successfully.........\n";	
	}
}
	
}

=pod

=head2 check_outputs

 Usage:     &check_outputs(ref_dir => $refdir,target_dir => $targetdir, bank => $bank , query => $query )
 Function:  try-catch compare_outputs
 Arg :      a hash table (samre as compare_outputs)
 Return:   the error_code (0 if everything was OK)

=cut
sub check_outputs {
	my %param=@_;
	my $error_code=0;
	
	try {
		&compare_outputs(%param);
	}
	catch Error::Simple with {
		my $error_mesg=shift;
		print STDERR "FAILED:......... $error_mesg \n";
		$error_code=1;
		return;
	}; 
			
	return 	$error_code;
}


=pod

=head2 launch_commands

 Usage:     &launch_commands(ref_dir => $refdir,target_dir => $targetdir, bank => $bank , query => $query )
 Function:  Launches in target_dir the indexing of $bank iwth danindex and the comparison between $bank and $query with compare (glint)
 Arg :      a hash table 
 Return:   none

=cut
sub launch_commands {
	my %param=@_;

	my $ref_dir=$param{ref_dir};
	my $target_dir=$param{target_dir};
	my $index=$param{bank};
	my $data=$param{query};
	my $compare_flg=$param{compare_flag};
		   
	if ($target_dir ne $ref_dir) {
		`$UNIX_RM -r $target_dir` if (-d $target_dir);
		mkdir($target_dir);
	} 

	my $sav_dir    = cwd;
	my $index_opt  = ($RH_PARAM->IsDefined('index_opt')) ? $RH_PARAM->Get('index_opt') : '-C 0';
	my $compare_opt= ($RH_PARAM->IsDefined('compare_opt')) ? $RH_PARAM->Get('compare_opt') : '';
	my $index_exe  = cwd . '/glintindex '.$index_opt;
	my $compare_exe= cwd . '/glint '.$compare_opt;

	chdir $target_dir;
	warn "Checking glintindex and glint executables\n";    

	symlink($ref_dir.'/'.$index,$index);
	symlink($ref_dir.'/'.$data,$data);

	my $prefix = &compute_prefix();

 	my $tim_file = "${prefix}.tim";
	my $time = "/usr/bin/time -f '%E %S %U %K %M %F' --output=$tim_file --append";

	my $out_file = "${prefix}.out";
	my $out =  new IO::File (">$out_file");
	print $out "TESTING glintindex and glint - standard output\n";

	print $out "INDEX    = $index\n";
	print $out "DATA     = $data\n";

	my $err_file = "${prefix}.err";
	my $err =  new IO::File (">$err_file");
	print $err "TESTING glintinedx and glint standard error\n";
	
	print $err "INDEX    = $index\n";
	print $err "DATA     = $data\n";

	print $out "\nSTARTING glintindex\n";
	print $out "=================\n";
	undef $out;

	print $err "\nSTARTING glintindex\n";
	print $err "=================\n";
	undef $err;

	system("$time $index_exe $index $data >>$out_file 2>>$err_file");

	$out = new IO::File(">>$out_file");
	print $out "\nSTARTING glint\n";
	print $out "=================\n";
	undef $out;

	$err = new IO::File(">>$err_file");
	print $err "\nSTARTING glint\n";
	print $err "=================\n";
	undef $err;

	system("$time $compare_exe -o compare_s.out -m 1 $index $data >>$out_file 2>>$err_file");
	#testing the pipe option
	system("cat $data | $time $compare_exe -o compare_p.out -m 2 $index - >>$out_file 2>>$err_file");
	#testing the auto-indexing mode
	system("$time $compare_exe $data $index -o compare_a.out -m 8 >>$out_file 2>>$err_file");

	system("cat compare_s.out compare_p.out compare_a.out > compare.out");

	chdir ($sav_dir);
}

=pod

=head2 compare_outputs

 Usage:     &compare_outputs(ref_dir => $refdir,target_dir => $targetdir, bank => $bank , query => $query )
 Function:  Compares the output generated by recently build executables with expected output
 Arg :      a hash table 
 Return:   none

=cut

sub compare_outputs {
	my %param=@_;

	my $ref_dir=$param{ref_dir};
	my $target_dir=$param{target_dir};
	my $index=$param{bank};
	my $data=$param{query};
	
	warn "Comparing the outputs with reference outputs in check_data/ref \n";	   
	
	# The reference files

	my $ref_prefix = &compute_prefix($ref_dir);
	my $ref_out_file = "$ref_dir/${ref_prefix}.out";
	my $ref_err_file = "$ref_dir/${ref_prefix}.err";
	my $ref_tim_file = "$ref_dir/${ref_prefix}.tim";
	my $ref_compare_file = "$ref_dir/compare.out";

	if (! (-e $ref_out_file) or !( -e $ref_err_file) or !( -e $ref_tim_file))
	{
    	die "Something wrong with the reference files ";
	};

	# The target files
	my $target_prefix = &compute_prefix($target_dir);
	my $target_out_file = "$target_dir/${target_prefix}.out";
	my $target_err_file = "$target_dir/${target_prefix}.err";
	my $target_tim_file = "$target_dir/${target_prefix}.tim";
	my $target_compare_file = "$target_dir/compare.out";

	if (!(-e $target_out_file) or !( -e $target_err_file) or !(-e $target_tim_file))
	{
    	die "Something wrong with the target files";
	};

	# Compare them: out and err should be nearly equal, time should be different

	my $diff_out_file = "$target_dir/diff_out_file.out";
	
	system("diff -I '\$Rev:' -I 'Command' $ref_out_file $target_out_file > $diff_out_file");

	if (-s $diff_out_file) {
		# This is not possible here because of the compiling time indicated in the output
		#throw Error::Simple( "WARNING : reference and target output file differ");
	}
	
	my $diff_err_file = "$target_dir/diff_err_file.out";

	system("diff -I '\$Rev:' -I 'Command' $ref_err_file $target_err_file > $diff_err_file");

	if (-s $diff_err_file) {
		throw Error::Simple( "WARNING : reference and target error file differ");
	}

	my $diff_compare_file = "$target_dir/diff_compare.out";

	system("diff -I '\$Rev:' -I 'Command' $ref_compare_file $target_compare_file > $diff_compare_file");

	if (-s $diff_compare_file) {
		throw Error::Simple("WARNING : reference and target compare output file differ ");
	} 
	
	my @ref_cpu_time;
	my $in= new IO::File ($ref_tim_file);
	while (my $line=<$in>) {
		my @fields=split(/\s+/,$line);
		push @ref_cpu_time, $fields[2];
	}
	undef $in;
	
	my @target_cpu_time;
	$in= new IO::File ($target_tim_file);
	while (my $line=<$in>) {
		my @fields=split(/\s+/,$line);
		push @target_cpu_time, $fields[2];
	}
	undef $in;
	
	my $ref_time_dnaindex=$ref_cpu_time[0];
	my $target_time_dnaindex=$target_cpu_time[0];	
	
	if ($target_time_dnaindex>2*$ref_time_dnaindex) {
		throw Error::Simple("WARNING : dnaindex time unexpectedly long ");
	}
	
	my $ref_time_compare=$ref_cpu_time[1];
	my $target_time_compare=$target_cpu_time[1];	
	
	if ($target_time_compare>2*$ref_time_compare) {
		throw Error::Simple("WARNING : compare time unexpectedly long ");
	}
		
}


=pod

=head2 compute_prefix

 Usage:     &compute_prefix($dir)
 Function:  Construit le prefixe : une chaine de caracteres apartir du nom de deux repertoires $dir (ou cwd) et le nom du pere
 Arg :      [optional] le nom du repertoire sinon repertoire courant
 Return:   La chaine de caracteres prefixe

=cut

sub compute_prefix
{
    my $dir = shift;
    $dir = cwd unless (defined $dir);
    my $prefix = basename(dirname($dir)).'_'.basename($dir);

    return $prefix;
}
    
sub Usage
{
    print "Generate reference files for further checking (from a committed release)\n";
    print "================================================\n";
    print "check.pl --reference_dir check_data/ref\n";
    print "        --target_dir    check_data/ref\n";
	print "        --no_compare  \n";
    
    print "Compare glintindex and glint outputs with reference outputs (from a modified directory)\n";
    print "=================================================\n";
    print "check.pl --reference_dir check_data/ref\n";
    print "        --target_dir    check_data/check\n";

}
