/*
   $Id: utils.cpp 649 2011-10-31 13:57:42Z manu $
*/
/*
 * Copyright INRA/CNRS - Thomas.Faraut@toulouse.inra.fr, Emmanuel.Courcelle@toulouse.inra.fr
 * This software is a computer program whose purpose is to align nucleic sequences, or perform other utility tasks
 * This software is governed by the CeCILL-C license under French law and abiding by the rules of distribution of free software.
 * 
 * Please read the file Licence_CeCILL-C_V1-en.txt which should have been distributed with this file
 * The fact that you are presently reading this, and the aforementioned file, means that you have had
 * knowledge of the CeCILL-C license and that you accept its terms.
*/
#include <iostream>
#include <iomanip>
#include <sstream>
#include <string>
#include <stdio.h>
#include <errno.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <limits.h>
#include <assert.h>
#include <time.h>
#include <list>
#include <vector>

using namespace std;

#include "glint.h"
#include "utils.h"
#include "Parameters.h"
#include "pstream.h"
#include "rtstack.h"
#include "../samtools/bam.h"

static string g_strListFileName;
static bool g_bListFileAppend = false;

/****************************************************************************************/
/*                    Some utility functions                                            */
/****************************************************************************************/

uint32_t String2uint32_t(const string& value)
{
	uint32_t temp;
	stringstream ss;
	ss<<value;
	ss>>temp;

	return temp;
}

uint16_t String2uint16_t(const string& value)
{
	uint16_t temp;
	stringstream ss;
	ss<<value;
	ss>>temp;

	return temp;
}

char String2Char(const string& value)
{
	char temp;
	stringstream ss;
	ss<<value;
	ss>>temp;

	return temp;
}

/**
 * \brief Open a file using fopen, building the file name:
 *        - Adding the extension to the file name
 *        - Adding an appropriate extension if compressed
 *        - If the mode is w (write) and the compression any, try to find the best compression algorithm
 * \brief Compression management: using the popen system call to start the compression program
 * \param filename The file name (ex: bank.fasta)
 * \param extension The file extension (ex: he, the file bank.fasta.he is opened) 
 * \param mode The mode (passed to fopen)
 * \param compression The compression algorithm
 * \return The opened filehandler, stdin or stdout if filename=="-" or ""
 * \exception Throw a ios_base::failure exception
 */

FILE *FOpen(const string& name, const string& extension, const char* mode, CompressionAlgorithm compression)
{
	FILE *pf;
	if ( name.length() == 0 || name == "-" )
	{
		return (mode[0]=='r') ? stdin : stdout;
	}
	else
	{
		string filename = name;
		string command  = "none";
		
		// add the extension to the file name
		if (extension != "")
		{
			filename += ".";
			filename += extension;
		}
		
		// No compression: just open the file
		if ( compression == NOCOMPRESSION )
		{
			pf=fopen(filename.c_str(),mode);
		}
		
		// compression wanted: add an appropriate extension (gz, bz2 etc.)
		else
		{
			const CompressionInfo &compression_infos = CompressionInfo::Instance();
			filename += ".";
			filename += compression_infos.GetExtension(compression);
			command   = compression_infos.GetProgram(compression);

			// try to open the file, just to know if this is possible
			pf=fopen(filename.c_str(),mode);
			if ( pf != NULL )   // OK - close the file and call popen !
			{
				fclose(pf);
				pf = NULL;

				// read mode: call the compression program with the -d switch
				if ( mode[0] == 'r' )
				{
					command  += " -d <";
					command  += filename;
					pf        = popen( command.c_str(), "r" );
				}
			
				// write mode: call the compression program without any switch
				else
				{
					command  += " >";
					command  += filename;
					pf        = popen( command.c_str(), "w" );
				}
			}
		}
		if (pf == NULL)
		{
			string msg = "ERROR - Cannot open file ";
			msg += filename;
			msg += " - mode = ";
			msg += mode;
			msg += " - compression program = ";
			msg += command;
#ifdef 	DEBUGLOG
			throw (rtstack(msg));
#else
			throw ios_base::failure(msg);
#endif
		}

	}
	return pf;
}
FILE* FOpen(const CompressedFileName& cfn, const char* mode) { return FOpen(cfn.GetName(),"",mode,cfn.GetCompression()); }

/**
 * \brief Open a file using an ifstring object, encapsulated through an auto_ptr
 *        If the filename is "-" or "", the ifstring is just cin
 *        If the compression algorithm is specified, use an ipstream to launch the compression program
 * \param filename The file name (ex: bank.fasta)
 * \param extension The file extension (ex: bi, the file bank.fasta.bi is opened)
 * \param compression the compression algorithm (defaults to NOCOMPRESSION)
 * \param mode The open mode (defauts to in)
 * \exception Throw an ios_base::failure exception
 */
IstreamPtr::IstreamPtr(const string& name, const string& extension, CompressionAlgorithm compression, ios_base::openmode mode)
{
	__Init(name, extension, compression, mode);
}
IstreamPtr::IstreamPtr(const CompressedFileName& name, ios_base::openmode mode)
{
	__Init(name.GetName(),"",name.GetCompression(),mode);
}
void IstreamPtr::__Init ( const string& name, const string& extension, CompressionAlgorithm compression, ios_base::openmode mode)
{
	if ( name.length() != 0 && name != "-" )
	{
		string filename = name;
		string command  = "none";

		if (extension != "")
		{
			filename += ".";
			filename += extension;
		}
		
		// No compression: we create an ifstream object
		if ( compression == NOCOMPRESSION )
		{
			ifstream_aptr.reset(new ifstream ( filename.c_str(), mode ));
			if ( ifstream_aptr -> fail() )
			{
		        string msg = (string) "ERROR opening file " + filename + " in read mode";
#ifdef 	DEBUGLOG
				throw (rtstack(msg));
#else
				throw ios_base::failure(msg);
#endif
			}
		}
		
		// Compression: creating an ipstream and starting the compression program
		else
		{
			const CompressionInfo &compression_infos = CompressionInfo::Instance();
			filename += ".";
			filename += compression_infos.GetExtension(compression);
			command   = compression_infos.GetProgram(compression);
			command  += " -d <";
			command  += filename;
			ipstream_aptr.reset(new redi::ipstream ( command, mode ));
			if ( ipstream_aptr -> fail() )
			{
		        string msg = (string) "ERROR opening file " + filename + " in read mode";
#ifdef 	DEBUGLOG
				throw (rtstack(msg));
#else
				throw ios_base::failure(msg);
#endif
			}
		}
	}
}

/**
 * \brief Open a file using an ofstring object, encapsulated through an auto_ptr
 *        If the filename is "-" or "", the ofstring is just cout
 *        If the compression algorithm is specified, use an opstream to launch the compression program
 * \param filename The file name (ex: bank.fasta)
 * \param extension The file extension (ex: bi, the file bank.fasta.bi is opened)
 * \param compression the compression algorithm (defaults to NOCOMPRESSION)
 * \param mode The open mode (defauts to out)
 * \exception Throw an ios_base::failure exception
 */
OstreamPtr::OstreamPtr ( const string& name, const string& extension, CompressionAlgorithm compression, ios_base::openmode mode)
{
	if ( name.length() != 0 && name != "-" )
	{
		string filename = name;
		if (extension != "")
		{
			filename += ".";
			filename += extension;
		}
		
		// No compression: we create an ifstream object
		if ( compression == NOCOMPRESSION )
		{
			ofstream_aptr.reset(new ofstream ( filename.c_str(), mode ));
			if ( ofstream_aptr -> fail() )
			{
		        string msg = (string) "ERROR opening file " + filename + " in write mode";
#ifdef 	DEBUGLOG
				throw (rtstack(msg));
#else
				throw ios_base::failure(msg);
#endif
			}
		}
		
		// Compression: creating an opstream and starting the compression program
		else
		{
			const CompressionInfo &compression_infos = CompressionInfo::Instance();
			filename += ".";
			filename += compression_infos.GetExtension(compression);
			string command   = compression_infos.GetProgram(compression);
			command  += " >";
			command  += filename;
			opstream_aptr.reset(new redi::opstream ( command, mode ));
			if ( opstream_aptr -> fail() )
			{
		        string msg = (string) "ERROR opening file " + filename + " in write mode";
#ifdef 	DEBUGLOG
				throw (rtstack(msg));
#else
				throw ios_base::failure(msg);
#endif
			}
		}
	}
}

/**
 * \brief call malloc, but throw a bad_alloc if malloc returns NULL
 *        If DEBUGLOG we throw an rtstack
 *        (was called memory_survey in the old times)
 */
void* MAlloc(size_t size)
{
	void* p = malloc(size);
	if ( p != NULL )
	{
		return p;
	}
	else
	{
#ifdef 	DEBUGLOG
				throw (rtstack("ERROR - Memory Allocation"));
#else
				throw bad_alloc();
#endif
	}
}
	
/**
 * \brief call realloc, but throw a bad_alloc if realloc returns NULL
 *        If DEBUGLOG we throw an rtstack
 */
void* ReAlloc(void* ptr, size_t size)
{
	void* p = realloc(ptr,size);
	if ( p != NULL )
	{
		return p;
	}
	else
	{
#ifdef 	DEBUGLOG
				throw (rtstack("ERROR - Memory Allocation"));
#else
				throw bad_alloc();
#endif
	}
}
	
void SetListFileName(const string& ListFileName, bool bAppend)
{
	g_strListFileName=ListFileName;
	g_bListFileAppend = bAppend;
}

void Log(const char szFormat[], ...)
{
	if (0 == g_strListFileName[0])
		return;

	static FILE *f = NULL;
	string mode;
	if (g_bListFileAppend)
		mode = "a";
	else
		mode = "w";
	if (NULL == f)
		f = fopen(g_strListFileName.c_str(), mode.c_str());
	if (NULL == f)
		{
		perror(g_strListFileName.c_str());
		exit(EXIT_NotStarted);
		}

	char szStr[4096];
	va_list ArgList;
	va_start(ArgList, szFormat);
	vsprintf(szStr, szFormat, ArgList);
	fprintf(f, "%s", szStr);
	fflush(f);
}

const char *GetTimeAsStr()
	{
	static char szStr[32];
	time_t t;
	time(&t);
	struct tm *ptmCurrentTime = localtime(&t);
	strcpy(szStr, asctime(ptmCurrentTime));
	assert('\n' == szStr[24]);
	szStr[24] = 0;
	return szStr;
	}

// Exit immediately with error message, printf-style.
void Quit(const char szFormat[], ...)
	{
	va_list ArgList;
	char szStr[4096];

	va_start(ArgList, szFormat);
	vsprintf(szStr, szFormat, ArgList);

	fprintf(stderr, "\n*** ERROR ***  %s\n", szStr);

	Log("\n*** FATAL ERROR ***  ");
	Log("%s\n", szStr);
	Log("Stopped %s\n", GetTimeAsStr());

	exit(EXIT_FatalError);
	}

void GetLineTokens(std::vector<std::string>& v, const string& s)
	{
		stringstream ss;
		string tmp;
		ss<<s;
		while ( getline( ss, tmp, '\t' )) v.push_back( tmp );
	}

/*
 Name		:  Validuint32_t
 Usage		:  if ( !Validuint32_t( integer_as_string ) ) { do something }
 Function	:  checks if the string is a valid uint32_t (unsigned int)
 Arg		:  value : a string that represents the integer
 Access		:  public function
 Return		:  false if the conversion from string to a positive integer fails or if the integer is negative
*/

bool Validuint32_t(const string& s,  bool failIfLeftoverChars)
{
	long l;
	istringstream iss(s);
	char c;
	if ( (iss >> l).fail() ||  ( failIfLeftoverChars && iss.get(c) ))
		return false;
	if (l<0)  // checks now if the integer is positive
		return false;
	return true;
}

/*
 Name		:  ValidCovar
 Usage		:  if ( !ValidCovar( covar_as_string ) ) { do something }
 Function	:  checks if the string is a valid covariance threshold ( a float bewteen 0 and 1)
 Arg		:  value : a string that represents the covariance threshold
 Access		:  public function
 Return		:  false if the conversion from string to a float fails of if the float is larger than 1
*/

bool ValidCovar(const string& s,  bool failIfLeftoverChars)
{
	float f;
	istringstream iss(s);
	char c;
	if ( (iss >> f).fail() || ( failIfLeftoverChars && iss.get(c) ) )
		return false;
	if ( f > 1 ) // checks now if the float is between 0 and 1
		return false;

	return true;
}

/*
void bin_print(long n, int taille)
	{
	unsigned long mask= 1 << ((taille * CHAR_BIT) - 1);

	printf("%ld: ", n);
	while (mask > 0)
    {
      putchar(n & mask ? '1' : '0');
      mask>>= 1;
    }
	putchar('\n');
	fflush(stdout);
	}

void bin_print_char(char n)
	{
	bin_print(n, sizeof(n));
	}

void bin_print_short(short n)
	{
	bin_print(n, sizeof(n));
	}

void bin_print_int(int n)
	{
	bin_print(n, sizeof(n));
	}

void bin_print_unlong(unsigned long n)
	{
	bin_print(n, sizeof(n));
	}
*/

/**
 * \brief Get the sequence name from an opened fasta/fastq file
 * \pre f is an already opened fasta/fastq file
 * \pre f is positioned to the First ('>' or '@') character of a header line
 * \param f The file descriptor
 * \return The sequence name (first word after '>') or "" if nothing read (end of file, blank line, etc)
 * |post f is positioned to the FIRST character of the sequence (start of next line)
 * \exception bad_alloc, runtime_error
 * \todo Faire un objet FastaFile ?
 */

string GetSeqName(FILE *fp)
{
    static char* charbuffer = NULL;
    if (!charbuffer)
    {
        charbuffer = (char*) malloc(MAX_FASTA_HEADER_SIZE+1);   // sizeof(char=1). We alloc one byte for the \0
        if (charbuffer==NULL)
        {
			throw (bad_alloc());
        };
    };

	char* rvl=fgets(charbuffer,MAX_FASTA_HEADER_SIZE+1,fp);
    if (rvl==NULL)
        return (string) "";     // Nothing read (may be we are at end of file)
    if (strlen(rvl)==MAX_FASTA_HEADER_SIZE)
    {
		string msg = "ERROR - Fasta header line too long - Please change MAX_FASTA_HEADER_SIZE and recompile";
#ifdef 	DEBUGLOG
		throw (rtstack(msg));
#else
		throw ios_base::failure(msg);
#endif
    };

    string bfr = charbuffer;
    size_t strt  = bfr.find_first_not_of(">@ "); // Strip leading '>', '@' and spaces
    if (strt==string::npos)
    {
		string msg = "ERROR - Bad file format";
#ifdef 	DEBUGLOG
		throw (rtstack(msg));
#else
		throw ios_base::failure(msg);
#endif
	}
    size_t end = bfr.find_first_of(" \t\n",strt); // Keep all characters up to the first space or \n, if any
    return bfr.substr(strt,end-strt);
}

/**
 * \brief Getting the actual read name : getting rid of the extension /1 or /2
 *        When fwd and bwd reads are in two different files the convention is to mark the fwd read by /1 extension (resp bwd by /2)
 *        
 * \return the read name
 */
string GetReadName( const string& fragmentId )
{
	size_t len = fragmentId.size();
	if ( len > 2 )
	{
		char near_last = fragmentId[len-2];
		char last      = fragmentId[len-1];
		if ( near_last=='/' && (last=='1' || last=='2' ) )
		{
			return fragmentId.substr(0,len-2);
		}
		else
		{
			return fragmentId;
		}
	}
	else
	{
		return fragmentId;
	}
}

/**
 * \brief Printing to cerr some information about glint (version number a.s.o.)
 * 
   Usage:    PrintHeader();
   Procedure: Print to stderr some things related to the program and to the parameters
   Parameter:    None
   Return value: The sequence name (first word after '>') or "" if nothing read (end of file, blank line, etc)
   Error:        Call SysDie
   TODO:         Faire un objet FastaFile ?

*/
void PrintHeader()
{
	cerr <<"glint - release " << (string) GLINT_VERSION << " - Compiled on " << __DATE__ << " at " << __TIME__;
	#ifdef __INTEL_COMPILER
	    cerr << " using the intel compiler " << __INTEL_COMPILER_BUILD_DATE << '\n';
	#elif __GNUG__
	    cerr << " using the gcc compiler " << __VERSION__ << '\n';
	#endif
	cerr << "        by T. Faraut & E. Courcelle (INRA-CNRS, see http://???.fr\n";
	cerr << "        samtools " << BAM_VERSION  << " by Heng Li and others, see http://samtools.sourceforge.net\n";
	cerr << "        Pstreams by Jonathan Wakely, see http://pstreams.sourceforge.net\n";
	cerr << "        SimpleOpt 3.0 by Brodie Thiesfield, see http://code.jellycan.com/simpleopt\n\n";
	cerr << "        Bit Counting routines by Gurmeet Singh Manku (manku@cs.stanford.edu)\n";
}

		

	
